"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnalysisStyle = exports.AnalysisStyleScalar = exports.AnalysisStyleDisplacement = void 0;
const geometry_core_1 = require("@bentley/geometry-core");
const ThematicDisplay_1 = require("./ThematicDisplay");
const Gradient_1 = require("./Gradient");
/** Describes how an [[AnalysisStyle]] deforms a [Polyface]($geometry-core) by applying translation to its vertices.
 * @see [[AnalysisStyle.displacement]].
 * @public
 */
class AnalysisStyleDisplacement {
    /** @internal */
    constructor(channelName, scale = 1) {
        this.channelName = channelName;
        this.scale = scale;
    }
    /** Create from JSON representation. */
    static fromJSON(props) {
        return new this(props.channelName, props.scale);
    }
    /** Convert to JSON representation. */
    toJSON() {
        const props = { channelName: this.channelName };
        if (this.scale !== 1)
            props.scale = this.scale;
        return props;
    }
    /** Return true if `this` is equivalent to `other`. */
    equals(other) {
        return this.channelName === other.channelName && this.scale === other.scale;
    }
}
exports.AnalysisStyleDisplacement = AnalysisStyleDisplacement;
/** Describes how an [[AnalysisStyle]] recolors [Polyface]($geometry-core) vertices by mapping scalar values supplied
 * by an [AuxChannel]($geometry-core) to colors supplied by a [[Gradient]] image.
 * @see [[AnalysisStyle.scalar]].
 * @public
 */
class AnalysisStyleScalar {
    /** @internal */
    constructor(props) {
        this.channelName = props.channelName;
        this.range = geometry_core_1.Range1d.fromJSON(props.range);
        this.thematicSettings = ThematicDisplay_1.ThematicGradientSettings.fromJSON(props.thematicSettings);
    }
    /** Create from JSON representation. */
    static fromJSON(props) {
        return new this(props);
    }
    /** Convert to JSON representation. */
    toJSON() {
        const props = {
            channelName: this.channelName,
            range: this.range.toJSON(),
        };
        if (!this.thematicSettings.equals(ThematicDisplay_1.ThematicGradientSettings.defaults))
            props.thematicSettings = this.thematicSettings.toJSON();
        return props;
    }
    /** The gradient computed from [[thematicSettings]]. */
    get gradient() {
        if (!this._gradient)
            this._gradient = Gradient_1.Gradient.Symb.createThematic(this.thematicSettings);
        return this._gradient;
    }
    /** Return true if `this` is equivalent to `other`. */
    equals(other) {
        return this.channelName === other.channelName && this.range.isAlmostEqual(other.range) && this.thematicSettings.equals(other.thematicSettings);
    }
}
exports.AnalysisStyleScalar = AnalysisStyleScalar;
function tryConvertLegacyProps(input) {
    if (input.displacement || input.scalar)
        return input;
    const legacy = input;
    if (undefined === legacy.displacementChannelName && undefined === legacy.scalarChannelName)
        return input;
    const output = {
        normalChannelName: input.normalChannelName,
    };
    if (undefined !== legacy.displacementChannelName) {
        output.displacement = {
            channelName: legacy.displacementChannelName,
            scale: legacy.displacementScale,
        };
    }
    if (undefined !== legacy.scalarChannelName && undefined !== legacy.scalarRange) {
        output.scalar = {
            channelName: legacy.scalarChannelName,
            range: legacy.scalarRange,
            thematicSettings: legacy.scalarThematicSettings,
        };
    }
    return output;
}
/** As part of a [[DisplayStyleSettings]], describes how to animate meshes in the view that have been augmented with
 * [PolyfaceAuxData]($geometry-core). The style specifies which channels to use, and can deform the meshes by
 * translating vertices and/or recolor vertices using [[ThematicDisplay]].
 * @see [[DisplayStyleSettings.analysisStyle]] to define the analysis style for a [DisplayStyle]($backend).
 * @see [[DisplayStyleSettings.analysisFraction]] to control playback of the animation.
 * @public
 */
class AnalysisStyle {
    /** @internal */
    constructor(props) {
        this.normalChannelName = props.normalChannelName;
        if (props.displacement)
            this.displacement = AnalysisStyleDisplacement.fromJSON(props.displacement);
        if (props.scalar)
            this.scalar = AnalysisStyleScalar.fromJSON(props.scalar);
    }
    /** Create an analysis style from its JSON representation.
     * @note AnalysisStyle is an immutable type - use [[clone]] to produce a modified copy.
     */
    static fromJSON(props) {
        if (!props)
            return this.defaults;
        props = tryConvertLegacyProps(props);
        if (!props.displacement && !props.scalar && undefined === props.normalChannelName)
            return this.defaults;
        return new AnalysisStyle(props);
    }
    /** Convert this style to its JSON representation. */
    toJSON() {
        const props = {};
        if (this === AnalysisStyle.defaults)
            return props;
        if (this.displacement)
            props.displacement = this.displacement.toJSON();
        if (this.scalar)
            props.scalar = this.scalar.toJSON();
        if (undefined !== this.normalChannelName)
            props.normalChannelName = this.normalChannelName;
        return props;
    }
    /** Produce a copy of this style identical except for properties explicitly specified by `changedProps`. */
    clone(changedProps) {
        return AnalysisStyle.fromJSON(Object.assign(Object.assign({}, this.toJSON()), changedProps));
    }
    /** Return true if this style is equivalent to `other`. */
    equals(other) {
        if (this.normalChannelName !== other.normalChannelName)
            return false;
        if ((undefined === this.displacement) !== (undefined === other.displacement))
            return false;
        else if (this.displacement && !this.displacement.equals(other.displacement))
            return false;
        if ((undefined === this.scalar) !== (undefined === other.scalar))
            return false;
        return undefined === this.scalar || this.scalar.equals(other.scalar);
    }
}
exports.AnalysisStyle = AnalysisStyle;
AnalysisStyle.defaults = new AnalysisStyle({});
//# sourceMappingURL=AnalysisStyle.js.map