"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.BackgroundMapSettings = exports.GlobeMode = exports.BackgroundMapType = void 0;
const PlanarClipMask_1 = require("./PlanarClipMask");
const TerrainSettings_1 = require("./TerrainSettings");
/** Describes the type of background map displayed by a [[DisplayStyle]]
 * @see [[BackgroundMapProps]]
 * @see [[DisplayStyleSettingsProps]]
 * @public
 */
var BackgroundMapType;
(function (BackgroundMapType) {
    BackgroundMapType[BackgroundMapType["Street"] = 1] = "Street";
    BackgroundMapType[BackgroundMapType["Aerial"] = 2] = "Aerial";
    BackgroundMapType[BackgroundMapType["Hybrid"] = 3] = "Hybrid";
})(BackgroundMapType = exports.BackgroundMapType || (exports.BackgroundMapType = {}));
/** Describes the projection of the background map
 * @see [[BackgroundMapProps]]
 * @see [[DisplayStyleSettingsProps]]
 * @public
 */
var GlobeMode;
(function (GlobeMode) {
    /** Display Earth as 3d ellipsoid */
    GlobeMode[GlobeMode["Ellipsoid"] = 0] = "Ellipsoid";
    /** Display Earth as plane. */
    GlobeMode[GlobeMode["Plane"] = 1] = "Plane";
})(GlobeMode = exports.GlobeMode || (exports.GlobeMode = {}));
function normalizeMapType(props) {
    var _a;
    switch ((_a = props.providerData) === null || _a === void 0 ? void 0 : _a.mapType) {
        case BackgroundMapType.Street:
        case BackgroundMapType.Aerial:
            return props.providerData.mapType;
        default:
            return BackgroundMapType.Hybrid;
    }
}
function normalizeGlobeMode(mode) {
    return GlobeMode.Plane === mode ? mode : GlobeMode.Ellipsoid;
}
function normalizeTransparency(trans) {
    if ("number" === typeof trans)
        return Math.min(1, Math.max(0, trans));
    return false;
}
function normalizeProviderName(provider) {
    return "MapBoxProvider" === provider ? provider : "BingProvider";
}
/** Normalized representation of a [[BackgroundMapProps]] for which type and provider have been validated and default values have been applied where explicit values not defined.
 * @public
 */
class BackgroundMapSettings {
    constructor(props) {
        var _a, _b, _c;
        this.groundBias = (_a = props.groundBias) !== null && _a !== void 0 ? _a : 0;
        this.providerName = normalizeProviderName(props.providerName);
        this.mapType = normalizeMapType(props);
        this.transparency = normalizeTransparency(props.transparency);
        this.useDepthBuffer = (_b = props.useDepthBuffer) !== null && _b !== void 0 ? _b : false;
        this.applyTerrain = (_c = props.applyTerrain) !== null && _c !== void 0 ? _c : false;
        this.terrainSettings = TerrainSettings_1.TerrainSettings.fromJSON(props.terrainSettings);
        this.globeMode = normalizeGlobeMode(props.globeMode);
        this._locatable = true !== props.nonLocatable;
        this.planarClipMask = PlanarClipMask_1.PlanarClipMaskSettings.fromJSON(props.planarClipMask);
    }
    /** If false, the map will be treated as non-locatable - i.e., tools will not interact with it. This is particularly useful when the map is transparent - it
     * allows the user to select elements that are behind the map.
     */
    get locatable() {
        // Handle deprecated TerrainProps.nonLocatable:
        // - If TerrainProps.nonLocatable=true and terrain is on, terrain is not locatable.
        // - Otherwise, use BackgroundMapProps.nonLocatable.
        if (this.applyTerrain && !this.terrainSettings.locatable) // eslint-disable-line deprecation/deprecation
            return false;
        return this._locatable;
    }
    /** If transparency is overridden, the transparency to apply; otherwise, undefined. */
    get transparencyOverride() { return false !== this.transparency ? this.transparency : undefined; }
    /** Construct from JSON, performing validation and applying default values for undefined fields. */
    static fromJSON(json) {
        return new BackgroundMapSettings(json !== null && json !== void 0 ? json : {});
    }
    toJSON() {
        const props = {};
        if (0 !== this.groundBias)
            props.groundBias = this.groundBias;
        if ("BingProvider" !== this.providerName)
            props.providerName = this.providerName;
        if (this.applyTerrain)
            props.applyTerrain = true;
        if (BackgroundMapType.Hybrid !== this.mapType)
            props.providerData = { mapType: this.mapType };
        if (false !== this.transparency)
            props.transparency = this.transparency;
        if (GlobeMode.Ellipsoid !== this.globeMode)
            props.globeMode = this.globeMode;
        if (this.useDepthBuffer)
            props.useDepthBuffer = true;
        if (!this._locatable)
            props.nonLocatable = true;
        const terrainSettings = this.terrainSettings.toJSON();
        for (const prop of Object.values(terrainSettings)) {
            if (undefined !== prop) {
                props.terrainSettings = terrainSettings;
                break;
            }
        }
        if (this.planarClipMask.isValid)
            props.planarClipMask = this.planarClipMask.toJSON();
        return props;
    }
    /** Returns true if these settings are equivalent to the supplied JSON settings. */
    equalsJSON(json) {
        return this.equals(BackgroundMapSettings.fromJSON(json));
    }
    equals(other) {
        return this.groundBias === other.groundBias && this.providerName === other.providerName && this.mapType === other.mapType
            && this.useDepthBuffer === other.useDepthBuffer && this.transparency === other.transparency && this.globeMode === other.globeMode
            && this._locatable === other._locatable && this.applyTerrain === other.applyTerrain && this.terrainSettings.equals(other.terrainSettings) && this.planarClipMask.equals(other.planarClipMask);
    }
    /** Create a copy of this BackgroundMapSettings, optionally modifying some of its properties.
     * @param changedProps JSON representation of the properties to change.
     * @returns A BackgroundMapSettings with all of its properties set to match those of `this`, except those explicitly defined in `changedProps`.
     * @note If changing the provider it is currently necessary to also make same change to update the imagery base layer.
     */
    clone(changedProps) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        if (undefined === changedProps)
            return this;
        const props = {
            providerName: (_a = changedProps.providerName) !== null && _a !== void 0 ? _a : this.providerName,
            groundBias: (_b = changedProps.groundBias) !== null && _b !== void 0 ? _b : this.groundBias,
            transparency: (_c = changedProps.transparency) !== null && _c !== void 0 ? _c : this.transparency,
            useDepthBuffer: (_d = changedProps.useDepthBuffer) !== null && _d !== void 0 ? _d : this.useDepthBuffer,
            globeMode: (_e = changedProps.globeMode) !== null && _e !== void 0 ? _e : this.globeMode,
            nonLocatable: (_f = changedProps.nonLocatable) !== null && _f !== void 0 ? _f : !this._locatable,
            applyTerrain: (_g = changedProps.applyTerrain) !== null && _g !== void 0 ? _g : this.applyTerrain,
            terrainSettings: changedProps.terrainSettings ? this.terrainSettings.clone(changedProps.terrainSettings).toJSON() : this.terrainSettings.toJSON(),
            providerData: {
                mapType: (_j = (_h = changedProps.providerData) === null || _h === void 0 ? void 0 : _h.mapType) !== null && _j !== void 0 ? _j : this.mapType,
            },
            planarClipMask: changedProps.planarClipMask ? this.planarClipMask.clone(changedProps.planarClipMask).toJSON() : this.planarClipMask.toJSON(),
        };
        return BackgroundMapSettings.fromJSON(props);
    }
    /** @internal */
    static providerFromMapLayer(props) {
        let providerName, mapType;
        if (!props.url)
            return undefined;
        if (props.formatId === "BingMaps") {
            providerName = "BingProvider";
            if (props.url.indexOf("Road") > 0)
                mapType = BackgroundMapType.Street;
            else if (props.url.indexOf("AerialWithLabels") > 0)
                mapType = BackgroundMapType.Hybrid;
            else if (props.url.indexOf("Aerial") > 0)
                mapType = BackgroundMapType.Aerial;
        }
        else if (props.formatId === "MapboxImagery") {
            providerName = "MapBoxProvider";
            if (props.url.indexOf("streets-satellite") > 0)
                mapType = BackgroundMapType.Hybrid;
            else if (props.url.indexOf("streets") > 0)
                mapType = BackgroundMapType.Street;
            else if (props.url.indexOf("satellite") > 0)
                mapType = BackgroundMapType.Aerial;
        }
        else
            return undefined;
        return mapType !== undefined && providerName !== undefined ? { providerName, providerData: { mapType } } : undefined;
    }
}
exports.BackgroundMapSettings = BackgroundMapSettings;
//# sourceMappingURL=BackgroundMapSettings.js.map