/** @packageDocumentation
 * @module Views
 */
import { ViewFlagOverrides, ViewFlagOverridesProps } from "./ViewFlags";
import { RgbColor, RgbColorProps } from "./RgbColor";
import { HiddenLine } from "./HiddenLine";
import { FeatureAppearance, FeatureAppearanceProps } from "./FeatureSymbology";
/** Wire format describing a [[CutStyle]] applied to section-cut geometry produced at intersections with a view's [ClipVector]($geometry-core).
 * @see [[ClipStyleProps.cutStyle]].
 * @public
 */
export interface CutStyleProps {
    /** If defined, overrides aspects of the view's [[ViewFlags]] when drawing the cut geometry. */
    viewflags?: ViewFlagOverridesProps;
    /** If defined, overrides the view's [[HiddenLine.Settings]] when drawing the cut geometry. */
    hiddenLine?: HiddenLine.SettingsProps;
    /** If defined, overrides aspects of the cut geometry's symbology. */
    appearance?: FeatureAppearanceProps;
}
/** As part of a [[ClipStyle]], describes how section-cut graphics should be displayed.
 * @note Section-cut graphics are only produced if [[ClipStyle.produceCutGeometry]] is `true`.
 * @public
 */
export declare class CutStyle {
    /** Selectively overrides some of the view's [[ViewFlags]] when drawing the section-cut graphics. */
    readonly viewflags: Readonly<ViewFlagOverrides>;
    /** If defined, overrides the settings the view uses to draw the edges of the section-cut graphics. */
    readonly hiddenLine?: HiddenLine.Settings;
    /** If defined, overrides aspects of the symbology of the section-cut graphics. */
    readonly appearance?: FeatureAppearance;
    /** The default CutStyle, configured to draw the section-cut graphics using the view's settings, with no overrides. */
    static readonly defaults: CutStyle;
    private constructor();
    /** Create a CutStyle from its components. */
    static create(viewflags?: Readonly<ViewFlagOverrides>, hiddenLine?: HiddenLine.Settings, appearance?: FeatureAppearance): CutStyle;
    static fromJSON(props?: CutStyleProps): CutStyle;
    /** Return JSON representation. The representation is `undefined` if this style matches the default style. */
    toJSON(): CutStyleProps | undefined;
    /** Returns true if this style matches the default style - that is, it overrides none of the view's settings. */
    get matchesDefaults(): boolean;
}
/** Wire format describing a [[ClipStyle]].
 * @see [[DisplayStyleSettingsProps.clipStyle]].
 * @public
 */
export interface ClipStyleProps {
    /** If `true`, geometry will be produced at the clip planes in a 3d view.
     * - Solids (closed volumes) will produce facets on the clip planes.
     * - Other surfaces will produce line strings representing the edges of the surface at the clip planes.
     * @note Cut geometry will only be produced for element geometry - not for, e.g., terrain or reality models.
     */
    produceCutGeometry?: boolean;
    /** Controls aspects of how the cut geometry is displayed, if [[produceCutGeometry]] is `true`. */
    cutStyle?: CutStyleProps;
    /** If defined, geometry inside the clip planes will be drawn in this color. */
    insideColor?: RgbColorProps;
    /** If defined, geometry outside of the clip planes will be drawn in this color instead of being clipped. */
    outsideColor?: RgbColorProps;
}
/** Describes symbology and behavior applied to a [ClipVector]($geometry-core) when applied to a [ViewState]($frontend) or [[ModelClipGroup]].
 * @see [[DisplayStyleSettings.clipStyle]].
 * @public
 */
export declare class ClipStyle {
    /** If `true`, geometry will be produced at the clip planes.
     * - Solids (closed volumes) will produce facets on the clip planes.
     * - Other surfaces will produce line strings representing the edges of the surface at the clip planes.
     * @note Cut geometry will only be produced for element geometry - not for, e.g., terrain or reality models.
     */
    readonly produceCutGeometry: boolean;
    /** Controls aspects of how the cut geometry is displayed, if [[produceCutGeometry]] is `true`. */
    readonly cutStyle: CutStyle;
    /** If defined, geometry inside the clip planes will be drawn in this color. */
    readonly insideColor?: RgbColor;
    /** If defined, geometry outside of the clip planes will be drawn in this color instead of being clipped. */
    readonly outsideColor?: RgbColor;
    /** The default style, which overrides none of the view's settings. */
    static readonly defaults: ClipStyle;
    private constructor();
    /** Create a style from its components. */
    static create(produceCutGeometry: boolean, cutStyle: CutStyle, insideColor?: RgbColor, outsideColor?: RgbColor): ClipStyle;
    static fromJSON(props?: ClipStyleProps): ClipStyle;
    /** The JSON representation of this style. It is `undefined` if this style matches the defaults. */
    toJSON(): ClipStyleProps | undefined;
    /** Returns true if this style matches the [[ClipStyle.defaults]] - that is, it overrides no settings from the view. */
    get matchesDefaults(): boolean;
}
//# sourceMappingURL=ClipStyle.d.ts.map