"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Views
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClipStyle = exports.CutStyle = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ViewFlags_1 = require("./ViewFlags");
const RgbColor_1 = require("./RgbColor");
const HiddenLine_1 = require("./HiddenLine");
const FeatureSymbology_1 = require("./FeatureSymbology");
/** As part of a [[ClipStyle]], describes how section-cut graphics should be displayed.
 * @note Section-cut graphics are only produced if [[ClipStyle.produceCutGeometry]] is `true`.
 * @public
 */
class CutStyle {
    constructor(viewflags, hiddenLine, appearance) {
        this.viewflags = viewflags !== null && viewflags !== void 0 ? viewflags : ViewFlags_1.ViewFlagOverrides.fromJSON();
        if (hiddenLine && !hiddenLine.matchesDefaults)
            this.hiddenLine = hiddenLine;
        if (appearance && !appearance.matchesDefaults)
            this.appearance = appearance;
    }
    /** Create a CutStyle from its components. */
    static create(viewflags, hiddenLine, appearance) {
        if ((viewflags && viewflags.anyOverridden()) || (hiddenLine && !hiddenLine.matchesDefaults) || (appearance && !appearance.matchesDefaults))
            return new CutStyle(viewflags, hiddenLine, appearance);
        return this.defaults;
    }
    static fromJSON(props) {
        if (bentleyjs_core_1.JsonUtils.isNonEmptyObject(props)) {
            const viewflags = ViewFlags_1.ViewFlagOverrides.fromJSON(props === null || props === void 0 ? void 0 : props.viewflags);
            const hiddenLine = (props === null || props === void 0 ? void 0 : props.hiddenLine) ? HiddenLine_1.HiddenLine.Settings.fromJSON(props.hiddenLine) : undefined;
            const appearance = (props === null || props === void 0 ? void 0 : props.appearance) ? FeatureSymbology_1.FeatureAppearance.fromJSON(props.appearance) : undefined;
            return this.create(viewflags, hiddenLine, appearance);
        }
        else {
            return this.defaults;
        }
    }
    /** Return JSON representation. The representation is `undefined` if this style matches the default style. */
    toJSON() {
        var _a;
        if (this.matchesDefaults)
            return undefined;
        const props = {};
        if (this.viewflags.anyOverridden())
            props.viewflags = this.viewflags.toJSON();
        if (this.hiddenLine && !this.hiddenLine.matchesDefaults)
            props.hiddenLine = (_a = this.hiddenLine) === null || _a === void 0 ? void 0 : _a.toJSON();
        if (this.appearance && !this.appearance.matchesDefaults)
            props.appearance = this.appearance.toJSON();
        return props;
    }
    /** Returns true if this style matches the default style - that is, it overrides none of the view's settings. */
    get matchesDefaults() {
        if (this === CutStyle.defaults)
            return true;
        return !this.viewflags.anyOverridden() && (!this.hiddenLine || this.hiddenLine.matchesDefaults) && (!this.appearance || this.appearance.matchesDefaults);
    }
}
exports.CutStyle = CutStyle;
/** The default CutStyle, configured to draw the section-cut graphics using the view's settings, with no overrides. */
CutStyle.defaults = new CutStyle();
/** Describes symbology and behavior applied to a [ClipVector]($geometry-core) when applied to a [ViewState]($frontend) or [[ModelClipGroup]].
 * @see [[DisplayStyleSettings.clipStyle]].
 * @public
 */
class ClipStyle {
    constructor(produceCutGeometry, cutStyle, inside, outside) {
        this.produceCutGeometry = produceCutGeometry;
        this.cutStyle = cutStyle;
        this.insideColor = inside;
        this.outsideColor = outside;
    }
    /** Create a style from its components. */
    static create(produceCutGeometry, cutStyle, insideColor, outsideColor) {
        if (!produceCutGeometry && cutStyle.matchesDefaults && !insideColor && !outsideColor)
            return this.defaults;
        return new ClipStyle(produceCutGeometry, cutStyle, insideColor, outsideColor);
    }
    static fromJSON(props) {
        var _a;
        if (bentleyjs_core_1.JsonUtils.isNonEmptyObject(props)) {
            const produceCutGeometry = (_a = props.produceCutGeometry) !== null && _a !== void 0 ? _a : false;
            const cutStyle = CutStyle.fromJSON(props.cutStyle);
            const inside = props.insideColor ? RgbColor_1.RgbColor.fromJSON(props.insideColor) : undefined;
            const outside = props.outsideColor ? RgbColor_1.RgbColor.fromJSON(props.outsideColor) : undefined;
            return this.create(produceCutGeometry, cutStyle, inside, outside);
        }
        return this.defaults;
    }
    /** The JSON representation of this style. It is `undefined` if this style matches the defaults. */
    toJSON() {
        if (this.matchesDefaults)
            return undefined;
        const props = {};
        if (this.produceCutGeometry)
            props.produceCutGeometry = true;
        const cutStyle = this.cutStyle.toJSON();
        if (cutStyle) {
            bentleyjs_core_1.assert(!this.cutStyle.matchesDefaults);
            props.cutStyle = cutStyle;
        }
        if (this.insideColor)
            props.insideColor = this.insideColor.toJSON();
        if (this.outsideColor)
            props.outsideColor = this.outsideColor.toJSON();
        return props;
    }
    /** Returns true if this style matches the [[ClipStyle.defaults]] - that is, it overrides no settings from the view. */
    get matchesDefaults() {
        if (this === ClipStyle.defaults)
            return true;
        return !this.produceCutGeometry && !this.insideColor && !this.outsideColor && this.cutStyle.matchesDefaults;
    }
}
exports.ClipStyle = ClipStyle;
/** The default style, which overrides none of the view's settings. */
ClipStyle.defaults = new ClipStyle(false, CutStyle.defaults, undefined, undefined);
//# sourceMappingURL=ClipStyle.js.map