"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module CloudStorage
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudStorageCache = exports.CloudStorageContainerUrl = exports.CloudStorageProvider = void 0;
/** @beta */
var CloudStorageProvider;
(function (CloudStorageProvider) {
    CloudStorageProvider[CloudStorageProvider["Azure"] = 0] = "Azure";
    CloudStorageProvider[CloudStorageProvider["Amazon"] = 1] = "Amazon";
    CloudStorageProvider[CloudStorageProvider["AliCloud"] = 2] = "AliCloud";
    CloudStorageProvider[CloudStorageProvider["External"] = 3] = "External";
    CloudStorageProvider[CloudStorageProvider["Unknown"] = 4] = "Unknown";
})(CloudStorageProvider = exports.CloudStorageProvider || (exports.CloudStorageProvider = {}));
/** @beta */
var CloudStorageContainerUrl;
(function (CloudStorageContainerUrl) {
    function empty() {
        return {
            descriptor: { name: "" },
            valid: 0,
            expires: 0,
            url: "",
        };
    }
    CloudStorageContainerUrl.empty = empty;
})(CloudStorageContainerUrl = exports.CloudStorageContainerUrl || (exports.CloudStorageContainerUrl = {}));
/** @beta */
class CloudStorageCache {
    constructor() {
        this.provider = CloudStorageProvider.Unknown;
        this._pendingContainerRequests = new Map();
        this._containers = new Map();
    }
    formContainerKey(id) { return this.formContainerName(id); }
    supplyUrlBase(_container, _id) { return undefined; }
    async retrieve(id) {
        return new Promise(async (resolve) => {
            try {
                const container = await this.getContainer(id);
                if (!container.url) {
                    resolve(undefined);
                    return;
                }
                const response = await this.requestResource(container, id);
                if (response.ok) {
                    const content = await this.instantiateResource(response);
                    resolve(content);
                }
                else {
                    resolve(undefined);
                }
            }
            catch (_err) {
                // todo: log this?
                resolve(undefined);
            }
        });
    }
    async requestResource(container, id) {
        const base = this.supplyUrlBase(container, id);
        const url = (undefined === base || "" === base) ? new URL(container.url) : new URL(container.url, base);
        if (!container.bound) {
            url.pathname += `/${this.formResourceName(id)}`;
        }
        const init = {
            headers: container.headers,
            method: "GET",
        };
        return fetch(url.toString(), init);
    }
    async getContainer(id) {
        const now = new Date().getTime();
        const name = this.formContainerName(id);
        const resource = this.formResourceName(id);
        const key = this.formContainerKey(id);
        let container = this._containers.get(key);
        if (container && container.url && (container.valid > now || container.expires < now)) {
            container = undefined;
            this._containers.delete(key);
        }
        if (!container) {
            let request = this._pendingContainerRequests.get(key);
            if (!request) {
                request = new Promise(async (resolve, reject) => {
                    try {
                        container = await this.obtainContainerUrl(id, { name, resource });
                        this._containers.set(key, container);
                        this._pendingContainerRequests.delete(key);
                        resolve(container);
                    }
                    catch (err) {
                        this._pendingContainerRequests.delete(key);
                        reject(err);
                    }
                });
                this._pendingContainerRequests.set(key, request);
            }
            return request;
        }
        return container;
    }
}
exports.CloudStorageCache = CloudStorageCache;
//# sourceMappingURL=CloudStorage.js.map