"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Codes
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeSpec = exports.CodeScopeSpec = exports.BisCodeSpec = exports.Code = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
/** A three-part structure containing information about the [Code]($docs/bis/intro/codes) of an Element
 * @public
 */
class Code {
    constructor(codeProps) {
        this.spec = bentleyjs_core_1.Id64.fromJSON(codeProps.spec);
        this.scope = bentleyjs_core_1.JsonUtils.asString(codeProps.scope);
        this.value = bentleyjs_core_1.JsonUtils.asString(codeProps.value);
    }
    /** The [CodeValue]($docs/bis/intro/codes.md#codevalue-property) of the Element
     * @note Leading and trailing whitespace is invalid so is automatically trimmed.
     */
    get value() { var _a; return (_a = this._value) !== null && _a !== void 0 ? _a : ""; }
    set value(val) { this._value = val === null || val === void 0 ? void 0 : val.trim(); }
    /** Create an empty, non-unique code with no special meaning. */
    static createEmpty() { const id = bentleyjs_core_1.Id64.fromLocalAndBriefcaseIds(1, 0); return new Code({ spec: id, scope: id }); }
    static fromJSON(json) { return json ? new Code(json) : Code.createEmpty(); }
    toJSON() { return { spec: this.spec, scope: this.scope, value: this.value }; }
    /** @deprecated Use the [[value]] property instead. */
    getValue() { return this.value; }
    equals(other) { return Code.equalCodes(this, other); }
    /** @internal */
    static equalCodes(c1, c2) {
        return c1.spec === c2.spec && c1.scope === c2.scope && c1.value === c2.value;
    }
    /** Determine whether this Code is valid. */
    static isValid(c) { return bentleyjs_core_1.Id64.isValidId64(c.spec); }
    /** Determine if this code is valid but not otherwise meaningful (and therefore not necessarily unique) */
    static isEmpty(c) { return this.isValid(c) && (c.value === undefined || c.value === ""); }
}
exports.Code = Code;
/** Names of the internal BIS CodeSpecs. These names match those specified by the native library.
 * For other domains, the best practice is to include the domain name or alias as part of the CodeSpec name to ensure global uniqueness.
 * @public
 * @see [CodeSpec]($docs/bis/intro/codes.md#codespec)
 */
var BisCodeSpec;
(function (BisCodeSpec) {
    /** The name of the standard [[CodeSpec]] used when creating *empty* codes.
     * @see [[Code.createEmpty]]
     */
    BisCodeSpec["nullCodeSpec"] = "bis:NullCodeSpec";
    /** @internal */
    BisCodeSpec["annotationFrameStyle"] = "bis:AnnotationFrameStyle";
    /** @internal */
    BisCodeSpec["annotationLeaderStyle"] = "bis:AnnotationLeaderStyle";
    /** @internal */
    BisCodeSpec["annotationTextStyle"] = "bis:AnnotationTextStyle";
    /** The name of the standard [[CodeSpec]] used when creating codes for [AuxCoordSystem2d]($backend) elements.
     * @see [AuxCoordSystem2d.createCode]($backend)
     */
    BisCodeSpec["auxCoordSystem2d"] = "bis:AuxCoordSystem2d";
    /** The name of the standard [[CodeSpec]] used when creating codes for [AuxCoordSystem3d]($backend) elements.
     * @see [AuxCoordSystem3d.createCode]($backend)
     */
    BisCodeSpec["auxCoordSystem3d"] = "bis:AuxCoordSystem3d";
    /** The name of the standard [[CodeSpec]] used when creating codes for [AuxCoordSystemSpatial]($backend) elements.
     * @see [AuxCoordSystemSpatial.createCode]($backend)
     */
    BisCodeSpec["auxCoordSystemSpatial"] = "bis:AuxCoordSystemSpatial";
    /** The name of the standard [[CodeSpec]] used when creating codes for [CategorySelector]($backend) elements.
     * @see [CategorySelector.createCode]($backend)
     */
    BisCodeSpec["categorySelector"] = "bis:CategorySelector";
    /** @internal */
    BisCodeSpec["colorBook"] = "bis:ColorBook";
    /** The name of the standard [[CodeSpec]] used when creating codes for [DisplayStyle]($backend) elements.
     * @see [DisplayStyle.createCode]($backend)
     */
    BisCodeSpec["displayStyle"] = "bis:DisplayStyle";
    /** The name of the standard [[CodeSpec]] used when creating codes for [Drawing]($backend) elements.
     * @see [Drawing.createCode]($backend)
     */
    BisCodeSpec["drawing"] = "bis:Drawing";
    /** The name of the standard [[CodeSpec]] used when creating codes for [DrawingCategory]($backend) elements.
     * @see [DrawingCategory.createCode]($backend)
     */
    BisCodeSpec["drawingCategory"] = "bis:DrawingCategory";
    /** The name of the standard [[CodeSpec]] used when creating codes for [ExternalSource]($backend) elements.
     * @note This CodeSpec is not automatically created, so use [ExternalSource.ensureCodeSpec]($backend) to make sure that it exists.
     * @see [ExternalSource.createCode]($backend)
     */
    BisCodeSpec["externalSource"] = "bis:ExternalSource";
    /** The name of the standard [[CodeSpec]] used when creating codes for [ExternalSourceAttachment]($backend) elements.
     * @note This CodeSpec is not automatically created, so use [ExternalSourceAttachment.ensureCodeSpec]($backend) to make sure that it exists.
     * @see [ExternalSource.createCode]($backend)
     */
    BisCodeSpec["externalSourceAttachment"] = "bis:ExternalSourceAttachment";
    /** The name of the standard [[CodeSpec]] used when creating codes for [GeometryPart]($backend) elements.
     * @see [GeometryPart.createCode]($backend)
     */
    BisCodeSpec["geometryPart"] = "bis:GeometryPart";
    /** The name of the standard [[CodeSpec]] used when creating codes for [GraphicalType2d]($backend) elements.
     * @see [GraphicalType2d.createCode]($backend)
     */
    BisCodeSpec["graphicalType2d"] = "bis:GraphicalType2d";
    /** The name of the standard [[CodeSpec]] used when creating codes for [LineStyle]($backend) elements.
     * @see [LineStyle.createCode]($backend)
     */
    BisCodeSpec["lineStyle"] = "bis:LineStyle";
    /** The name of the standard [[CodeSpec]] used when creating codes for [LinkElement]($backend) elements.
     * @see [LinkElement.createCode]($backend)
     */
    BisCodeSpec["linkElement"] = "bis:LinkElement";
    /** The name of the standard [[CodeSpec]] used when creating codes for [ModelSelector]($backend) elements.
     * @see [ModelSelector.createCode]($backend)
     */
    BisCodeSpec["modelSelector"] = "bis:ModelSelector";
    /** The name of the standard [[CodeSpec]] used when creating codes for [PhysicalMaterial]($backend) elements.
     * @see [PhysicalMaterial.createCode]($backend)
     */
    BisCodeSpec["physicalMaterial"] = "bis:PhysicalMaterial";
    /** The name of the standard [[CodeSpec]] used when creating codes for [PhysicalType]($backend) elements.
     * @see [PhysicalType.createCode]($backend)
     */
    BisCodeSpec["physicalType"] = "bis:PhysicalType";
    /** The name of the standard [[CodeSpec]] used when creating codes for [InformationPartitionElement]($backend) elements.
     * @see [InformationPartitionElement.createCode]($backend)
     */
    BisCodeSpec["informationPartitionElement"] = "bis:InformationPartitionElement";
    /** The name of the standard [[CodeSpec]] used when creating codes for [RenderMaterialElement]($backend) elements.
     * @see [RenderMaterialElement.createCode]($backend)
     */
    BisCodeSpec["renderMaterial"] = "bis:RenderMaterial";
    /** The name of the standard [[CodeSpec]] used when creating codes for [Sheet]($backend) elements.
     * @see [Sheet.createCode]($backend)
     */
    BisCodeSpec["sheet"] = "bis:Sheet";
    /** The name of the standard [[CodeSpec]] used when creating codes for [SpatialCategory]($backend) elements.
     * @see [SpatialCategory.createCode]($backend)
     */
    BisCodeSpec["spatialCategory"] = "bis:SpatialCategory";
    /** The name of the standard [[CodeSpec]] used when creating codes for [SpatialLocationType]($backend) elements.
     * @see [SpatialLocationType.createCode]($backend)
     */
    BisCodeSpec["spatialLocationType"] = "bis:SpatialLocationType";
    /** The name of the standard [[CodeSpec]] used when creating codes for [SubCategory]($backend) elements.
     * @see [SubCategory.createCode]($backend)
     */
    BisCodeSpec["subCategory"] = "bis:SubCategory";
    /** The name of the standard [[CodeSpec]] used when creating codes for [Subject]($backend) elements.
     * @see [Subject.createCode]($backend)
     */
    BisCodeSpec["subject"] = "bis:Subject";
    /** The name of the standard [[CodeSpec]] used when creating codes for [TemplateRecipe2d]($backend) elements.
     * @see [TemplateRecipe2d.createCode]($backend)
     */
    BisCodeSpec["templateRecipe2d"] = "bis:TemplateRecipe2d";
    /** The name of the standard [[CodeSpec]] used when creating codes for [TemplateRecipe3d]($backend) elements.
     * @see [TemplateRecipe3d.createCode]($backend)
     */
    BisCodeSpec["templateRecipe3d"] = "bis:TemplateRecipe3d";
    /** @internal */
    BisCodeSpec["textAnnotationSeed"] = "bis:TextAnnotationSeed";
    /** The name of the standard [[CodeSpec]] used when creating codes for [Texture]($backend) elements.
     * @see [Texture.createCode]($backend)
     */
    BisCodeSpec["texture"] = "bis:Texture";
    /** The name of the standard [[CodeSpec]] used when creating codes for [ViewDefinition]($backend) elements.
     * @see [ViewDefinition.createCode]($backend)
     */
    BisCodeSpec["viewDefinition"] = "bis:ViewDefinition";
})(BisCodeSpec = exports.BisCodeSpec || (exports.BisCodeSpec = {}));
/** The scope of the Code.
 * @public
 */
var CodeScopeSpec;
(function (CodeScopeSpec) {
    /** The standard ways the CodeScope can be specified.
     * @public
     */
    let Type;
    (function (Type) {
        /** The Code value must be unique within (at least) the iModel repository */
        Type[Type["Repository"] = 1] = "Repository";
        /** The Code value must be unique within the scope of the Model */
        Type[Type["Model"] = 2] = "Model";
        /** The Code value must be unique among other children of the same parent element */
        Type[Type["ParentElement"] = 3] = "ParentElement";
        /** The Code value must be unique among other elements also scoped by the same element */
        Type[Type["RelatedElement"] = 4] = "RelatedElement";
    })(Type = CodeScopeSpec.Type || (CodeScopeSpec.Type = {}));
    /** Requirements for how the CodeScope Element is identified.
     * @public
     */
    let ScopeRequirement;
    (function (ScopeRequirement) {
        /** The Code is required to have a valid ElementId as its scope */
        ScopeRequirement[ScopeRequirement["ElementId"] = 1] = "ElementId";
        /** The Code is required to have a valid FederationGuid as its scope */
        ScopeRequirement[ScopeRequirement["FederationGuid"] = 2] = "FederationGuid";
    })(ScopeRequirement = CodeScopeSpec.ScopeRequirement || (CodeScopeSpec.ScopeRequirement = {}));
})(CodeScopeSpec = exports.CodeScopeSpec || (exports.CodeScopeSpec = {}));
/** A [Code Specification]($docs/bis/intro/glossary#codespec) captures the rules for encoding and decoding significant business information into
 * and from a Code (string). This specification is used to generate and validate Codes.
 *
 * A CodeSpec defines the format of a Code for a certain type of Element in an IModel.
 * A CodeSpec can identify an external system that maintains and/or assigns Codes.
 * @public
 */
class CodeSpec {
    /** Internal-only constructor. Proper use is to supply `properties` only or `scopeType` and `scopeReq` but not `properties`.
     * > Note: The deprecation has to do with moving the constructor from public to internal
     * @deprecated Use [[create]] or [[createFromJson]] instead of the internal constructor
     * @internal
     */
    constructor(iModel, id, name, scopeType, scopeReq, properties) {
        this.iModel = iModel;
        this.id = id;
        this.name = name;
        if (properties) {
            this.properties = properties;
            if (!this.properties.scopeSpec) {
                this.properties.scopeSpec = {};
                this.scopeType = CodeScopeSpec.Type.Repository;
            }
        }
        else {
            this.properties = { scopeSpec: {} };
            this.scopeType = CodeScopeSpec.Type.Repository;
        }
        if (undefined !== scopeType)
            this.scopeType = scopeType;
        if (undefined !== scopeReq)
            this.scopeReq = scopeReq;
    }
    /** Create a new CodeSpec from the specified parameters
     * > Note: CodeSpec.id will not be valid until inserted
     * @see [CodeSpecs.insert]($backend)
     */
    static create(iModel, name, scopeType, scopeReq) {
        return new CodeSpec(iModel, bentleyjs_core_1.Id64.invalid, name, scopeType, scopeReq, undefined); // eslint-disable-line deprecation/deprecation
    }
    /** Create a new CodeSpec directly from JSON. Used internally by the CodeSpecs.load function.
     * @internal
     */
    static createFromJson(iModel, id, name, properties) {
        return new CodeSpec(iModel, id, name, undefined, undefined, properties); // eslint-disable-line deprecation/deprecation
    }
    /** Will be true if the id of this CodeSpec is valid. */
    get isValid() { return bentleyjs_core_1.Id64.isValid(this.id); }
    /** The scope type of this CodeSpec.
     * @deprecated Use scopeType instead.
     */
    get specScopeType() { return this.scopeType; }
    set specScopeType(scopeType) { this.scopeType = scopeType; }
    /** The scope type of this CodeSpec. */
    get scopeType() { return this.properties.scopeSpec.type; }
    set scopeType(scopeType) { this.properties.scopeSpec.type = scopeType; }
    /** Will be `CodeScopeSpec.ScopeRequirement.FederationGuid` if the scoping element is required to have a FederationGuid or `CodeScopeSpec.ScopeRequirement.ElementId` otherwise (which is the default). */
    get scopeReq() {
        return this.properties.scopeSpec.fGuidRequired ? CodeScopeSpec.ScopeRequirement.FederationGuid : CodeScopeSpec.ScopeRequirement.ElementId;
    }
    set scopeReq(req) {
        if (CodeScopeSpec.ScopeRequirement.FederationGuid === req)
            this.properties.scopeSpec.fGuidRequired = true;
        else
            this.properties.scopeSpec.fGuidRequired = undefined;
    }
    /** Will be true if the codes associated with this CodeSpec are managed along with the iModel and false if the codes are managed by an external service.
     * @beta
     */
    get isManagedWithIModel() {
        if (this.properties.spec && this.properties.spec.isManagedWithDgnDb !== undefined) {
            return this.properties.spec.isManagedWithDgnDb;
        }
        return true;
    }
    set isManagedWithIModel(value) {
        if (!this.properties.spec)
            this.properties.spec = {};
        this.properties.spec.isManagedWithDgnDb = value;
    }
}
exports.CodeSpec = CodeSpec;
//# sourceMappingURL=Code.js.map