"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContextRealityModels = exports.ContextRealityModel = exports.ContextRealityModelProps = exports.RealityDataFormat = exports.RealityDataProvider = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const FeatureSymbology_1 = require("./FeatureSymbology");
const PlanarClipMask_1 = require("./PlanarClipMask");
const SpatialClassification_1 = require("./SpatialClassification");
/** Identify the Reality Data service provider
 * @alpha
 */
var RealityDataProvider;
(function (RealityDataProvider) {
    /**
     * This is the legacy mode where the access to the 3d tiles is harcoded in ContextRealityModelProps.tilesetUrl property.
     * It was use to support RealityMesh3DTiles, Terrain3DTiles, Cesium3DTiles
     * You should use other mode when possible
     * @see [[RealityDataSource.createRealityDataSourceKeyFromUrl]] that will try to detect provider from an URL
     */
    RealityDataProvider["TilesetUrl"] = "TilesetUrl";
    /**
     * This is the legacy mode where the access to the 3d tiles is harcoded in ContextRealityModelProps.OrbitGtBlob property.
     * It was use to support OrbitPointCloud (OPC) from other server than ContextShare
     * You should use other mode when possible
     * @see [[RealityDataSource.createRealityDataSourceKeyFromUrl]] that will try to detect provider from an URL
     */
    RealityDataProvider["OrbitGtBlob"] = "OrbitGtBlob";
    /**
     * Will provide access url from realityDataId and iTwinId on contextShare for 3dTile storage format or  OPC storage format
     * This provider support all type of 3dTile storage fomat and OrbitPointCloud: RealityMesh3DTiles, Terrain3DTiles, Cesium3DTiles, OPC
     * @see [[RealityDataFormat]].
     */
    RealityDataProvider["ContextShare"] = "ContextShare";
    /**
     * Will provide Open Street Map Building (OSM) from Cesium Ion (in 3dTile format)
     */
    RealityDataProvider["CesiumIonAsset"] = "CesiumIonAsset";
})(RealityDataProvider = exports.RealityDataProvider || (exports.RealityDataProvider = {}));
/** Identify the Reality Data storage format
 * @alpha
 */
var RealityDataFormat;
(function (RealityDataFormat) {
    /**
     * 3dTile supported formats; RealityMesh3DTiles, Terrain3DTiles, Cesium3DTiles
     * */
    RealityDataFormat["ThreeDTile"] = "ThreeDTile";
    /**
     * Orbit Point Cloud (OPC) storage format (RealityDataType.OPC)
    */
    RealityDataFormat["OPC"] = "OPC";
})(RealityDataFormat = exports.RealityDataFormat || (exports.RealityDataFormat = {}));
/** @public */
var ContextRealityModelProps;
(function (ContextRealityModelProps) {
    /** Produce a deep copy of `input`. */
    function clone(input) {
        var _a;
        // Spread operator is shallow, and includes `undefined` properties and empty strings.
        // We want to make deep copies, omit undefined properties and empty strings, and require tilesetUrl to be defined.
        const output = { tilesetUrl: (_a = input.tilesetUrl) !== null && _a !== void 0 ? _a : "" };
        if (input.rdSourceKey)
            output.rdSourceKey = Object.assign({}, input.rdSourceKey);
        if (input.name)
            output.name = input.name;
        if (input.realityDataId)
            output.realityDataId = input.realityDataId;
        if (input.description)
            output.description = input.description;
        if (input.orbitGtBlob)
            output.orbitGtBlob = Object.assign({}, input.orbitGtBlob);
        if (input.appearanceOverrides) {
            output.appearanceOverrides = Object.assign({}, input.appearanceOverrides);
            if (input.appearanceOverrides.rgb)
                output.appearanceOverrides.rgb = Object.assign({}, input.appearanceOverrides.rgb);
        }
        if (input.planarClipMask)
            output.planarClipMask = Object.assign({}, input.planarClipMask);
        if (input.classifiers)
            output.classifiers = input.classifiers.map((x) => { return Object.assign(Object.assign({}, x), { flags: Object.assign({}, x.flags) }); });
        return output;
    }
    ContextRealityModelProps.clone = clone;
})(ContextRealityModelProps = exports.ContextRealityModelProps || (exports.ContextRealityModelProps = {}));
/** A reality model not associated with a [GeometricModel]($backend) but instead defined in a [DisplayStyle]($backend) or [DisplayStyleState]($frontend).
 * Such reality models are displayed to provide context to the view and can be freely attached and detached at display time.
 * @see [this interactive example](https://www.itwinjs.org/sample-showcase/?group=Viewer&sample=reality-data-sample)
 * @see [[DisplayStyleSettings.contextRealityModels]] to define context reality models for a display style.
 * @public
 */
class ContextRealityModel {
    /** Construct a new context reality model.
     * @param props JSON representation of the reality model, which will be kept in sync with changes made via the ContextRealityModel's methods.
     */
    constructor(props) {
        var _a, _b, _c;
        /** Event dispatched just before assignment to [[planarClipMaskSettings]]. */
        this.onPlanarClipMaskChanged = new bentleyjs_core_1.BeEvent();
        /** Event dispatched just before assignment to [[appearanceOverrides]]. */
        this.onAppearanceOverridesChanged = new bentleyjs_core_1.BeEvent();
        this._props = props;
        this.rdSourceKey = props.rdSourceKey;
        this.name = (_a = props.name) !== null && _a !== void 0 ? _a : "";
        this.url = (_b = props.tilesetUrl) !== null && _b !== void 0 ? _b : "";
        this.orbitGtBlob = props.orbitGtBlob;
        this.realityDataId = props.realityDataId;
        this.description = (_c = props.description) !== null && _c !== void 0 ? _c : "";
        this._appearanceOverrides = props.appearanceOverrides ? FeatureSymbology_1.FeatureAppearance.fromJSON(props.appearanceOverrides) : undefined;
        if (props.planarClipMask && props.planarClipMask.mode !== PlanarClipMask_1.PlanarClipMaskMode.None)
            this._planarClipMask = PlanarClipMask_1.PlanarClipMaskSettings.fromJSON(props.planarClipMask);
        if (props.classifiers)
            this.classifiers = new SpatialClassification_1.SpatialClassifiers(props);
    }
    /** Optionally describes how the geometry of the reality model can be masked by other models. */
    get planarClipMaskSettings() {
        return this._planarClipMask;
    }
    set planarClipMaskSettings(settings) {
        this.onPlanarClipMaskChanged.raiseEvent(settings, this);
        if (!settings)
            delete this._props.planarClipMask;
        else
            this._props.planarClipMask = settings.toJSON();
        this._planarClipMask = settings;
    }
    /** Overrides applied to the appearance of the reality model. Only the rgb, transparency, nonLocatable, and emphasized properties are applicable - the rest are ignored. */
    get appearanceOverrides() {
        return this._appearanceOverrides;
    }
    set appearanceOverrides(overrides) {
        this.onAppearanceOverridesChanged.raiseEvent(overrides, this);
        if (!overrides)
            delete this._props.appearanceOverrides;
        else
            this._props.appearanceOverrides = overrides.toJSON();
        this._appearanceOverrides = overrides;
    }
    /** Convert this model to its JSON representation. */
    toJSON() {
        return ContextRealityModelProps.clone(this._props);
    }
    /** Returns true if [[name]] and [[url]] match the specified name and url. */
    matchesNameAndUrl(name, url) {
        return this.name === name && this.url === url;
    }
}
exports.ContextRealityModel = ContextRealityModel;
/** A list of [[ContextRealityModel]]s attached to a [[DisplayStyleSettings]]. The list may be presented to the user with the name and description of each model.
 * The list is automatically synchronized with the underlying JSON representation provided by the input [[ContextRealityModelsContainer]].
 * @see [this interactive example](https://www.itwinjs.org/sample-showcase/?group=Viewer&sample=reality-data-sample)
 * @see [[DisplayStyleSettings.contextRealityModels]].
 * @public
 */
class ContextRealityModels {
    /** Construct a new list of reality models from its JSON representation. THe list will be initialized from `container.classifiers` and that JSON representation
     * will be kept in sync with changes made to the list. The caller should not directly modify `container.classifiers` or its contents as that will cause the list
     * to become out of sync with the JSON representation.
     * @param container The object that holds the JSON representation of the list.
     * @param createContextRealityModel Optional function used to instantiate ContextRealityModels added to the list.
     */
    constructor(container, createContextRealityModel) {
        this._models = [];
        /** Event dispatched just before [[ContextRealityModel.planarClipMaskSettings]] is modified for one of the reality models. */
        this.onPlanarClipMaskChanged = new bentleyjs_core_1.BeEvent();
        /** Event dispatched just before [[ContextRealityModel.appearanceOverrides]] is modified for one of the reality models. */
        this.onAppearanceOverridesChanged = new bentleyjs_core_1.BeEvent();
        /** Event dispatched when a model is [[add]]ed, [[delete]]d, [[replace]]d, or [[update]]d. */
        this.onChanged = new bentleyjs_core_1.BeEvent();
        this._container = container;
        this._createModel = createContextRealityModel !== null && createContextRealityModel !== void 0 ? createContextRealityModel : ((props) => new ContextRealityModel(props));
        const models = container.contextRealityModels;
        if (models)
            for (const model of models)
                this._models.push(this.createModel(model));
    }
    /** The read-only list of reality models. */
    get models() {
        return this._models;
    }
    /** Append a new reality model to the list.
     * @param The JSON representation of the reality model.
     * @returns the newly-added reality model.
     */
    add(props) {
        if (!this._container.contextRealityModels)
            this._container.contextRealityModels = [];
        props = ContextRealityModelProps.clone(props);
        const model = this.createModel(props);
        this.onChanged.raiseEvent(undefined, model);
        this._models.push(model);
        this._container.contextRealityModels.push(props);
        return model;
    }
    /** Remove the specified reality model from the list.
     * @param model The reality model to remove.
     * @returns true if the model was removed, or false if the model was not present in the list.
     */
    delete(model) {
        const index = this._models.indexOf(model);
        if (-1 === index)
            return false;
        bentleyjs_core_1.assert(undefined !== this._container.contextRealityModels);
        bentleyjs_core_1.assert(index < this._container.contextRealityModels.length);
        this.dropEventListeners(model);
        this.onChanged.raiseEvent(model, undefined);
        this._models.splice(index, 1);
        if (this.models.length === 0)
            this._container.contextRealityModels = undefined;
        else
            this._container.contextRealityModels.splice(index, 1);
        return true;
    }
    /** Remove all reality models from the list. */
    clear() {
        for (const model of this.models) {
            this.dropEventListeners(model);
            this.onChanged.raiseEvent(model, undefined);
        }
        this._container.contextRealityModels = undefined;
        this._models.length = 0;
    }
    /** Replace a reality model in the list.
     * @param toReplace The reality model to be replaced.
     * @param replaceWith The JSON representation of the replacement reality model.
     * @returns the newly-created reality model that replaced `toReplace`.
     * @throws Error if `toReplace` is not present in the list
     * @note The replacement occupies the same index in the list as `toReplace` did.
     */
    replace(toReplace, replaceWith) {
        const index = this._models.indexOf(toReplace);
        if (-1 === index)
            throw new Error("ContextRealityModel not present in list.");
        bentleyjs_core_1.assert(undefined !== this._container.contextRealityModels);
        bentleyjs_core_1.assert(index < this._container.contextRealityModels.length);
        replaceWith = ContextRealityModelProps.clone(replaceWith);
        const model = this.createModel(replaceWith);
        this.onChanged.raiseEvent(toReplace, model);
        this.dropEventListeners(toReplace);
        this._models[index] = model;
        this._container.contextRealityModels[index] = replaceWith;
        return model;
    }
    /** Change selected properties of a reality model.
     * @param toUpdate The reality model whose properties are to be modified.
     * @param updateProps The properties to change.
     * @returns The updated reality model, identical to `toUpdate` except for properties explicitly supplied by `updateProps`.
     * @throws Error if `toUpdate` is not present in the list.
     */
    update(toUpdate, updateProps) {
        const props = Object.assign(Object.assign({}, toUpdate.toJSON()), updateProps);
        // Partial<> makes it possible to pass `undefined` for tilesetUrl...preserve previous URL in that case.
        if (undefined === props.tilesetUrl)
            props.tilesetUrl = toUpdate.url;
        return this.replace(toUpdate, props);
    }
    createModel(props) {
        const model = this._createModel(props);
        // eslint-disable-next-line @typescript-eslint/unbound-method
        model.onPlanarClipMaskChanged.addListener(this.handlePlanarClipMaskChanged, this);
        // eslint-disable-next-line @typescript-eslint/unbound-method
        model.onAppearanceOverridesChanged.addListener(this.handleAppearanceOverridesChanged, this);
        return model;
    }
    dropEventListeners(model) {
        // eslint-disable-next-line @typescript-eslint/unbound-method
        model.onPlanarClipMaskChanged.removeListener(this.handlePlanarClipMaskChanged, this);
        // eslint-disable-next-line @typescript-eslint/unbound-method
        model.onAppearanceOverridesChanged.removeListener(this.handleAppearanceOverridesChanged, this);
    }
    handlePlanarClipMaskChanged(mask, model) {
        this.onPlanarClipMaskChanged.raiseEvent(model, mask);
    }
    handleAppearanceOverridesChanged(app, model) {
        this.onAppearanceOverridesChanged.raiseEvent(model, app);
    }
}
exports.ContextRealityModels = ContextRealityModels;
//# sourceMappingURL=ContextRealityModel.js.map