"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DisplayStyle3dSettings = exports.DisplayStyleSettings = exports.MonochromeMode = void 0;
// cspell:ignore greyscale ovrs
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const AmbientOcclusion_1 = require("./AmbientOcclusion");
const AnalysisStyle_1 = require("./AnalysisStyle");
const BackgroundMapSettings_1 = require("./BackgroundMapSettings");
const ClipStyle_1 = require("./ClipStyle");
const ColorDef_1 = require("./ColorDef");
const HiddenLine_1 = require("./HiddenLine");
const FeatureSymbology_1 = require("./FeatureSymbology");
const PlanarClipMask_1 = require("./PlanarClipMask");
const SubCategoryOverride_1 = require("./SubCategoryOverride");
const LightSettings_1 = require("./LightSettings");
const MapImagerySettings_1 = require("./MapImagerySettings");
const PlanProjectionSettings_1 = require("./PlanProjectionSettings");
const SolarShadows_1 = require("./SolarShadows");
const ThematicDisplay_1 = require("./ThematicDisplay");
const ViewFlags_1 = require("./ViewFlags");
const Cartographic_1 = require("./geometry/Cartographic");
const IModel_1 = require("./IModel");
const SolarCalculate_1 = require("./SolarCalculate");
const ContextRealityModel_1 = require("./ContextRealityModel");
const WhiteOnWhiteReversalSettings_1 = require("./WhiteOnWhiteReversalSettings");
/** Describes the style in which monochrome color is applied by a [[DisplayStyleSettings]].
 * @public
 */
var MonochromeMode;
(function (MonochromeMode) {
    /** The color of the geometry is replaced with the monochrome color. e.g., if monochrome color is white, the geometry will be white. */
    MonochromeMode[MonochromeMode["Flat"] = 0] = "Flat";
    /** The color of surfaces is computed as normal, then scaled to a shade of the monochrome color based on the surface color's intensity.
     * For example, if the monochrome color is white, this results in a greyscale effect.
     * Geometry other than surfaces is treated the same as [[MonochromeMode.Flat]].
     */
    MonochromeMode[MonochromeMode["Scaled"] = 1] = "Scaled";
})(MonochromeMode = exports.MonochromeMode || (exports.MonochromeMode = {}));
/** DisplayStyleSettings initially persisted its excluded elements as an array of Id64Strings in JSON, and exposed them as a Set<string>.
 * This becomes problematic when these arrays become very large, in terms of the amount of data and the time required to convert them to a Set.
 * The Ids are now persisted to JSON as a [[CompressedId64Set]], significantly reducing their size. However, for backwards API compatibility we must
 * continue to expose [[DisplayStyleSettings.excludedElements]] as a Set<string>. The [[ExcludedElements]] class tries to minimize the impact of that requirement by
 * maintaining the Ids primarily as a [[MutableCompressedId64Set]], only allocating the Set<string> if a caller actually requests it.
 * The only operation Set provides more efficiently than MutableCompressedId64Set is checking for the presence of an Id (the `has()` method).
 * @internal
 */
class ExcludedElements {
    constructor(json) {
        this._synchronizing = false;
        this._json = json;
        if (Array.isArray(json.excludedElements))
            this._ids = new bentleyjs_core_1.MutableCompressedId64Set(bentleyjs_core_1.CompressedId64Set.compressIds(bentleyjs_core_1.OrderedId64Iterable.sortArray(json.excludedElements)));
        else
            this._ids = new bentleyjs_core_1.MutableCompressedId64Set(json.excludedElements);
    }
    reset(ids) {
        this.synchronize(() => {
            var _a;
            (_a = this._set) === null || _a === void 0 ? void 0 : _a.clear();
            this._ids.reset((ids && "string" !== typeof ids) ? bentleyjs_core_1.CompressedId64Set.compressIds(ids) : ids);
        });
    }
    get ids() {
        return this._ids.ids;
    }
    add(ids) {
        this.synchronize(() => {
            var _a;
            for (const id of ids) {
                this._ids.add(id);
                (_a = this._set) === null || _a === void 0 ? void 0 : _a.add(id);
            }
        });
    }
    delete(ids) {
        this.synchronize(() => {
            var _a;
            for (const id of ids) {
                this._ids.delete(id);
                (_a = this._set) === null || _a === void 0 ? void 0 : _a.delete(id);
            }
        });
    }
    obtainSet() {
        if (this._set)
            return this._set;
        this.synchronize(() => {
            this._set = new bentleyjs_core_1.ObservableSet(this._ids);
            this._set.onAdded.addListener((id) => this.onAdded(id));
            this._set.onDeleted.addListener((id) => this.onDeleted(id));
            this._set.onCleared.addListener(() => this.onCleared());
        });
        bentleyjs_core_1.assert(undefined !== this._set);
        return this._set;
    }
    [Symbol.iterator]() {
        return this._ids[Symbol.iterator]();
    }
    onAdded(id) {
        this.synchronize(() => this._ids.add(id));
    }
    onDeleted(id) {
        this.synchronize(() => this._ids.delete(id));
    }
    onCleared() {
        this.synchronize(() => this._ids.clear());
    }
    /** The JSON must be kept up-to-date at all times. */
    synchronize(func) {
        if (this._synchronizing)
            return;
        this._synchronizing = true;
        try {
            func();
        }
        finally {
            this._synchronizing = false;
            const ids = this._ids.ids;
            if (0 === ids.length)
                delete this._json.excludedElements;
            else
                this._json.excludedElements = ids;
        }
    }
}
/** An implementation of Map that is based on a JSON array, used for a display styles subcategory overrides, model appearance overrides,
 * and planar clip masks. Ensures:
 *  - JSON representation kept in sync with changes to map; and
 *  - Events dispatched when map contents change.
 */
class OverridesMap extends Map {
    constructor(_json, _arrayKey, _event, _idFromProps, _overrideToProps, _overrideFromProps) {
        super();
        this._json = _json;
        this._arrayKey = _arrayKey;
        this._event = _event;
        this._idFromProps = _idFromProps;
        this._overrideToProps = _overrideToProps;
        this._overrideFromProps = _overrideFromProps;
        this.populate();
    }
    // This is required for mock framework used by ui libraries, which otherwise try to clone this as a standard Map.
    get [Symbol.toStringTag]() { return "OverridesMap"; }
    set(id, override) {
        this._event.raiseEvent(id, override);
        super.set(id, override);
        const index = this.findOrAllocateIndex(id);
        const array = this._array;
        bentleyjs_core_1.assert(undefined !== array);
        array[index] = this._overrideToProps(override, id);
        return this;
    }
    delete(id) {
        this._event.raiseEvent(id, undefined);
        if (!super.delete(id))
            return false;
        const index = this.findExistingIndex(id);
        if (undefined !== index) {
            bentleyjs_core_1.assert(undefined !== this._array);
            this._array.splice(index, 1);
        }
        return true;
    }
    clear() {
        for (const id of this.keys())
            this.delete(id);
        this._json[this._arrayKey] = undefined;
    }
    populate() {
        super.clear();
        const ovrs = this._array;
        if (!ovrs)
            return;
        for (const props of ovrs) {
            const id = this._idFromProps(props);
            if (undefined !== id && bentleyjs_core_1.Id64.isValidId64(id)) {
                const ovr = this._overrideFromProps(props);
                if (ovr)
                    super.set(id, ovr);
            }
        }
    }
    get _array() {
        return bentleyjs_core_1.JsonUtils.asArray(this._json[this._arrayKey]);
    }
    findOrAllocateIndex(id) {
        const index = this.findExistingIndex(id);
        if (undefined !== index)
            return index;
        let ovrs = this._array;
        if (!ovrs)
            ovrs = this._json[this._arrayKey] = [];
        return ovrs.length;
    }
    findExistingIndex(id) {
        const ovrs = this._array;
        if (!ovrs)
            return undefined;
        for (let i = 0; i < ovrs.length; i++)
            if (this._idFromProps(ovrs[i]) === id)
                return i;
        return undefined;
    }
}
/** Provides access to the settings defined by a [[DisplayStyle]] or [[DisplayStyleState]], and ensures that
 * the style's JSON properties are kept in sync.
 * @public
 */
class DisplayStyleSettings {
    /** Construct a new DisplayStyleSettings from an [[ElementProps.jsonProperties]].
     * @param jsonProperties An object with an optional `styles` property containing a display style's settings.
     * @param options Options for customizing the display style settings.
     * @note When the `DisplayStyleSetting`'s properties are modified by public setters, the `jsonProperties`'s `styles` object will be updated to reflect the change.
     * @note If `jsonProperties` contains no `styles` member, one will be added as an empty object.
     * @note Generally there is no reason to create an object of this type directly; a [[DisplayStyle]] or [[DisplayStyleState]] constructs one as part of its own construction.
     */
    constructor(jsonProperties, options) {
        /** Event raised by [[applyOverrides]] just before the overrides are applied. */
        this.onApplyOverrides = new bentleyjs_core_1.BeEvent();
        /** Event raised by [[applyOverrides]] after the overrides are applied. */
        this.onOverridesApplied = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[viewFlags]] property. */
        this.onViewFlagsChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[backgroundColor]] property. */
        this.onBackgroundColorChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[monochromeColor]] property. */
        this.onMonochromeColorChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[monochromeMode]] property. */
        this.onMonochromeModeChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[backgroundMap]] property. */
        this.onBackgroundMapChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[mapImagery]] property.
         * @alpha
         */
        this.onMapImageryChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the `scheduleScriptProps` property.
         * @deprecated Use onRenderTimelineChanged
         * @internal
         */
        this.onScheduleScriptPropsChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[renderTimeline]] property. */
        this.onRenderTimelineChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[timePoint]] property. */
        this.onTimePointChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[analysisStyle]] property. */
        this.onAnalysisStyleChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[analysisFraction]] property. */
        this.onAnalysisFractionChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised when the contents of [[excludedElementIds]] changes. */
        this.onExcludedElementsChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[clipStyle]] property. */
        this.onClipStyleChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised when the [[SubCategoryOverride]]s change. */
        this.onSubCategoryOverridesChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just before changing the appearance override for a model. */
        this.onModelAppearanceOverrideChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[thematic]] property. */
        this.onThematicChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[hiddenLineSettings]] property. */
        this.onHiddenLineSettingsChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[ambientOcclusionSettings]] property. */
        this.onAmbientOcclusionSettingsChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[solarShadows]] property. */
        this.onSolarShadowsChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[environment]] property. */
        this.onEnvironmentChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[lights]] property. */
        this.onLightsChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just before changing the plan projection settings for a model. */
        this.onPlanProjectionSettingsChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just before adding or removing an entry from [[planarClipMasks]]. */
        this.onPlanarClipMaskChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised just prior to assignment to the [[whiteOnWhiteReversal]] property. */
        this.onWhiteOnWhiteReversalChanged = new bentleyjs_core_1.BeEvent();
        if (undefined === jsonProperties.styles)
            jsonProperties.styles = {};
        this._json = jsonProperties.styles;
        this._viewFlags = ViewFlags_1.ViewFlags.fromJSON(this._json.viewflags);
        this._background = ColorDef_1.ColorDef.fromJSON(this._json.backgroundColor);
        this._monochrome = undefined !== this._json.monochromeColor ? ColorDef_1.ColorDef.fromJSON(this._json.monochromeColor) : ColorDef_1.ColorDef.white;
        this._monochromeMode = MonochromeMode.Flat === this._json.monochromeMode ? MonochromeMode.Flat : MonochromeMode.Scaled;
        this._backgroundMap = BackgroundMapSettings_1.BackgroundMapSettings.fromJSON(this._json.backgroundMap);
        this._mapImagery = MapImagerySettings_1.MapImagerySettings.fromJSON(this._json.mapImagery, this._json.backgroundMap);
        this._excludedElements = new ExcludedElements(this._json);
        if (this._json.analysisStyle)
            this._analysisStyle = AnalysisStyle_1.AnalysisStyle.fromJSON(this._json.analysisStyle);
        this._whiteOnWhiteReversal = WhiteOnWhiteReversalSettings_1.WhiteOnWhiteReversalSettings.fromJSON(this._json.whiteOnWhiteReversal);
        this._clipStyle = ClipStyle_1.ClipStyle.fromJSON(this._json.clipStyle);
        this._subCategoryOverrides = new OverridesMap(this._json, "subCategoryOvr", this.onSubCategoryOverridesChanged, (props) => props.subCategory, (ovr, subCategory) => { return Object.assign(Object.assign({}, ovr.toJSON()), { subCategory }); }, (props) => {
            const ovr = SubCategoryOverride_1.SubCategoryOverride.fromJSON(props);
            return ovr.anyOverridden ? ovr : undefined;
        });
        this._modelAppearanceOverrides = new OverridesMap(this._json, "modelOvr", this.onModelAppearanceOverrideChanged, (props) => props.modelId, (ovr, modelId) => { return Object.assign(Object.assign({}, ovr.toJSON()), { modelId }); }, (props) => {
            const app = FeatureSymbology_1.FeatureAppearance.fromJSON(props);
            return app.anyOverridden ? app : undefined;
        });
        this._planarClipMasks = new OverridesMap(this._json, "planarClipOvr", this.onPlanarClipMaskChanged, (props) => props.modelId, (ovr, modelId) => { return Object.assign(Object.assign({}, ovr.toJSON()), { modelId }); }, (props) => {
            const settings = PlanarClipMask_1.PlanarClipMaskSettings.fromJSON(props);
            return settings.isValid ? settings : undefined;
        });
        this._contextRealityModels = new ContextRealityModel_1.ContextRealityModels(this._json, options === null || options === void 0 ? void 0 : options.createContextRealityModel);
    }
    is3d() {
        return false;
    }
    /** Planar clip masks to be applied to persistent reality models (@see [SpatialModelState.isRealityModel]($frontend).
     * The key for each entry is the Id of the model to which the mask settings apply.
     */
    get planarClipMasks() {
        return this._planarClipMasks;
    }
    /** Reality models to be displayed in the view. */
    get contextRealityModels() {
        return this._contextRealityModels;
    }
    /** Flags controlling various aspects of the display style. To change the style's view flags, do something like:
     * ```ts
     *  const flags = settings.viewFlags.clone();
     *  flags.renderMode = RenderMode.SmoothShade; // or any other alterations.
     *  settings.viewFlags = flags;
     * @note Don't modify this object directly - clone it and modify the clone, then pass the clone to the setter.
     */
    get viewFlags() { return this._viewFlags; }
    set viewFlags(flags) {
        if (this.viewFlags.equals(flags))
            return;
        this.onViewFlagsChanged.raiseEvent(flags);
        flags.clone(this._viewFlags);
        this._json.viewflags = flags.toJSON();
    }
    /** The background color.
     * @note Do not modify the color in place. Clone it and pass the clone to the setter.
     */
    get backgroundColor() { return this._background; }
    set backgroundColor(color) {
        if (this.backgroundColor.equals(color))
            return;
        this.onBackgroundColorChanged.raiseEvent(color);
        this._background = color;
        this._json.backgroundColor = color.toJSON();
    }
    /** The color used to draw geometry in monochrome mode.
     * @note Do not modify the color in place. Clone it and pass the clone to the setter.
     * @see [[ViewFlags.monochrome]] for enabling monochrome mode.
     */
    get monochromeColor() { return this._monochrome; }
    set monochromeColor(color) {
        if (this.monochromeColor.equals(color))
            return;
        this.onMonochromeColorChanged.raiseEvent(color);
        this._monochrome = color;
        this._json.monochromeColor = color.toJSON();
    }
    /** The style in which [[monochromeColor]] is applied. */
    get monochromeMode() { return this._monochromeMode; }
    set monochromeMode(mode) {
        if (this.monochromeMode === mode)
            return;
        this.onMonochromeModeChanged.raiseEvent(mode);
        this._monochromeMode = mode;
        this._json.monochromeMode = mode;
    }
    /** Settings controlling display of the background map within the view. */
    get backgroundMap() { return this._backgroundMap; }
    set backgroundMap(map) {
        if (!this.backgroundMap.equals(map)) {
            this.onBackgroundMapChanged.raiseEvent(map);
            this._backgroundMap = map; // it's an immutable type.
            this._json.backgroundMap = map.toJSON();
        }
    }
    /** Get the map imagery for this display style.  Map imagery includes the background map base as well as background layers and overlay layers.
     * In earlier versions only a background map image was supported as specified by the providerName and mapType members of [[BackgroundMapSettings]] object.
     * In order to provide backward compatibility the original [[BackgroundMapSettings]] are synchronized with the [[MapImagerySettings]] base layer as long as
     * the settings are compatible.  The map imagery typically only should be modified only through  [DisplayStyleState]($frontend) methods.
     * Map imagery should only be modified from backend, changes to map imagery from front end should be handled only through [DisplayStyleState]($frontend) methods.
     * @alpha
     */
    get mapImagery() { return this._mapImagery; }
    set mapImagery(mapImagery) {
        this.onMapImageryChanged.raiseEvent(mapImagery);
        this._mapImagery = mapImagery;
        this._json.mapImagery = this._mapImagery.toJSON();
    }
    /** @internal
     * Handles keeping the map imagery layers in synch after changes have been made (used internally only by front end)
     */
    synchMapImagery() {
        this.onMapImageryChanged.raiseEvent(this._mapImagery);
        this._json.mapImagery = this._mapImagery.toJSON();
    }
    /** The Id of a [RenderTimeline]($backend) element containing a [[RenderSchedule.Script]] used to animate the view. */
    get renderTimeline() {
        return this._json.renderTimeline;
    }
    set renderTimeline(id) {
        this.onRenderTimelineChanged.raiseEvent(id);
        this._json.renderTimeline = id;
    }
    /** @internal @deprecated */
    get scheduleScriptProps() {
        // eslint-disable-next-line deprecation/deprecation
        return this._json.scheduleScript;
    }
    set scheduleScriptProps(props) {
        // eslint-disable-next-line deprecation/deprecation
        this.onScheduleScriptPropsChanged.raiseEvent(props);
        // eslint-disable-next-line deprecation/deprecation
        this._json.scheduleScript = props;
    }
    /** The point in time reflected by the view, in UNIX seconds.
     * This identifies a point on the timeline of the style's [[RenderSchedule.Script]], if any; it may also affect display of four-dimensional reality models.
     */
    get timePoint() {
        return this._json.timePoint;
    }
    set timePoint(timePoint) {
        if (timePoint !== this.timePoint) {
            this.onTimePointChanged.raiseEvent(timePoint);
            this._json.timePoint = timePoint;
        }
    }
    /** Settings controlling the display of analytical models.
     * @see [[analysisFraction]] to control playback of the animation.
     */
    get analysisStyle() { return this._analysisStyle; }
    set analysisStyle(style) {
        if (style === this.analysisStyle)
            return;
        this.onAnalysisStyleChanged.raiseEvent(style);
        this._analysisStyle = style;
        if (style)
            this._json.analysisStyle = style.toJSON();
        else
            delete this._json.analysisStyle;
    }
    /** A value in [0..1] indicating the current point in animation of the [[analysisStyle]], where 0 corresponds to the beginning of
     * the animation and 1 to the end.
     */
    get analysisFraction() {
        var _a;
        const fraction = (_a = this._json.analysisFraction) !== null && _a !== void 0 ? _a : 0;
        return Math.max(0, Math.min(1, fraction));
    }
    set analysisFraction(fraction) {
        if (this.analysisFraction === fraction)
            return;
        this.onAnalysisFractionChanged.raiseEvent(fraction);
        this._json.analysisFraction = Math.max(0, Math.min(1, fraction));
    }
    /** Settings controlling how white-on-white reversal is applied. */
    get whiteOnWhiteReversal() { return this._whiteOnWhiteReversal; }
    set whiteOnWhiteReversal(settings) {
        if (settings.equals(this.whiteOnWhiteReversal))
            return;
        this.onWhiteOnWhiteReversalChanged.raiseEvent(settings);
        this._whiteOnWhiteReversal = settings;
        const json = settings.toJSON();
        if (json)
            this._json.whiteOnWhiteReversal = json;
        else
            delete this._json.whiteOnWhiteReversal;
    }
    /** Customize the way geometry belonging to a [[SubCategory]] is drawn by this display style.
     * @param id The Id of the SubCategory whose appearance is to be overridden.
     * @param ovr The overrides to apply to the [[SubCategoryAppearance]].
     * @see [[dropSubCategoryOverride]]
     */
    overrideSubCategory(id, ovr) {
        this.subCategoryOverrides.set(id, ovr);
    }
    /** Remove any [[SubCategoryOverride]] applied to a [[SubCategoryAppearance]] by this style.
     * @param id The Id of the [[SubCategory]].
     * @see [[overrideSubCategory]]
     */
    dropSubCategoryOverride(id) {
        this.subCategoryOverrides.delete(id);
    }
    /** The overrides applied by this style. */
    get subCategoryOverrides() {
        return this._subCategoryOverrides;
    }
    /** Obtain the override applied to a [[SubCategoryAppearance]] by this style.
     * @param id The Id of the [[SubCategory]].
     * @returns The corresponding SubCategoryOverride, or undefined if the SubCategory's appearance is not overridden.
     * @see [[overrideSubCategory]]
     */
    getSubCategoryOverride(id) {
        return this.subCategoryOverrides.get(id);
    }
    /** Returns true if an [[SubCategoryOverride]]s are defined by this style. */
    get hasSubCategoryOverride() {
        return this.subCategoryOverrides.size > 0;
    }
    /** Customize the way a [Model]($backend)   is drawn by this display style.
     * @param modelId The Id of the [Model]($backend)  whose appearance is to be overridden.
     * @param ovr The overrides to apply to the [Model]($backend) .
     * @see [[dropModelAppearanceOverride]]
     */
    overrideModelAppearance(modelId, ovr) {
        this.modelAppearanceOverrides.set(modelId, ovr);
    }
    /** Remove any appearance overrides applied to a [Model]($backend)  by this style.
     * @param modelId The Id of the [Model]($backend) .
     * @param ovr The overrides to apply to the [Model]($backend) .
     * @see [[overrideModelAppearance]]
     */
    dropModelAppearanceOverride(id) {
        this.modelAppearanceOverrides.delete(id);
    }
    /** The overrides applied by this style. */
    get modelAppearanceOverrides() {
        return this._modelAppearanceOverrides;
    }
    /** Obtain the override applied to a [Model]($backend)  by this style.
     * @param id The Id of the [Model]($backend).
     * @returns The corresponding FeatureAppearance, or undefined if the Model's appearance is not overridden.
     * @see [[overrideModelAppearance]]
     */
    getModelAppearanceOverride(id) {
        return this.modelAppearanceOverrides.get(id);
    }
    /** Returns true if model appearance overrides are defined by this style. */
    get hasModelAppearanceOverride() {
        return this.modelAppearanceOverrides.size > 0;
    }
    /** The set of elements that will not be drawn by this display style.
     * @returns The set of excluded elements.
     * @note The set and the Ids it contains may be very large. It is allocated the first time this property is requested.
     * @deprecated Use [[excludedElementIds]] for better performance and reduced memory overhead, unless efficient lookup of Ids within the set is required.
     */
    get excludedElements() {
        return this._excludedElements.obtainSet();
    }
    /** The set of elements that will not be drawn by this display style.
     * @returns An iterable over the elements' Ids.
     */
    get excludedElementIds() {
        return this._excludedElements;
    }
    /** @internal */
    get compressedExcludedElementIds() {
        return this._excludedElements.ids;
    }
    /** Add one or more elements to the set of elements not to be displayed.
     * @param id The Ids of the element(s) to be excluded.
     */
    addExcludedElements(id) {
        this._excludedElements.add("string" === typeof id ? [id] : id);
        this.onExcludedElementsChanged.raiseEvent();
    }
    /** Remove an element from the set of elements not to be displayed. */
    dropExcludedElement(id) {
        this._excludedElements.delete([id]);
        this.onExcludedElementsChanged.raiseEvent();
    }
    /** Remove one or more elements from the set of elements not to be displayed.
     * @param id The Ids of the element(s) to be removed from the set of excluded elements.
     */
    dropExcludedElements(id) {
        this._excludedElements.delete("string" === typeof id ? [id] : id);
        this.onExcludedElementsChanged.raiseEvent();
    }
    /** Remove all elements from the set of elements not to be displayed. */
    clearExcludedElements() {
        this._excludedElements.reset(undefined);
        this.onExcludedElementsChanged.raiseEvent();
    }
    /** The style applied to the view's [ClipVector]($geometry-core). */
    get clipStyle() {
        return this._clipStyle;
    }
    set clipStyle(style) {
        this.onClipStyleChanged.raiseEvent(style);
        this._clipStyle = style;
        if (style.matchesDefaults)
            delete this._json.clipStyle;
        else
            this._json.clipStyle = style.toJSON();
    }
    /** @internal */
    toJSON() {
        return this._json;
    }
    /** Serialize a subset of these settings to JSON, such that they can be applied to another DisplayStyleSettings to selectively override those settings.
     * @param options Specifies which settings should be serialized. By default, settings that are specific to an iModel (e.g., subcategory overrides) or project (e.g., context reality models)
     * are omitted, as are drawing aids (e.g., ACS triad and grid).
     * @returns a JSON representation of the selected settings suitable for passing to [[applyOverrides]].
     * @see [[applyOverrides]] to apply the overrides to another DisplayStyleSettings..
     */
    toOverrides(options) {
        var _a;
        if (options === null || options === void 0 ? void 0 : options.includeAll) {
            return Object.assign(Object.assign({}, this.toJSON()), { viewflags: this.viewFlags.toFullyDefinedJSON() });
        }
        const viewflags = this.viewFlags.toFullyDefinedJSON();
        const props = {
            viewflags,
            backgroundColor: this.backgroundColor.toJSON(),
            monochromeColor: this.monochromeColor.toJSON(),
            monochromeMode: this.monochromeMode,
            whiteOnWhiteReversal: (_a = this.whiteOnWhiteReversal.toJSON()) !== null && _a !== void 0 ? _a : { ignoreBackgroundColor: false },
        };
        if (options === null || options === void 0 ? void 0 : options.includeBackgroundMap)
            props.backgroundMap = this.backgroundMap.toJSON();
        else
            delete viewflags.backgroundMap;
        if (!(options === null || options === void 0 ? void 0 : options.includeDrawingAids)) {
            delete viewflags.acs;
            delete viewflags.grid;
        }
        if ((options === null || options === void 0 ? void 0 : options.includeProjectSpecific) || (options === null || options === void 0 ? void 0 : options.includeIModelSpecific)) {
            props.timePoint = this.timePoint;
            if (this._json.contextRealityModels) {
                props.contextRealityModels = this._json.contextRealityModels;
                if (!(options === null || options === void 0 ? void 0 : options.includeIModelSpecific))
                    for (const model of this._json.contextRealityModels)
                        delete model.classifiers;
            }
        }
        if (options === null || options === void 0 ? void 0 : options.includeIModelSpecific) {
            if (this.analysisStyle) {
                props.analysisStyle = this.analysisStyle.toJSON();
                props.analysisFraction = this.analysisFraction;
            }
            // eslint-disable-next-line deprecation/deprecation
            if (this.scheduleScriptProps) {
                // eslint-disable-next-line deprecation/deprecation
                props.scheduleScript = [...this.scheduleScriptProps];
            }
            if (this.renderTimeline)
                props.renderTimeline = this.renderTimeline;
            props.subCategoryOvr = this._json.subCategoryOvr ? [...this._json.subCategoryOvr] : [];
            props.modelOvr = this._json.modelOvr ? [...this._json.modelOvr] : [];
            props.excludedElements = this._excludedElements.ids;
        }
        return props;
    }
    /** Selectively override some of these settings. Any field that is explicitly defined by the input will be overridden in these settings; any fields left undefined in the input
     * will retain their current values in these settings. The input's [[ViewFlags]] are applied individually - only those flags that are explicitly defined will be overridden.
     * For example, the following overrides will set the render mode to "smooth", change the background color to white, turn shadows off, and leave all other settings intact:
     * ```ts
     *  {
     *    viewflags: {
     *      renderMode: RenderMode.SmoothShade,
     *      shadows: false,
     *    },
     *    backgroundColor: ColorByName.white,
     *  }
     * ```
     * @see [[toOverrides]] to produce overrides from an existing DisplayStyleSettings.
     */
    applyOverrides(overrides) {
        this._applyOverrides(overrides);
        this.onOverridesApplied.raiseEvent(overrides);
    }
    /** @internal */
    _applyOverrides(overrides) {
        this.onApplyOverrides.raiseEvent(overrides);
        if (overrides.viewflags) {
            this.viewFlags = ViewFlags_1.ViewFlags.fromJSON(Object.assign(Object.assign({}, this.viewFlags.toJSON()), overrides.viewflags));
        }
        if (undefined !== overrides.backgroundColor)
            this.backgroundColor = ColorDef_1.ColorDef.fromJSON(overrides.backgroundColor);
        if (undefined !== overrides.monochromeColor)
            this.monochromeColor = ColorDef_1.ColorDef.fromJSON(overrides.monochromeColor);
        if (undefined !== overrides.monochromeMode)
            this.monochromeMode = overrides.monochromeMode;
        if (overrides.backgroundMap)
            this.backgroundMap = BackgroundMapSettings_1.BackgroundMapSettings.fromJSON(overrides.backgroundMap);
        if (undefined !== overrides.timePoint)
            this.timePoint = overrides.timePoint;
        if (overrides.contextRealityModels) {
            this.contextRealityModels.clear();
            for (const props of overrides.contextRealityModels)
                this.contextRealityModels.add(props);
        }
        if (overrides.analysisStyle)
            this.analysisStyle = AnalysisStyle_1.AnalysisStyle.fromJSON(overrides.analysisStyle);
        if (overrides.whiteOnWhiteReversal)
            this.whiteOnWhiteReversal = WhiteOnWhiteReversalSettings_1.WhiteOnWhiteReversalSettings.fromJSON(overrides.whiteOnWhiteReversal);
        if (undefined !== overrides.analysisFraction)
            this.analysisFraction = overrides.analysisFraction;
        // eslint-disable-next-line deprecation/deprecation
        if (overrides.scheduleScript) {
            // eslint-disable-next-line deprecation/deprecation
            this.scheduleScriptProps = [...overrides.scheduleScript];
        }
        if (overrides.renderTimeline)
            this.renderTimeline = overrides.renderTimeline;
        if (overrides.subCategoryOvr) {
            this._json.subCategoryOvr = [...overrides.subCategoryOvr];
            this._subCategoryOverrides.populate();
        }
        if (overrides.modelOvr) {
            this._json.modelOvr = [...overrides.modelOvr];
            this._modelAppearanceOverrides.populate();
        }
        if (overrides.excludedElements)
            this._excludedElements.reset("string" === typeof overrides.excludedElements ? overrides.excludedElements : [...overrides.excludedElements]);
        this.onOverridesApplied.raiseEvent(overrides);
    }
}
exports.DisplayStyleSettings = DisplayStyleSettings;
/** Provides access to the settings defined by a [[DisplayStyle3d]] or [[DisplayStyle3dState]], and ensures that
 * the style's JSON properties are kept in sync.
 * @public
 */
class DisplayStyle3dSettings extends DisplayStyleSettings {
    constructor(jsonProperties, options) {
        var _a;
        super(jsonProperties, options);
        this._thematic = ThematicDisplay_1.ThematicDisplay.fromJSON(this._json3d.thematic);
        this._hline = HiddenLine_1.HiddenLine.Settings.fromJSON(this._json3d.hline);
        this._ao = AmbientOcclusion_1.AmbientOcclusion.Settings.fromJSON(this._json3d.ao);
        this._solarShadows = SolarShadows_1.SolarShadowSettings.fromJSON(this._json3d.solarShadows);
        // Very long ago we used to stick MicroStation's light settings into json.sceneLights. Later we started adding the sunDir.
        // We don't want any of MicroStation's settings. We do want to preserve the sunDir if present.
        if (this._json3d.lights) {
            this._lights = LightSettings_1.LightSettings.fromJSON(this._json3d.lights);
        }
        else {
            const sunDir = (_a = this._json3d.sceneLights) === null || _a === void 0 ? void 0 : _a.sunDir; // eslint-disable-line deprecation/deprecation
            this._lights = LightSettings_1.LightSettings.fromJSON(sunDir ? { solar: { direction: sunDir } } : undefined);
        }
        this.populatePlanProjectionsFromJSON();
    }
    get _json3d() { return this._json; }
    is3d() {
        return true;
    }
    populatePlanProjectionsFromJSON() {
        this._planProjections = undefined;
        const projections = this._json3d.planProjections;
        if (undefined !== projections) {
            for (const key of Object.keys(projections)) {
                const id = bentleyjs_core_1.Id64.fromJSON(key);
                if (!bentleyjs_core_1.Id64.isValidId64(id)) {
                    delete projections[key];
                    continue;
                }
                const settings = PlanProjectionSettings_1.PlanProjectionSettings.fromJSON(projections[key]);
                if (undefined === settings) {
                    delete projections[key];
                    continue;
                }
                if (undefined === this._planProjections)
                    this._planProjections = new Map();
                this._planProjections.set(id, settings);
            }
        }
    }
    /** @internal */
    toJSON() {
        return this._json3d;
    }
    /** @internal */
    toOverrides(options) {
        const props = super.toOverrides(options);
        if (options === null || options === void 0 ? void 0 : options.includeAll)
            return props;
        bentleyjs_core_1.assert(undefined !== props.viewflags);
        props.environment = Object.assign({}, this.environment);
        props.hline = this.hiddenLineSettings.toJSON();
        props.ao = this.ambientOcclusionSettings.toJSON();
        props.solarShadows = this.solarShadows.toJSON();
        props.lights = this.lights.toJSON();
        if (options === null || options === void 0 ? void 0 : options.includeIModelSpecific) {
            props.thematic = this.thematic.toJSON();
            if (this._json3d.planProjections)
                props.planProjections = Object.assign({}, this._json3d.planProjections);
        }
        else if (ThematicDisplay_1.ThematicDisplayMode.InverseDistanceWeightedSensors !== this.thematic.displayMode) {
            props.thematic = Object.assign(Object.assign({}, this.thematic.toJSON()), { sensorSettings: undefined });
            if (ThematicDisplay_1.ThematicDisplayMode.Height === props.thematic.displayMode) {
                // DisplayStyle3dState will compute range based on project extents.
                props.thematic.range = undefined;
            }
        }
        return props;
    }
    /** Selectively override some of these settings. Any field that is explicitly defined by the input will be overridden in these settings; any fields left undefined in the input
     * will retain their current values in these settings. The input's [[ViewFlags]] are applied individually - only those flags that are explicitly defined will be overridden.
     * For example, the following overrides will set the render mode to "smooth", change the background color to white, turn shadows off, and leave all other settings intact:
     * ```ts
     *  {
     *    viewflags: {
     *      renderMode: RenderMode.SmoothShade,
     *      shadows: false,
     *    },
     *    backgroundColor: ColorByName.white,
     *  }
     * ```
     * @see [[toOverrides]] to produce overrides from an existing DisplayStyleSettings.
     * @internal override
     */
    applyOverrides(overrides) {
        super._applyOverrides(overrides);
        if (overrides.environment)
            this.environment = Object.assign({}, overrides.environment);
        if (overrides.hline)
            this.hiddenLineSettings = HiddenLine_1.HiddenLine.Settings.fromJSON(overrides.hline);
        if (overrides.ao)
            this.ambientOcclusionSettings = AmbientOcclusion_1.AmbientOcclusion.Settings.fromJSON(overrides.ao);
        if (overrides.solarShadows)
            this.solarShadows = SolarShadows_1.SolarShadowSettings.fromJSON(overrides.solarShadows);
        if (overrides.lights)
            this.lights = LightSettings_1.LightSettings.fromJSON(overrides.lights);
        if (overrides.planProjections) {
            this._json3d.planProjections = Object.assign({}, overrides.planProjections);
            this.populatePlanProjectionsFromJSON();
        }
        if (overrides.thematic)
            this.thematic = ThematicDisplay_1.ThematicDisplay.fromJSON(overrides.thematic);
        this.onOverridesApplied.raiseEvent(overrides);
    }
    /** The settings that control thematic display. */
    get thematic() { return this._thematic; }
    set thematic(thematic) {
        if (thematic.equals(this.thematic))
            return;
        this.onThematicChanged.raiseEvent(thematic);
        this._thematic = thematic;
        this._json3d.thematic = thematic.toJSON();
    }
    /** The settings that control how visible and hidden edges are displayed.  */
    get hiddenLineSettings() { return this._hline; }
    set hiddenLineSettings(hline) {
        if (hline.equals(this.hiddenLineSettings))
            return;
        this.onHiddenLineSettingsChanged.raiseEvent(hline);
        this._hline = hline;
        this._json3d.hline = hline.toJSON();
    }
    /** The settings that control how ambient occlusion is displayed. */
    get ambientOcclusionSettings() { return this._ao; }
    set ambientOcclusionSettings(ao) {
        this.onAmbientOcclusionSettingsChanged.raiseEvent(ao);
        this._ao = ao;
        this._json3d.ao = ao.toJSON();
    }
    /** The settings that control how solar shadows are displayed. */
    get solarShadows() {
        return this._solarShadows;
    }
    set solarShadows(solarShadows) {
        if (solarShadows.equals(this.solarShadows))
            return;
        this.onSolarShadowsChanged.raiseEvent(solarShadows);
        this._solarShadows = solarShadows;
        const json = solarShadows.toJSON();
        if (!json)
            delete this._json3d.solarShadows;
        else
            this._json3d.solarShadows = json;
    }
    /** @internal */
    get environment() {
        const env = this._json3d.environment;
        return undefined !== env ? env : {};
    }
    set environment(environment) {
        this.onEnvironmentChanged.raiseEvent(environment);
        this._json3d.environment = environment;
    }
    get lights() {
        return this._lights;
    }
    set lights(lights) {
        if (this.lights.equals(lights))
            return;
        this.onLightsChanged.raiseEvent(lights);
        this._lights = lights;
        this._json3d.lights = lights.toJSON();
    }
    /** Adjust the solar light direction based on a date and time at a geographic location.
     * This replaces `this.lights` with a copy that records the time point and the computed direction.
     * @param timePoint The time in UNIX milliseconds.
     * @param location The geographic location; or an iModel, in which case the iModel's [[EcefLocation]] is used.
     * @see [[sunTime]] to get the current sun time.
     * @see [[clearSunTime]] to clear the time point.
     * @note If `location` is an iModel lacking an EcefLocation, a location in Exton, Pennsylvania will be used to compute the light direction instead.
     */
    setSunTime(timePoint, location) {
        let cartoCenter;
        if (location instanceof IModel_1.IModel) {
            if (location.ecefLocation)
                cartoCenter = Cartographic_1.Cartographic.fromEcef(location.ecefLocation.origin);
            if (!cartoCenter)
                cartoCenter = Cartographic_1.Cartographic.fromDegrees(-75.17035, 39.954927, 0.0);
        }
        else {
            cartoCenter = location;
        }
        const direction = SolarCalculate_1.calculateSolarDirection(new Date(timePoint), cartoCenter);
        this.lights = this.lights.clone({ solar: { direction, timePoint } });
    }
    /** Clear the solar time point stored in `this.lights.solarLight`.
     * @note This does not affect the solar light direction.
     * @see [[sunTime]] to get the current sun time.
     * @see [[setSunTime]] to set the time point and the solar light direction derived from it.
     */
    clearSunTime() {
        var _a;
        if (this.lights.solar.timePoint === undefined)
            return;
        const solar = (_a = this.lights.solar.toJSON()) !== null && _a !== void 0 ? _a : {};
        solar.timePoint = undefined;
        this.lights = this.lights.clone({ solar });
    }
    /** The time point from which the solar light direction was derived, in UNIX milliseconds.
     * @see [[setSunTime]] to change the time point and solar direction.
     * @see [[clearSunTime]] to reset the time point to `undefined`.
     */
    get sunTime() {
        return this.lights.solar.timePoint;
    }
    /** Get the plan projection settings associated with the specified model, if defined. */
    getPlanProjectionSettings(modelId) {
        return undefined !== this._planProjections ? this._planProjections.get(modelId) : undefined;
    }
    /** Set or clear the plan projection settings associated with the specified model. */
    setPlanProjectionSettings(modelId, settings) {
        this.onPlanProjectionSettingsChanged.raiseEvent(modelId, settings);
        if (undefined === settings) {
            if (undefined !== this._planProjections) {
                bentleyjs_core_1.assert(undefined !== this._json3d.planProjections);
                this._planProjections.delete(modelId);
                delete this._json3d.planProjections[modelId];
                if (0 === this._planProjections.size) {
                    this._planProjections = undefined;
                    delete this._json3d.planProjections;
                }
            }
            return;
        }
        if (undefined === this._planProjections) {
            this._planProjections = new Map();
            this._json3d.planProjections = {};
        }
        this._planProjections.set(modelId, settings);
        this._json3d.planProjections[modelId] = settings.toJSON();
    }
    /** An iterator over all of the defined plan projection settings. The iterator includes the Id of the model associated with each settings object. */
    get planProjectionSettings() {
        return undefined !== this._planProjections ? this._planProjections.entries() : undefined;
    }
}
exports.DisplayStyle3dSettings = DisplayStyle3dSettings;
//# sourceMappingURL=DisplayStyleSettings.js.map