"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FeatureGates = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
/** A set of "gates" that can enable or disable features at runtime.
 * @internal
 */
class FeatureGates {
    constructor() {
        /** Event raised every time any feature changes. */
        this.onChanged = new bentleyjs_core_1.BeEvent();
        /** A map of the current set of features. */
        this.gates = new Map();
    }
    /** Get the value of a potentially gated feature.
     * @param feature The name of the feature to check. May be a "path" of period-separated feature sub-groups (e.g. "feature1.groupA.showMe").
     *       Feature names are case-sensitive.
     * @param defaultVal Optionally, value to return if feature is undefined.
     */
    check(feature, defaultVal) {
        const val = this.gates.get(feature);
        return val === undefined ? defaultVal : val;
    }
    /** Gate access to a feature.
     * @param feature The name of the feature to gate. May be a "path" of period-separated feature sub-groups (e.g. "feature1.groupA.showMe").
     *  Feature names are case-sensitive.
     * @param val Value to set. If undefined, feature is deleted.
     */
    setGate(feature, val) {
        if (feature.length === 0)
            return;
        if (val === undefined)
            this.gates.delete(feature);
        else
            this.gates.set(feature, val);
        this.onChanged.raiseEvent(feature, val);
    }
    /** Register a listener to be called whenever the value of a specific gate changes.
     * @param feature The name of the feature to monitor
     * @param monitor The listener to call when `feature` changes. Receives a single argument holding the new value of the feature (may be undefined).
     * @returns A function that may be called to remove the listener.
     * @note Use [[onChanged]] to listen to changes for *all* gates.
     */
    addMonitor(feature, monitor) { return this.onChanged.addListener((changed, val) => { if (changed === feature)
        monitor(val); }); }
}
exports.FeatureGates = FeatureGates;
//# sourceMappingURL=FeatureGates.js.map