"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.GroundPlane = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ColorByName_1 = require("./ColorByName");
const ColorDef_1 = require("./ColorDef");
const Gradient_1 = require("./Gradient");
/** A circle drawn at a Z elevation, whose diameter is the the XY diagonal of the project extents, used to represent the ground as a reference point within a spatial view.
 * @public
 */
class GroundPlane {
    constructor(ground) {
        /** Whether the ground plane should be displayed. */
        this.display = false;
        /** The Z height at which to draw the plane. */
        this.elevation = 0.0;
        ground = ground ? ground : {};
        this.display = bentleyjs_core_1.JsonUtils.asBool(ground.display, false);
        this.elevation = bentleyjs_core_1.JsonUtils.asDouble(ground.elevation, -.01);
        this.aboveColor = (undefined !== ground.aboveColor) ? ColorDef_1.ColorDef.fromJSON(ground.aboveColor) : ColorDef_1.ColorDef.fromTbgr(ColorByName_1.ColorByName.darkGreen);
        this.belowColor = (undefined !== ground.belowColor) ? ColorDef_1.ColorDef.fromJSON(ground.belowColor) : ColorDef_1.ColorDef.fromTbgr(ColorByName_1.ColorByName.darkBrown);
    }
    toJSON() {
        return {
            display: this.display,
            elevation: this.elevation,
            aboveColor: this.aboveColor.toJSON(),
            belowColor: this.belowColor.toJSON(),
        };
    }
    /** Returns and locally stores gradient symbology for the ground plane texture depending on whether we are looking from above or below.
     * Will store the ground colors used in the optional ColorDef array provided.
     * @internal
     */
    getGroundPlaneGradient(aboveGround) {
        let gradient = aboveGround ? this._aboveSymb : this._belowSymb;
        if (undefined !== gradient)
            return gradient;
        const values = [0, .25, .5]; // gradient goes from edge of rectangle (0.0) to center (1.0)...
        const color = aboveGround ? this.aboveColor : this.belowColor;
        const alpha = aboveGround ? 0x80 : 0x85;
        const groundColors = [color.withTransparency(0xff), color, color];
        groundColors[1] = groundColors[2] = color.withTransparency(alpha);
        // Get the possibly cached gradient from the system, specific to whether or not we want ground from above or below.
        gradient = new Gradient_1.Gradient.Symb();
        gradient.mode = Gradient_1.Gradient.Mode.Spherical;
        gradient.keys = [{ color: groundColors[0], value: values[0] }, { color: groundColors[1], value: values[1] }, { color: groundColors[2], value: values[2] }];
        // Store the gradient for possible future use
        if (aboveGround)
            this._aboveSymb = gradient;
        else
            this._belowSymb = gradient;
        return gradient;
    }
}
exports.GroundPlane = GroundPlane;
//# sourceMappingURL=GroundPlane.js.map