"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModels
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModel = exports.EcefLocation = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const geometry_core_1 = require("@bentley/geometry-core");
const Cartographic_1 = require("./geometry/Cartographic");
const CoordinateReferenceSystem_1 = require("./geometry/CoordinateReferenceSystem");
const IModelError_1 = require("./IModelError");
/** The position and orientation of an iModel on the earth in [ECEF](https://en.wikipedia.org/wiki/ECEF) (Earth Centered Earth Fixed) coordinates
 * @see [GeoLocation of iModels]($docs/learning/GeoLocation.md)
 * @public
 */
class EcefLocation {
    /** Construct a new EcefLocation. Once constructed, it is frozen and cannot be modified. */
    constructor(props) {
        this.origin = geometry_core_1.Point3d.fromJSON(props.origin).freeze();
        this.orientation = geometry_core_1.YawPitchRollAngles.fromJSON(props.orientation).freeze();
        if (props.cartographicOrigin)
            this.cartographicOrigin = Cartographic_1.Cartographic.fromRadians(props.cartographicOrigin.longitude, props.cartographicOrigin.latitude, props.cartographicOrigin.height).freeze();
        if (props.xVector && props.yVector) {
            this.xVector = geometry_core_1.Vector3d.fromJSON(props.xVector);
            this.yVector = geometry_core_1.Vector3d.fromJSON(props.yVector);
        }
        let matrix;
        if (this.xVector && this.yVector) {
            const zVector = this.xVector.crossProduct(this.yVector);
            if (zVector.normalizeInPlace())
                matrix = geometry_core_1.Matrix3d.createColumns(this.xVector, this.yVector, zVector);
        }
        if (!matrix)
            matrix = this.orientation.toMatrix3d();
        this._transform = geometry_core_1.Transform.createOriginAndMatrix(this.origin, matrix);
    }
    /** Get the transform from iModel Spatial coordinates to ECEF from this EcefLocation */
    getTransform() { return this._transform; }
    /** Construct ECEF Location from cartographic origin with optional known point and angle.   */
    static createFromCartographicOrigin(origin, point, angle) {
        const ecefOrigin = origin.toEcef();
        const deltaRadians = 10 / geometry_core_1.Constant.earthRadiusWGS84.polar;
        const northCarto = Cartographic_1.Cartographic.fromRadians(origin.longitude, origin.latitude + deltaRadians, origin.height);
        const eastCarto = Cartographic_1.Cartographic.fromRadians(origin.longitude + deltaRadians, origin.latitude, origin.height);
        const ecefNorth = northCarto.toEcef();
        const ecefEast = eastCarto.toEcef();
        const xVector = geometry_core_1.Vector3d.createStartEnd(ecefOrigin, ecefEast).normalize();
        const yVector = geometry_core_1.Vector3d.createStartEnd(ecefOrigin, ecefNorth).normalize();
        const matrix = geometry_core_1.Matrix3d.createRigidFromColumns(xVector, yVector, geometry_core_1.AxisOrder.XYZ);
        if (angle !== undefined) {
            const north = geometry_core_1.Matrix3d.createRotationAroundAxisIndex(geometry_core_1.AxisIndex.Z, angle);
            matrix.multiplyMatrixMatrix(north, matrix);
        }
        if (point !== undefined) {
            const delta = matrix.multiplyVector(geometry_core_1.Vector3d.create(-point.x, -point.y, -point.z));
            ecefOrigin.addInPlace(delta);
        }
        return new EcefLocation({ origin: ecefOrigin, orientation: geometry_core_1.YawPitchRollAngles.createFromMatrix3d(matrix), cartographicOrigin: origin });
    }
    /** Get the location center of the earth in the iModel coordinate system. */
    get earthCenter() {
        const matrix = this.orientation.toMatrix3d();
        return geometry_core_1.Point3d.createFrom(matrix.multiplyTransposeXYZ(-this.origin.x, -this.origin.y, -this.origin.z));
    }
    /** Return true if this location is equivalent to another location within a small tolerance. */
    isAlmostEqual(other) {
        if (!this.origin.isAlmostEqual(other.origin) || !this.orientation.isAlmostEqual(other.orientation))
            return false;
        if ((this.xVector === undefined) !== (other.xVector === undefined) || (this.yVector === undefined) !== (other.yVector === undefined))
            return false;
        if (this.xVector !== undefined && other.xVector !== undefined && !this.xVector.isAlmostEqual(other.xVector))
            return false;
        if (this.yVector !== undefined && other.yVector !== undefined && !this.yVector.isAlmostEqual(other.yVector))
            return false;
        const thisCarto = this.cartographicOrigin;
        const otherCarto = other.cartographicOrigin;
        if (undefined === thisCarto || undefined === otherCarto)
            return undefined === thisCarto && undefined === otherCarto;
        return thisCarto.equalsEpsilon(otherCarto, geometry_core_1.Geometry.smallMetricDistance);
    }
    toJSON() {
        const props = {
            origin: this.origin.toJSON(),
            orientation: this.orientation.toJSON(),
        };
        if (this.cartographicOrigin)
            props.cartographicOrigin = this.cartographicOrigin.toJSON();
        if (this.xVector)
            props.xVector = this.xVector.toJSON();
        if (this.yVector)
            props.yVector = this.yVector.toJSON();
        return props;
    }
}
exports.EcefLocation = EcefLocation;
/** Represents an iModel in JavaScript.
 * @see [GeoLocation of iModels]($docs/learning/GeoLocation.md)
 * @public
 */
class IModel {
    /** @internal */
    constructor(tokenProps, openMode) {
        /** Event raised after [[name]] changes. */
        this.onNameChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised after [[rootSubject]] changes. */
        this.onRootSubjectChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised after [[projectExtents]] changes. */
        this.onProjectExtentsChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised after [[globalOrigin]] changes. */
        this.onGlobalOriginChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised after [[ecefLocation]] changes. */
        this.onEcefLocationChanged = new bentleyjs_core_1.BeEvent();
        /** Event raised after [[geographicCoordinateSystem]] changes. */
        this.onGeographicCoordinateSystemChanged = new bentleyjs_core_1.BeEvent();
        this.changeset = { id: "", index: 0 };
        this._fileKey = "";
        if (tokenProps) {
            this._fileKey = tokenProps.key;
            this._contextId = tokenProps.contextId;
            this._iModelId = tokenProps.iModelId;
            if (tokenProps.changeSetId)
                this.changeset = { id: tokenProps.changeSetId, index: tokenProps.changesetIndex };
        }
        this.openMode = openMode;
    }
    /** Name of the iModel */
    get name() {
        bentleyjs_core_1.assert(this._name !== undefined);
        return this._name;
    }
    set name(name) {
        if (name !== this._name) {
            const old = this._name;
            this._name = name;
            if (undefined !== old)
                this.onNameChanged.raiseEvent(old);
        }
    }
    /** The name and description of the root subject of this iModel */
    get rootSubject() {
        bentleyjs_core_1.assert(this._rootSubject !== undefined);
        return this._rootSubject;
    }
    set rootSubject(subject) {
        if (undefined === this._rootSubject || this._rootSubject.name !== subject.name || this._rootSubject.description !== subject.description) {
            const old = this._rootSubject;
            this._rootSubject = subject;
            if (old)
                this.onRootSubjectChanged.raiseEvent(old);
        }
    }
    /**
     * The volume, in spatial coordinates, inside which the entire project is contained.
     * @note The object returned from this method is frozen. You *must* make a copy before you do anything that might attempt to modify it.
     */
    get projectExtents() {
        bentleyjs_core_1.assert(undefined !== this._projectExtents);
        return this._projectExtents;
    }
    set projectExtents(extents) {
        // Don't allow any axis of the project extents to be less than 1 meter.
        const projectExtents = extents.clone();
        projectExtents.ensureMinLengths(1.0);
        if (!this._projectExtents || !this._projectExtents.isAlmostEqual(projectExtents)) {
            const old = this._projectExtents;
            projectExtents.freeze();
            this._projectExtents = projectExtents;
            if (old)
                this.onProjectExtentsChanged.raiseEvent(old);
        }
    }
    /** An offset to be applied to all spatial coordinates. */
    get globalOrigin() {
        bentleyjs_core_1.assert(this._globalOrigin !== undefined);
        return this._globalOrigin;
    }
    set globalOrigin(org) {
        if (!this._globalOrigin || !this._globalOrigin.isAlmostEqual(org)) {
            const old = this._globalOrigin;
            org.freeze();
            this._globalOrigin = org;
            if (old)
                this.onGlobalOriginChanged.raiseEvent(old);
        }
    }
    /** The [EcefLocation]($docs/learning/glossary#ecefLocation) of the iModel in Earth Centered Earth Fixed coordinates. */
    get ecefLocation() {
        return this._ecefLocation;
    }
    set ecefLocation(ecefLocation) {
        const old = this._ecefLocation;
        if (!old && !ecefLocation)
            return;
        else if (old && ecefLocation && old.isAlmostEqual(ecefLocation))
            return;
        this._ecefLocation = ecefLocation;
        this.onEcefLocationChanged.raiseEvent(old);
    }
    /** Set the [EcefLocation]($docs/learning/glossary#ecefLocation) for this iModel. */
    setEcefLocation(ecef) {
        this.ecefLocation = new EcefLocation(ecef);
    }
    /** The geographic coordinate reference system of the iModel. */
    get geographicCoordinateSystem() {
        return this._geographicCoordinateSystem;
    }
    set geographicCoordinateSystem(geoCRS) {
        const old = this._geographicCoordinateSystem;
        if (!old && !geoCRS)
            return;
        else if (old && geoCRS && old.equals(geoCRS))
            return;
        this._geographicCoordinateSystem = geoCRS;
        this.onGeographicCoordinateSystemChanged.raiseEvent(old);
    }
    /** Sets the geographic coordinate reference system from GeographicCRSProps. */
    setGeographicCoordinateSystem(geoCRS) {
        this.geographicCoordinateSystem = new CoordinateReferenceSystem_1.GeographicCRS(geoCRS);
    }
    /** @internal */
    getConnectionProps() {
        return Object.assign({ name: this.name, rootSubject: this.rootSubject, projectExtents: this.projectExtents.toJSON(), globalOrigin: this.globalOrigin.toJSON(), ecefLocation: this.ecefLocation, geographicCoordinateSystem: this.geographicCoordinateSystem }, this.getRpcProps());
    }
    /** @internal */
    toJSON() {
        return this.getConnectionProps();
    }
    /** Get the key that was used to open this iModel. This is the value used for Rpc and Ipc communications. */
    get key() { return this._fileKey; }
    /** The Guid that identifies the *context* that owns this iModel. */
    get contextId() { return this._contextId; }
    /** The Guid that identifies this iModel. */
    get iModelId() { return this._iModelId; }
    /** The Id of the last changeset that was applied to this iModel.
     * @note An empty string indicates the first version
     * @deprecated use changeset.id
     */
    get changeSetId() { return this.changeset.id; }
    /** Return a token for RPC operations. */
    getRpcProps() {
        if (!this.isOpen)
            throw new IModelError_1.IModelError(bentleyjs_core_1.IModelStatus.BadRequest, "IModel is not open for rpc", bentleyjs_core_1.Logger.logError);
        return {
            key: this._fileKey,
            contextId: this.contextId,
            iModelId: this.iModelId,
            changeSetId: this.changeset.id,
            changesetIndex: this.changeset.index,
            openMode: this.openMode,
        };
    }
    /** @internal */
    initialize(name, props) {
        this.name = name;
        this.rootSubject = props.rootSubject;
        this.projectExtents = geometry_core_1.Range3d.fromJSON(props.projectExtents);
        this.globalOrigin = geometry_core_1.Point3d.fromJSON(props.globalOrigin);
        this.ecefLocation = props.ecefLocation ? new EcefLocation(props.ecefLocation) : undefined;
        this.geographicCoordinateSystem = props.geographicCoordinateSystem ? new CoordinateReferenceSystem_1.GeographicCRS(props.geographicCoordinateSystem) : undefined;
    }
    /** Get the default subCategoryId for the supplied categoryId */
    static getDefaultSubCategoryId(categoryId) {
        return bentleyjs_core_1.Id64.isValid(categoryId) ? bentleyjs_core_1.Id64.fromLocalAndBriefcaseIds(bentleyjs_core_1.Id64.getLocalId(categoryId) + 1, bentleyjs_core_1.Id64.getBriefcaseId(categoryId)) : bentleyjs_core_1.Id64.invalid;
    }
    /** True if this iModel has an [EcefLocation]($docs/learning/glossary#ecefLocation). */
    get isGeoLocated() { return undefined !== this._ecefLocation; }
    /** Get the Transform from this iModel's Spatial coordinates to ECEF coordinates using its [[IModel.ecefLocation]].
     * @throws IModelError if [[isGeoLocated]] is false.
     */
    getEcefTransform() {
        if (undefined === this._ecefLocation)
            throw new IModelError_1.IModelError(bentleyjs_core_1.GeoServiceStatus.NoGeoLocation, "iModel is not GeoLocated");
        if (this._ecefTrans === undefined) {
            this._ecefTrans = this._ecefLocation.getTransform();
            this._ecefTrans.freeze();
        }
        return this._ecefTrans;
    }
    /** Convert a point in this iModel's Spatial coordinates to an ECEF point using its [[IModel.ecefLocation]].
     * @param spatial A point in the iModel's spatial coordinates
     * @param result If defined, use this for output
     * @returns A Point3d in ECEF coordinates
     * @throws IModelError if [[isGeoLocated]] is false.
     */
    spatialToEcef(spatial, result) { return this.getEcefTransform().multiplyPoint3d(spatial, result); }
    /** Convert a point in ECEF coordinates to a point in this iModel's Spatial coordinates using its [[ecefLocation]].
     * @param ecef A point in ECEF coordinates
     * @param result If defined, use this for output
     * @returns A Point3d in this iModel's spatial coordinates
     * @throws IModelError if [[isGeoLocated]] is false.
     * @note The resultant point will only be meaningful if the ECEF coordinate is close on the earth to the iModel.
     */
    ecefToSpatial(ecef, result) { return this.getEcefTransform().multiplyInversePoint3d(ecef, result); }
    /** Convert a point in this iModel's Spatial coordinates to a [[Cartographic]] using its [[IModel.ecefLocation]].
     * @param spatial A point in the iModel's spatial coordinates
     * @param result If defined, use this for output
     * @returns A Cartographic location
     * @throws IModelError if [[isGeoLocated]] is false.
     */
    spatialToCartographicFromEcef(spatial, result) { return Cartographic_1.Cartographic.fromEcef(this.spatialToEcef(spatial), result); }
    /** Convert a [[Cartographic]] to a point in this iModel's Spatial coordinates using its [[IModel.ecefLocation]].
     * @param cartographic A cartographic location
     * @param result If defined, use this for output
     * @returns A point in this iModel's spatial coordinates
     * @throws IModelError if [[isGeoLocated]] is false.
     * @note The resultant point will only be meaningful if the ECEF coordinate is close on the earth to the iModel.
     */
    cartographicToSpatialFromEcef(cartographic, result) { return this.ecefToSpatial(cartographic.toEcef(result), result); }
}
exports.IModel = IModel;
/** The Id of the repository model. */
IModel.repositoryModelId = "0x1";
/** The Id of the root subject element. */
IModel.rootSubjectId = "0x1";
/** The Id of the dictionary model. */
IModel.dictionaryId = "0x10";
//# sourceMappingURL=IModel.js.map