"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModels
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelVersion = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const imodelhub_client_1 = require("@bentley/imodelhub-client");
const IModelError_1 = require("./IModelError");
/** Option to specify the version of the iModel to be acquired and used
 * @public
 */
class IModelVersion {
    constructor() { }
    /** Describes the first version of the iModel */
    static first() {
        const version = new IModelVersion();
        version._first = true;
        return version;
    }
    /** Describes the latest version of the iModel  */
    static latest() {
        const version = new IModelVersion();
        version._latest = true;
        return version;
    }
    /** Describes a version of the iModel by the last change set that needs
     * to be applied or merged to the iModel.
     * Note that all ChangeSets up to and and including the specified ChangeSet
     * needs to be applied.
     * If the changeSetId is an empty string, it is assumed to be the first version
     * before any change sets have been applied.
     */
    static asOfChangeSet(changeSetId) {
        const version = new IModelVersion();
        if (changeSetId === "") {
            version._first = true;
            return version;
        }
        version._afterChangeSetId = changeSetId;
        return version;
    }
    /** Describes a version of the iModel with the specified version name  */
    static named(versionName) {
        const version = new IModelVersion();
        version._versionName = versionName;
        return version;
    }
    toJSON() {
        return this._versionName ? { versionName: this._versionName } :
            this._afterChangeSetId ? { afterChangeSetId: this._afterChangeSetId } :
                this._first ? { first: this._first } :
                    { latest: true };
    }
    /** Creates a version from an IModelVersionProps */
    static fromJSON(json) {
        const version = new IModelVersion();
        version._first = json.first;
        version._afterChangeSetId = json.afterChangeSetId;
        version._latest = json.latest;
        version._versionName = json.versionName;
        return version;
    }
    /** Creates a version from an untyped JSON object
     * @deprecated use fromJSON
    */
    static fromJson(jsonObj) {
        const version = new IModelVersion();
        Object.assign(version, jsonObj);
        return version;
    }
    /** Returns true if this describes the first version */
    get isFirst() { return !!this._first; }
    /** Returns true if this describes the latest version */
    get isLatest() { return !!this._latest; }
    /** Returns the last change set id to be applied to the iModel
     * to get to this specified version. @see asOfChangeSet().
     * Note that this method does not attempt to resolve the change set
     * if this describes the first version, last version, named version, etc.
     * @see evaluateChangeSet() for those use cases.
     */
    getAsOfChangeSet() { return this._afterChangeSetId; }
    /** Returns the name of the version if this describes a named version. @see named() */
    getName() { return this._versionName; }
    /** Evaluate the ChangeSet Id corresponding to the version. All change sets up to and including
     * the returned ChangeSet Id need to be applied to update the iModel to this version.
     * Returns an empty string if this contains the first version (before any change sets). If the
     * version was already specified as of a ChangeSet, the method simply returns
     * that Id without any validation.
     * @deprecated use IModelHost/IModelApp hubAccess.getChangesetIdFromVersion
     */
    async evaluateChangeSet(requestContext, iModelId, imodelClient) {
        if (this._first)
            return "";
        if (this._afterChangeSetId)
            return this._afterChangeSetId;
        if (this._latest)
            return IModelVersion.getLatestChangeSetId(requestContext, imodelClient, iModelId); // eslint-disable-line deprecation/deprecation
        if (this._versionName)
            return IModelVersion.getChangeSetFromNamedVersion(requestContext, imodelClient, iModelId, this._versionName); // eslint-disable-line deprecation/deprecation
        throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Invalid version");
    }
    /** Gets the last change set that was applied to the imodel
     * @internal - public only so tests can catch calls to it and fail.
     * @deprecated use IModelHost/IModelApp hubAccess.getChangesetIdFromVersion
     */
    static async getLatestChangeSetId(requestContext, imodelClient, iModelId) {
        const changeSets = await imodelClient.changeSets.get(requestContext, iModelId, new imodelhub_client_1.ChangeSetQuery().top(1).latest());
        return (changeSets.length === 0) ? "" : changeSets[changeSets.length - 1].wsgId;
    }
    /** Get the change set from the specified named version
     * @internal - public only so tests can catch calls to it and fail.
     * @deprecated use IModelHost/IModelApp hubAccess.getChangesetIdFromVersion
     */
    static async getChangeSetFromNamedVersion(requestContext, imodelClient, iModelId, versionName) {
        const versions = await imodelClient.versions.get(requestContext, iModelId, new imodelhub_client_1.VersionQuery().select("ChangeSetId").byName(versionName));
        if (!versions[0] || !versions[0].changeSetId)
            throw new IModelError_1.IModelError(bentleyjs_core_1.IModelStatus.NotFound, `Named version ${versionName} not found`);
        return versions[0].changeSetId;
    }
}
exports.IModelVersion = IModelVersion;
//# sourceMappingURL=IModelVersion.js.map