"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.LightSettings = exports.HemisphereLights = exports.AmbientLight = exports.SolarLight = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const geometry_core_1 = require("@bentley/geometry-core");
const RgbColor_1 = require("./RgbColor");
function extractIntensity(value, defaultValue) {
    const maxIntensity = 5;
    return typeof value === "number" ? Math.max(0, Math.min(maxIntensity, value)) : defaultValue;
}
const defaultSolarDirection = geometry_core_1.Vector3d.create(0.272166, 0.680414, 0.680414);
/** Describes the solar directional light associated with a [[LightSettings]].
 * @see [[SolarLightProps]].
 * @public
 */
class SolarLight {
    constructor(json) {
        json = json || {};
        this.intensity = extractIntensity(json.intensity, 1);
        this.alwaysEnabled = bentleyjs_core_1.JsonUtils.asBool(json.alwaysEnabled);
        if (json.direction)
            this.direction = geometry_core_1.Vector3d.fromJSON(json.direction);
        else
            this.direction = defaultSolarDirection.clone();
        if (typeof json.timePoint === "number")
            this.timePoint = json.timePoint;
    }
    toJSON() {
        const direction = this.direction.isAlmostEqual(defaultSolarDirection) ? undefined : this.direction.toJSON();
        const intensity = this.intensity !== 1 ? this.intensity : undefined;
        const alwaysEnabled = this.alwaysEnabled ? true : undefined;
        const timePoint = this.timePoint;
        if (undefined === direction && undefined === intensity && undefined === alwaysEnabled && undefined === timePoint)
            return undefined;
        const json = {};
        if (direction)
            json.direction = direction;
        if (undefined !== intensity)
            json.intensity = intensity;
        if (undefined !== alwaysEnabled)
            json.alwaysEnabled = alwaysEnabled;
        if (undefined !== timePoint)
            json.timePoint = timePoint;
        return json;
    }
    /** Create a copy of this SolarLight, identical except in any properties explicitly specified by `changedProps`, with a possible exception for [[timePoint]].
     * If `this.timePoint` is defined and `changedProps` defines `direction` but **doesn't** define `timePoint`, the time point will only be preserved in the
     * copy if `changesProps.direction` is equal to `this.direction`.
     */
    clone(changedProps) {
        var _a;
        if (!changedProps)
            return this;
        const props = (_a = this.toJSON()) !== null && _a !== void 0 ? _a : {};
        if (undefined !== changedProps.direction)
            props.direction = changedProps.direction;
        if (undefined !== changedProps.intensity)
            props.intensity = changedProps.intensity;
        if (undefined !== changedProps.alwaysEnabled)
            props.alwaysEnabled = changedProps.alwaysEnabled;
        if (undefined !== changedProps.timePoint)
            props.timePoint = changedProps.timePoint;
        // If our direction was computed from a time point and the caller only supplies a direction, invalidate the time point unless the input direction matches our direction.
        // If caller explicitly supplied a timePoint, trust it.
        if (undefined !== this.timePoint && undefined === changedProps.timePoint && undefined !== changedProps.direction) {
            const newDirection = geometry_core_1.Vector3d.fromJSON(changedProps.direction);
            if (!newDirection.isAlmostEqual(this.direction))
                props.timePoint = undefined;
        }
        return new SolarLight(props);
    }
    equals(rhs) {
        return this.intensity === rhs.intensity && this.alwaysEnabled === rhs.alwaysEnabled && this.direction.isExactEqual(rhs.direction) && this.timePoint === rhs.timePoint;
    }
}
exports.SolarLight = SolarLight;
/** Describes the ambient light associated with a [[LightSettings]].
 * @see [[AmbientLightProps]]
 * @public
 */
class AmbientLight {
    constructor(json) {
        json = json || {};
        this.intensity = extractIntensity(json.intensity, 0.2);
        this.color = json.color ? RgbColor_1.RgbColor.fromJSON(json.color) : new RgbColor_1.RgbColor(0, 0, 0);
    }
    toJSON() {
        const color = this.color.r !== 0 || this.color.g !== 0 || this.color.b !== 0 ? this.color.toJSON() : undefined;
        const intensity = 0.2 !== this.intensity ? this.intensity : undefined;
        if (undefined === color && undefined === intensity)
            return undefined;
        const json = {};
        if (color)
            json.color = color;
        if (undefined !== intensity)
            json.intensity = intensity;
        return json;
    }
    /** Create a copy of this light, identical except for any properties explicitly specified by `changed`. */
    clone(changed) {
        var _a;
        if (!changed)
            return this;
        const props = (_a = this.toJSON()) !== null && _a !== void 0 ? _a : {};
        if (undefined !== changed.intensity)
            props.intensity = changed.intensity;
        if (undefined !== changed.color)
            props.color = changed.color;
        return new AmbientLight(props);
    }
    equals(rhs) {
        return this.intensity === rhs.intensity && this.color.equals(rhs.color);
    }
}
exports.AmbientLight = AmbientLight;
const defaultUpperHemisphereColor = new RgbColor_1.RgbColor(143, 205, 255);
const defaultLowerHemisphereColor = new RgbColor_1.RgbColor(120, 143, 125);
/** Describes a pair of hemisphere lights associated with a [[LightSettings]].
 * @see [[HemisphereLightsProps]]
 * @public
 */
class HemisphereLights {
    constructor(json) {
        json = json || {};
        this.intensity = extractIntensity(json.intensity, 0);
        this.upperColor = json.upperColor ? RgbColor_1.RgbColor.fromJSON(json.upperColor) : defaultUpperHemisphereColor;
        this.lowerColor = json.lowerColor ? RgbColor_1.RgbColor.fromJSON(json.lowerColor) : defaultLowerHemisphereColor;
    }
    toJSON() {
        const upperColor = this.upperColor.equals(defaultUpperHemisphereColor) ? undefined : this.upperColor.toJSON();
        const lowerColor = this.lowerColor.equals(defaultLowerHemisphereColor) ? undefined : this.lowerColor.toJSON();
        const intensity = 0 === this.intensity ? undefined : this.intensity;
        if (undefined === upperColor && undefined === lowerColor && undefined === intensity)
            return undefined;
        const json = {};
        if (upperColor)
            json.upperColor = upperColor;
        if (lowerColor)
            json.lowerColor = lowerColor;
        if (undefined !== intensity)
            json.intensity = intensity;
        return json;
    }
    /** Create a copy of these lights, identical except for any properties explicitly specified by `changed`. */
    clone(changed) {
        if (!changed)
            return this;
        const props = this.toJSON() || {};
        if (undefined !== changed.upperColor)
            props.upperColor = changed.upperColor;
        if (undefined !== changed.lowerColor)
            props.lowerColor = changed.lowerColor;
        if (undefined !== changed.intensity)
            props.intensity = changed.intensity;
        return new HemisphereLights(props);
    }
    equals(rhs) {
        return this.intensity === rhs.intensity && this.upperColor.equals(rhs.upperColor) && this.lowerColor.equals(rhs.lowerColor);
    }
}
exports.HemisphereLights = HemisphereLights;
/** Describes the lighting for a 3d scene, associated with a [[DisplayStyle3dSettings]] in turn associated with a [DisplayStyle3d]($backend) or [DisplayStyle3dState]($frontend).
 * @see [[LightSettingsProps]]
 * @public
 */
class LightSettings {
    constructor(solar, ambient, hemisphere, portraitIntensity, specularIntensity, numCels) {
        this.solar = solar;
        this.ambient = ambient;
        this.hemisphere = hemisphere;
        this.portraitIntensity = portraitIntensity;
        this.specularIntensity = specularIntensity;
        this.numCels = numCels;
    }
    static fromJSON(props) {
        var _a;
        const solar = new SolarLight(props === null || props === void 0 ? void 0 : props.solar);
        const ambient = new AmbientLight(props === null || props === void 0 ? void 0 : props.ambient);
        const hemisphere = new HemisphereLights(props === null || props === void 0 ? void 0 : props.hemisphere);
        const portraitIntensity = extractIntensity((_a = props === null || props === void 0 ? void 0 : props.portrait) === null || _a === void 0 ? void 0 : _a.intensity, 0.3);
        const specularIntensity = extractIntensity(props === null || props === void 0 ? void 0 : props.specularIntensity, 1.0);
        const numCels = bentleyjs_core_1.JsonUtils.asInt(props === null || props === void 0 ? void 0 : props.numCels, 0);
        return new LightSettings(solar, ambient, hemisphere, portraitIntensity, specularIntensity, numCels);
    }
    toJSON() {
        const solar = this.solar.toJSON();
        const ambient = this.ambient.toJSON();
        const hemisphere = this.hemisphere.toJSON();
        const portrait = 0.3 !== this.portraitIntensity ? { intensity: this.portraitIntensity } : undefined;
        const specularIntensity = 1 !== this.specularIntensity ? this.specularIntensity : undefined;
        const numCels = 0 !== this.numCels ? this.numCels : undefined;
        if (!solar && !ambient && !hemisphere && !portrait && undefined === specularIntensity && undefined === numCels)
            return undefined;
        const json = {};
        if (solar)
            json.solar = solar;
        if (ambient)
            json.ambient = ambient;
        if (hemisphere)
            json.hemisphere = hemisphere;
        if (portrait)
            json.portrait = portrait;
        if (undefined !== specularIntensity)
            json.specularIntensity = specularIntensity;
        if (undefined !== numCels)
            json.numCels = numCels;
        return json;
    }
    /** Create a copy of these light settings, identical except for any properties explicitly specified by `changed`.
     * Note that the solar, ambient, and hemisphere lights will also be cloned using their own `clone` methods - so for example, the following:
     * `  clone({ ambient: { intensity: 0.5 } })`
     * will overwrite the ambient light's intensity but preserve its current color, rather than replacing the color with the default color.
     */
    clone(changed) {
        var _a, _b, _c, _d;
        if (!changed)
            return this;
        const solar = this.solar.clone(changed.solar);
        const ambient = this.ambient.clone(changed.ambient);
        const hemisphere = this.hemisphere.clone(changed.hemisphere);
        const portrait = (_b = (_a = changed.portrait) === null || _a === void 0 ? void 0 : _a.intensity) !== null && _b !== void 0 ? _b : this.portraitIntensity;
        const specular = (_c = changed.specularIntensity) !== null && _c !== void 0 ? _c : this.specularIntensity;
        const numCels = (_d = changed.numCels) !== null && _d !== void 0 ? _d : this.numCels;
        return new LightSettings(solar, ambient, hemisphere, portrait, specular, numCels);
    }
    equals(rhs) {
        if (this === rhs)
            return true;
        return this.portraitIntensity === rhs.portraitIntensity && this.specularIntensity === rhs.specularIntensity && this.numCels === rhs.numCels
            && this.ambient.equals(rhs.ambient) && this.solar.equals(rhs.solar) && this.hemisphere.equals(rhs.hemisphere);
    }
}
exports.LightSettings = LightSettings;
//# sourceMappingURL=LightSettings.js.map