"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.MapImagerySettings = void 0;
const BackgroundMapSettings_1 = require("./BackgroundMapSettings");
const ColorDef_1 = require("./ColorDef");
const MapLayerSettings_1 = require("./MapLayerSettings");
/** Provides access to the map imagery settings (Base and layers).
 * In earlier versions only a background map was supported as specified by the providerName and mapType members of [[BackgroundMapSettings]] object.
 * In order to provide backward compatibility the original [[BackgroundMapSettings]] are synchronized with the [[MapImagerySettings]] base layer as long as
 * the settings are compatible.
 * @beta
 */
class MapImagerySettings {
    constructor(backgroundBaseProps, backgroundLayerProps, overlayLayersProps, mapProps) {
        this._backgroundLayers = new Array();
        this._overlayLayers = new Array();
        const base = typeof backgroundBaseProps === "number" ? ColorDef_1.ColorDef.create(backgroundBaseProps) : MapLayerSettings_1.MapLayerSettings.fromJSON(backgroundBaseProps);
        this._backgroundBase = base ? base : MapLayerSettings_1.MapLayerSettings.fromMapSettings(BackgroundMapSettings_1.BackgroundMapSettings.fromJSON(mapProps));
        if (backgroundLayerProps) {
            for (const layerProps of backgroundLayerProps) {
                const layer = MapLayerSettings_1.MapLayerSettings.fromJSON(layerProps);
                if (layer)
                    this._backgroundLayers.push(layer);
            }
        }
        if (overlayLayersProps) {
            for (const overlayLayerProps of overlayLayersProps) {
                const overlayLayer = MapLayerSettings_1.MapLayerSettings.fromJSON(overlayLayerProps);
                if (overlayLayer)
                    this._overlayLayers.push(overlayLayer);
            }
        }
    }
    /** The settings for the base layer.
     *  @note If changing the base provider it is currently necessary to also update the background map settings.
     */
    get backgroundBase() { return this._backgroundBase; }
    set backgroundBase(base) { this._backgroundBase = base; }
    get backgroundLayers() { return this._backgroundLayers; }
    get overlayLayers() { return this._overlayLayers; }
    /** Return base transparency as a number between 0 and 1.
     * @internal
     */
    get baseTransparency() {
        return (this._backgroundBase instanceof ColorDef_1.ColorDef) ? (this._backgroundBase.getTransparency() / 255) : this._backgroundBase.transparency;
    }
    /** Construct from JSON, performing validation and applying default values for undefined fields. */
    static fromJSON(imageryJson, mapProps) {
        return new MapImagerySettings(imageryJson === null || imageryJson === void 0 ? void 0 : imageryJson.backgroundBase, imageryJson === null || imageryJson === void 0 ? void 0 : imageryJson.backgroundLayers, imageryJson === null || imageryJson === void 0 ? void 0 : imageryJson.overlayLayers, mapProps);
    }
    toJSON() {
        return {
            backgroundBase: this._backgroundBase.toJSON(),
            backgroundLayers: this._backgroundLayers.length > 0 ? this._backgroundLayers.map((layer) => layer.toJSON()) : undefined,
            overlayLayers: this._overlayLayers.length > 0 ? this._overlayLayers.map((layer) => layer.toJSON()) : undefined,
        };
    }
}
exports.MapImagerySettings = MapImagerySettings;
//# sourceMappingURL=MapImagerySettings.js.map