"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.MapLayerSettings = exports.MapSubLayerSettings = void 0;
const BackgroundMapSettings_1 = require("./BackgroundMapSettings");
/** Normalized representation of a [[MapSubLayerProps]] for which values
 * have been validated and default values have been applied where explicit values not defined.
 * A map sub layer represents a set of objects within the layer that can be controlled separately.  These
 * are produced only from map servers that produce images on demand and are not supported by tiled (cached) servers.
 * This class can represent an hierarchy, in this case a sub layer is visible only if all its ancestors are also visible.
 * @see [[MapLayerSettings]]
 * @beta
 */
class MapSubLayerSettings {
    constructor(name, title, visible, id, parent, children) {
        this.name = name;
        this.title = title;
        this.visible = visible !== undefined && visible;
        this.id = (id === undefined) ? this.name : id;
        this.parent = parent;
        this.children = children;
    }
    /** Construct from JSON, performing validation and applying default values for undefined fields. */
    static fromJSON(json) {
        if (undefined === json || undefined === json.name)
            return undefined;
        return new MapSubLayerSettings(json.name, json.title, json.visible, (json.id === json.name) ? undefined : json.id, json.parent, json.children);
    }
    toJSON() {
        return {
            name: this.name,
            title: this.title,
            visible: this.visible,
            id: (this.id === this.name) ? undefined : this.id,
            parent: this.parent,
            children: this.children,
        };
    }
    /** Creating a copy of this MapSubLayer, optionally modifying some if its properties */
    clone(changedProps) {
        var _a;
        if (undefined === changedProps)
            return this;
        const props = {
            name: undefined !== changedProps.name ? changedProps.name : this.name,
            id: undefined !== changedProps.id ? changedProps.id : this.id,
            visible: undefined !== changedProps.visible ? changedProps.visible : this.visible,
            parent: undefined !== changedProps.parent ? changedProps.parent : this.parent,
            children: undefined !== changedProps.children ? changedProps.children.slice() : (_a = this.children) === null || _a === void 0 ? void 0 : _a.slice(),
            title: undefined !== changedProps.title ? changedProps.title : this.title,
        };
        return MapSubLayerSettings.fromJSON(props);
    }
    /** @internal */
    displayMatches(other) {
        return this.name === other.name && this.visible === other.visible;
    }
    /** return true if this sublayer is named. */
    get isNamed() { return this.name.length > 0; }
    /** return true if this sublayer is a leaf (has no children) */
    get isLeaf() { return this.children === undefined || this.children.length === 0; }
    /** return true if this sublayer is an unnamed group */
    get isUnnamedGroup() { return !this.isLeaf && !this.isNamed; }
    /** return a string representing this sublayer id (converting to string if underlying id is number) */
    get idString() { return (typeof this.id === "number") ? this.id.toString(10) : this.id; }
}
exports.MapSubLayerSettings = MapSubLayerSettings;
/** Normalized representation of a [[MapLayerProps]] for which values have been  validated and default values have been applied where explicit values not defined.
 * One or more map layers may be included within [[MapImagerySettings]] object.
 * @see [[MapImagerySettings]]
 * @beta
 */
class MapLayerSettings {
    constructor(url, name, formatId = "WMS", visible = true, jsonSubLayers = undefined, transparency = 0, transparentBackground = true, isBase = false, userName, password, accessKey) {
        var _a;
        this.formatId = formatId;
        this.name = name;
        this.visible = visible;
        this.transparentBackground = transparentBackground;
        this.isBase = isBase;
        this.subLayers = new Array();
        if (jsonSubLayers !== undefined) {
            let hasUnnamedGroups = false;
            for (const jsonSubLayer of jsonSubLayers) {
                const subLayer = MapSubLayerSettings.fromJSON(jsonSubLayer);
                if (undefined !== subLayer) {
                    this.subLayers.push(subLayer);
                    if (((_a = subLayer.children) === null || _a === void 0 ? void 0 : _a.length) !== 0 && !subLayer.isNamed && !hasUnnamedGroups)
                        hasUnnamedGroups = true;
                }
            }
            this.userName = userName;
            this.password = password;
        }
        this.accessKey = accessKey;
        this.transparency = transparency;
        this.url = url;
    }
    setCredentials(userName, password) {
        this.userName = userName;
        this.password = password;
    }
    /** Construct from JSON, performing validation and applying default values for undefined fields. */
    static fromJSON(json) {
        if (json === undefined || json.url === undefined || undefined === json.name)
            return undefined;
        const transparentBackground = (json.transparentBackground === undefined) ? true : json.transparentBackground;
        return new MapLayerSettings(json.url, json.name, json.formatId, json.visible, json.subLayers, json.transparency, transparentBackground, json.isBase === true, undefined, undefined, json.accessKey);
    }
    /** return JSON representation of this MapLayerSettings object */
    toJSON() {
        const props = {};
        if (this.subLayers) {
            props.subLayers = [];
            this.subLayers.forEach((subLayer) => {
                const subLayerJson = subLayer.toJSON();
                if (subLayerJson)
                    props.subLayers.push(subLayerJson);
            });
        }
        props.formatId = this.formatId;
        props.name = this.name;
        props.url = this.url;
        props.accessKey = this.accessKey;
        if (0 !== this.transparency)
            props.transparency = this.transparency;
        if (this.transparentBackground === false)
            props.transparentBackground = this.transparentBackground;
        if (this.isBase === true)
            props.isBase = this.isBase;
        props.visible = this.visible;
        return props;
    }
    /** @internal */
    static mapTypeName(type) {
        switch (type) {
            case BackgroundMapSettings_1.BackgroundMapType.Aerial:
                return "Aerial Imagery";
            default:
            case BackgroundMapSettings_1.BackgroundMapType.Hybrid:
                return "Aerial Imagery with labels";
            case BackgroundMapSettings_1.BackgroundMapType.Street:
                return "Streets";
        }
    }
    /** Create a [[MapLayerSettings]] object from the image settings within a [[BackgroundMapSettings]] object (providerName and mapType).  */
    static fromMapSettings(mapSettings) {
        let formatId, url, name;
        switch (mapSettings.providerName) {
            case "BingProvider":
            default:
                formatId = "BingMaps";
                let imagerySet;
                switch (mapSettings.mapType) {
                    case BackgroundMapSettings_1.BackgroundMapType.Street:
                        imagerySet = "Road";
                        break;
                    case BackgroundMapSettings_1.BackgroundMapType.Aerial:
                        imagerySet = "Aerial";
                        break;
                    case BackgroundMapSettings_1.BackgroundMapType.Hybrid:
                    default:
                        imagerySet = "AerialWithLabels";
                        break;
                }
                name = `Bing Maps: ${MapLayerSettings.mapTypeName(mapSettings.mapType)}`;
                url = `https://dev.virtualearth.net/REST/v1/Imagery/Metadata/${imagerySet}?o=json&incl=ImageryProviders&key={bingKey}`;
                break;
            case "MapBoxProvider":
                formatId = "MapboxImagery";
                name = `MapBox: ${MapLayerSettings.mapTypeName(mapSettings.mapType)}`;
                switch (mapSettings.mapType) {
                    case BackgroundMapSettings_1.BackgroundMapType.Street:
                        url = "https://api.mapbox.com/v4/mapbox.streets/";
                        break;
                    case BackgroundMapSettings_1.BackgroundMapType.Aerial:
                        url = "https://api.mapbox.com/v4/mapbox.satellite/";
                        break;
                    case BackgroundMapSettings_1.BackgroundMapType.Hybrid:
                        url = "https://api.mapbox.com/v4/mapbox.streets-satellite/";
                        break;
                }
                break;
        }
        return MapLayerSettings.fromJSON({ name, formatId, url, transparentBackground: false, isBase: true });
    }
    /** Create a copy of this MapLayerSettings, optionally modifying some of its properties.
     * @param changedProps JSON representation of the properties to change.
     * @returns A MapLayerSettings with all of its properties set to match those of `this`, except those explicitly defined in `changedProps`.
     */
    clone(changedProps) {
        if (undefined === changedProps)
            return this;
        const props = {
            name: undefined !== changedProps.name ? changedProps.name : this.name,
            formatId: undefined !== changedProps.formatId ? changedProps.formatId : this.formatId,
            visible: undefined !== changedProps.visible ? changedProps.visible : this.visible,
            url: undefined !== changedProps.url ? changedProps.url : this.url,
            transparency: undefined !== changedProps.transparency ? changedProps.transparency : this.transparency,
            transparentBackground: undefined !== changedProps.transparentBackground ? changedProps.transparentBackground : this.transparentBackground,
            subLayers: undefined !== changedProps.subLayers ? changedProps.subLayers : this.subLayers,
            accessKey: undefined !== changedProps.accessKey ? changedProps.accessKey : this.accessKey,
        };
        const clone = MapLayerSettings.fromJSON(props);
        // Clone members not part of MapLayerProps
        clone.userName = this.userName;
        clone.password = this.password;
        return clone;
    }
    /** @internal */
    displayMatches(other) {
        if (!this.matchesNameAndUrl(other.name, other.url)
            || this.visible !== other.visible
            || this.transparency !== other.transparency
            || this.transparentBackground !== other.transparentBackground
            || this.subLayers.length !== other.subLayers.length) {
            return false;
        }
        if (this.userName !== other.userName || this.password !== other.password) {
            return false;
        }
        for (let i = 0; i < this.subLayers.length; i++)
            if (!this.subLayers[i].displayMatches(other.subLayers[i]))
                return false;
        return true;
    }
    /** @internal */
    matchesNameAndUrl(name, url) {
        return this.name === name && this.url === url;
    }
    /** Return a sublayer matching id -- or undefined if not found */
    subLayerById(id) {
        return id === undefined ? undefined : this.subLayers.find((subLayer) => subLayer.id === id);
    }
    hasInvisibleAncestors(subLayer) {
        if (!subLayer || !subLayer.parent)
            return false;
        const parent = this.subLayerById(subLayer.parent);
        if (!parent)
            return false;
        // Visibility of named group has no impact on the visibility of children (only unnamed group does)
        // i.e For WMS, its should be possible to request a child layer when its parent is not visible (if the parent is also named)
        return (!parent.visible && !parent.isNamed) || this.hasInvisibleAncestors(parent);
    }
    /** Return true if sublayer is visible -- testing ancestors for visibility if they exist. */
    isSubLayerVisible(subLayer) {
        if (!subLayer.visible)
            return false;
        return !this.hasInvisibleAncestors(subLayer);
    }
    /** Return true if all sublayers are visible. */
    get allSubLayersInvisible() {
        if (this.subLayers.length === 0)
            return false;
        return this.subLayers.every((subLayer) => (subLayer.isUnnamedGroup || !this.isSubLayerVisible(subLayer)));
    }
    /** Return the children for a sublayer */
    getSubLayerChildren(subLayer) {
        if (!subLayer.children)
            return undefined;
        const children = new Array();
        subLayer.children.forEach((childId) => {
            const child = this.subLayerById(childId);
            if (child !== undefined)
                children.push(child);
        });
        return children;
    }
}
exports.MapLayerSettings = MapLayerSettings;
//# sourceMappingURL=MapLayerSettings.js.map