"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlanarClipMaskSettings = exports.PlanarClipMaskPriority = exports.PlanarClipMaskMode = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
/** The different modes by which a [[PlanarClipMaskSettings]] collects the geometry used to mask a model.
 * @public
 */
var PlanarClipMaskMode;
(function (PlanarClipMaskMode) {
    /** No masking. */
    PlanarClipMaskMode[PlanarClipMaskMode["None"] = 0] = "None";
    /** Mask based on priority. Different types of models have different default priorities as enumerated by [[PlanarClipMaskPriority]].
     * For example, background maps have the lowest priority, so they are masked by all other types, while design models have the highest priority and are therefore never masked.
     * The priority of a reality model can be overridden by [[PlanarClipMaskSettings.priority]]. This is useful to allow one reality model to mask another overlapping one.
     */
    PlanarClipMaskMode[PlanarClipMaskMode["Priority"] = 1] = "Priority";
    /** Indicates that masks should be produced from the geometry in a set of [GeometricModel]($backend)s. */
    PlanarClipMaskMode[PlanarClipMaskMode["Models"] = 2] = "Models";
    /** Indicates that masks should be produced from geometry belonging to a set of subcategories. */
    PlanarClipMaskMode[PlanarClipMaskMode["IncludeSubCategories"] = 3] = "IncludeSubCategories";
    /** Indicates that masks should be produced from the geometry of a set of [GeometricElement]($backend)s. */
    PlanarClipMaskMode[PlanarClipMaskMode["IncludeElements"] = 4] = "IncludeElements";
    /** Indicates that masks should be produced from the geometry of all [GeometricElement]($backend)s in a view, **except** for a specified set of excluded elements. */
    PlanarClipMaskMode[PlanarClipMaskMode["ExcludeElements"] = 5] = "ExcludeElements";
})(PlanarClipMaskMode = exports.PlanarClipMaskMode || (exports.PlanarClipMaskMode = {}));
/** The default priority values for a [[PlanarClipMaskSettings]], based on model type. Models with a lower priority are masked by models with a higher priority.
 * The default can be overridden by [[PlanarClipMaskSettings.priority]].
 *  @public
 */
var PlanarClipMaskPriority;
(function (PlanarClipMaskPriority) {
    /** Background map. */
    PlanarClipMaskPriority[PlanarClipMaskPriority["BackgroundMap"] = -2048] = "BackgroundMap";
    /** A reality model that spans the globe - e.g., OpenStreetMaps Buildings. */
    PlanarClipMaskPriority[PlanarClipMaskPriority["GlobalRealityModel"] = -1024] = "GlobalRealityModel";
    /** A reality model with a bounded range. */
    PlanarClipMaskPriority[PlanarClipMaskPriority["RealityModel"] = 0] = "RealityModel";
    /** A design model stored in the [IModelDb]($backend). */
    PlanarClipMaskPriority[PlanarClipMaskPriority["DesignModel"] = 2048] = "DesignModel";
})(PlanarClipMaskPriority = exports.PlanarClipMaskPriority || (exports.PlanarClipMaskPriority = {}));
/** Describes how to mask the geometry of one [GeometricModel]($backend) for display. The mask is produced by projecting geometry from any number of other models -
 * optionally filtered by [SubCategory]($backend) or by a set of specific [GeometricElement]($backend)s - onto a plane. Regions of the masked model that intersect the
 * mask are rendered partially or completely transparent. This is useful for, e.g., making subsurface geometry visible below the background map, or clipping out portions
 * of a reality model that intersect a design model.
 * @note Currently reality models (including background maps and terrain) can be masked, but design models can only produce masks.
 * @see [[DisplayStyleSettings.planarClipMasks]] to define clip masks for a [DisplayStyle]($backend).
 * @see [[ContextRealityModel.planarClipMaskSettings]] to apply a clip mask to a context reality model.
 * @public
 */
class PlanarClipMaskSettings {
    constructor(mode, transparency, modelIds, subCategoryOrElementIds, priority) {
        this.mode = mode;
        this._modelIds = modelIds;
        this._subCategoryOrElementIds = subCategoryOrElementIds;
        this.priority = priority;
        this.transparency = undefined !== transparency ? Math.max(0, Math.min(1, transparency)) : undefined;
        if (modelIds)
            this.modelIds = bentleyjs_core_1.CompressedId64Set.iterable(modelIds);
        if (subCategoryOrElementIds)
            this.subCategoryOrElementIds = bentleyjs_core_1.CompressedId64Set.iterable(subCategoryOrElementIds);
    }
    /** The compressed representation of [[modelIds]]. */
    get compressedModelIds() {
        return this._modelIds;
    }
    /** Create a new [[PlanarClipMaskSettings]] object from its JSON representation. */
    static fromJSON(json) {
        if (!json || undefined === json.mode)
            return this.defaults;
        return new PlanarClipMaskSettings(json.mode, json.transparency, json.modelIds, json.subCategoryOrElementIds, json.priority);
    }
    /** Create settings for [[PlanarClipMaskMode.Models]]. */
    static createForModels(modelIds, transparency) {
        return this.fromJSON({
            mode: PlanarClipMaskMode.Models,
            transparency,
            modelIds: modelIds ? bentleyjs_core_1.CompressedId64Set.sortAndCompress(modelIds) : undefined,
        });
    }
    /** Create settings that filter by element or subcategory. */
    static createForElementsOrSubCategories(mode, elementOrSubCategoryIds, modelIds, transparency) {
        return this.fromJSON({
            mode,
            transparency,
            modelIds: modelIds ? bentleyjs_core_1.CompressedId64Set.sortAndCompress(modelIds) : undefined,
            subCategoryOrElementIds: bentleyjs_core_1.CompressedId64Set.sortAndCompress(elementOrSubCategoryIds),
        });
    }
    /** Create settings that mask by priority.
     * @see [[PlanarClipMaskPriority]] for default priority values based on model type.
     */
    static createByPriority(priority, transparency) {
        return new PlanarClipMaskSettings(PlanarClipMaskMode.Priority, transparency, undefined, undefined, priority);
    }
    /** Create JSON object representing this [[PlanarClipMaskSettings]] */
    toJSON() {
        const props = { mode: this.mode };
        if (undefined !== this._modelIds)
            props.modelIds = this._modelIds;
        if (undefined !== this._subCategoryOrElementIds)
            props.subCategoryOrElementIds = this._subCategoryOrElementIds;
        if (undefined !== this.priority)
            props.priority = this.priority;
        if (undefined !== this.transparency)
            props.transparency = this.transparency;
        return props;
    }
    /** Returns true if masking is enabled. */
    get isValid() {
        return this.mode !== PlanarClipMaskMode.None;
    }
    equals(other) {
        return this.mode === other.mode &&
            this.priority === other.priority &&
            this.transparency === other.transparency &&
            this._modelIds === other._modelIds &&
            this._subCategoryOrElementIds === other._subCategoryOrElementIds;
    }
    /** Create a copy of this TerrainSettings, optionally modifying some of its properties.
     * @param changedProps JSON representation of the properties to change.
     * @returns A PlanarClipMaskSettings with all of its properties set to match those of`this`, except those explicitly defined in `changedProps`.
     */
    clone(changedProps) {
        if (undefined === changedProps)
            return this;
        return PlanarClipMaskSettings.fromJSON(Object.assign(Object.assign({}, this.toJSON()), changedProps));
    }
}
exports.PlanarClipMaskSettings = PlanarClipMaskSettings;
/** A default PlanarClipMask which masks nothing. */
PlanarClipMaskSettings.defaults = new PlanarClipMaskSettings(PlanarClipMaskMode.None);
//# sourceMappingURL=PlanarClipMask.js.map