"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Rendering
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.PolylineEdgeArgs = exports.SilhouetteEdgeArgs = exports.EdgeArgs = exports.MeshEdges = exports.MeshEdge = exports.MeshPolylineList = exports.MeshPolyline = exports.PolylineData = exports.PolylineFlags = exports.PolylineTypeFlags = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
// cSpell:ignore vals
/** @internal */
var PolylineTypeFlags;
(function (PolylineTypeFlags) {
    PolylineTypeFlags[PolylineTypeFlags["Normal"] = 0] = "Normal";
    PolylineTypeFlags[PolylineTypeFlags["Edge"] = 1] = "Edge";
    PolylineTypeFlags[PolylineTypeFlags["Outline"] = 2] = "Outline";
})(PolylineTypeFlags = exports.PolylineTypeFlags || (exports.PolylineTypeFlags = {}));
/** Flags describing a polyline. A polyline may represent a continuous line string, or a set of discrete points.
 * @internal
 */
class PolylineFlags {
    constructor(is2d = false, isPlanar = false, isDisjoint = false, type = PolylineTypeFlags.Normal) {
        this.isDisjoint = isDisjoint;
        this.isPlanar = isPlanar;
        this.is2d = is2d;
        this.type = type;
    }
    clone() { return new PolylineFlags(this.is2d, this.isPlanar, this.isDisjoint, this.type); }
    /** Create a PolylineFlags from a serialized numeric representation. */
    static unpack(value) {
        const isDisjoint = 0 !== (value & 1);
        const isPlanar = 0 !== (value & 2);
        const is2d = 0 !== (value & 4);
        const type = (value >> 3);
        bentleyjs_core_1.assert(type === PolylineTypeFlags.Normal || type === PolylineTypeFlags.Edge || type === PolylineTypeFlags.Outline);
        return new PolylineFlags(is2d, isPlanar, isDisjoint, type);
    }
    initDefaults() {
        this.isDisjoint = this.isPlanar = this.is2d = false;
        this.type = PolylineTypeFlags.Normal;
    }
    get isOutlineEdge() { return PolylineTypeFlags.Outline === this.type; }
    get isNormalEdge() { return PolylineTypeFlags.Edge === this.type; }
    get isAnyEdge() { return PolylineTypeFlags.Normal !== this.type; }
    setIsNormalEdge() { this.type = PolylineTypeFlags.Edge; }
    setIsOutlineEdge() { this.type = PolylineTypeFlags.Outline; }
    /** Convert these flags to a numeric representation for serialization. */
    pack() {
        let val = 0;
        if (this.isDisjoint)
            val += 1;
        if (this.isPlanar)
            val += 1 << 1;
        if (this.is2d)
            val += 1 << 2;
        val += this.type << 3;
        return val;
    }
    equals(other) {
        return this.type === other.type && this.is2d === other.is2d && this.isPlanar === other.isPlanar && this.isDisjoint === other.isDisjoint;
    }
}
exports.PolylineFlags = PolylineFlags;
/** @internal */
class PolylineData {
    constructor(vertIndices = [], numIndices = 0) {
        this.vertIndices = vertIndices;
        this.numIndices = numIndices;
    }
    get isValid() { return 0 < this.numIndices; }
    reset() { this.numIndices = 0; this.vertIndices = []; }
    init(polyline) {
        this.numIndices = polyline.indices.length;
        this.vertIndices = 0 < this.numIndices ? polyline.indices : [];
        return this.isValid;
    }
}
exports.PolylineData = PolylineData;
/** @internal */
class MeshPolyline {
    constructor(indices = []) {
        this.indices = indices.slice();
    }
    addIndex(index) {
        const { indices } = this;
        if (indices.length === 0 || indices[indices.length - 1] !== index)
            indices.push(index);
    }
    clear() { this.indices.length = 0; }
}
exports.MeshPolyline = MeshPolyline;
/** @internal */
class MeshPolylineList extends Array {
    constructor(...args) { super(...args); }
}
exports.MeshPolylineList = MeshPolylineList;
/** @internal */
class MeshEdge {
    constructor(index0, index1) {
        this.indices = [0, 0];
        if (undefined === index0 || undefined === index1)
            return;
        if (index0 < index1) {
            this.indices[0] = index0;
            this.indices[1] = index1;
        }
        else {
            this.indices[0] = index1;
            this.indices[1] = index0;
        }
    }
    compareTo(other) {
        let diff = this.indices[0] - other.indices[0];
        if (0 === diff)
            diff = this.indices[1] - other.indices[1];
        return diff;
    }
}
exports.MeshEdge = MeshEdge;
/** @internal */
class MeshEdges {
    constructor() {
        this.visible = [];
        this.silhouette = [];
        this.polylines = new MeshPolylineList();
        this.silhouetteNormals = [];
    }
}
exports.MeshEdges = MeshEdges;
/** @internal */
class EdgeArgs {
    init(meshEdges) {
        this.clear();
        if (undefined !== meshEdges && 0 < meshEdges.visible.length)
            this.edges = meshEdges.visible;
        return this.isValid;
    }
    clear() { this.edges = undefined; }
    get isValid() { return 0 < this.numEdges; }
    get numEdges() { return undefined !== this.edges ? this.edges.length : 0; }
}
exports.EdgeArgs = EdgeArgs;
/** @internal */
class SilhouetteEdgeArgs extends EdgeArgs {
    init(meshEdges) {
        this.clear();
        if (undefined !== meshEdges && 0 < meshEdges.silhouette.length) {
            this.edges = meshEdges.silhouette;
            this.normals = meshEdges.silhouetteNormals;
        }
        return this.isValid;
    }
    clear() { this.normals = undefined; super.clear(); }
}
exports.SilhouetteEdgeArgs = SilhouetteEdgeArgs;
/** @internal */
class PolylineEdgeArgs {
    constructor(lines) { this.init(lines); }
    init(lines) {
        this.lines = undefined !== lines && 0 < lines.length ? lines : undefined;
        return this.isValid;
    }
    get numLines() { return undefined !== this.lines ? this.lines.length : 0; }
    get isValid() { return this.numLines > 0; }
    clear() { this.lines = undefined; }
}
exports.PolylineEdgeArgs = PolylineEdgeArgs;
//# sourceMappingURL=Render.js.map