"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Rendering
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RenderMaterial = void 0;
/** Represents a material which can be applied to a surface to control aspects of its appearance such as color, reflectivity, texture, and so on.
 * @public
 */
class RenderMaterial {
    constructor(params) {
        this.key = params.key;
        this.textureMapping = params.textureMapping;
    }
    get hasTexture() {
        var _a;
        return undefined !== ((_a = this.textureMapping) === null || _a === void 0 ? void 0 : _a.texture);
    }
}
exports.RenderMaterial = RenderMaterial;
/** @public */
(function (RenderMaterial) {
    function clampToNormalizedRange(value) {
        return Math.max(0.0, Math.min(1.0, value));
    }
    /** Parameters used to construct a [[RenderMaterial]] */
    class Params {
        constructor(key) {
            /** Diffuse weight in [0..1] */
            this.diffuse = 0.6;
            /** Specular weight in [0..1] */
            this.specular = 0.4;
            this.specularExponent = 13.5;
            /** Currently unused. */
            this.reflect = 0.0;
            /** Currently unused. */
            this.refract = 1.0;
            /** Currently unused. */
            this.ambient = .3;
            /** Currently unused. */
            this.shadows = true;
            this.key = key;
        }
        /** A value from 0.0 (fully-transparent) to 1.0 (fully-opaque) controlling the transparency of surfaces to which this material is applied;
         * or undefined if this material does not override surface transparency.
         */
        get alpha() { return this._alpha; }
        set alpha(alpha) {
            this._alpha = undefined !== alpha ? clampToNormalizedRange(alpha) : undefined;
        }
        /** Create a RenderMaterial params object using specified key and ColorDef values, as well as an optional texture mapping. */
        static fromColors(key, diffuseColor, specularColor, emissiveColor, reflectColor, textureMap) {
            const materialParams = new Params();
            materialParams.key = key;
            materialParams.diffuseColor = diffuseColor;
            materialParams.specularColor = specularColor;
            materialParams.emissiveColor = emissiveColor;
            materialParams.reflectColor = reflectColor;
            materialParams.textureMapping = textureMap;
            return materialParams;
        }
    }
    /** Obtain an immutable instance of a RenderMaterial with all default properties. */
    Params.defaults = new Params();
    RenderMaterial.Params = Params;
})(RenderMaterial = exports.RenderMaterial || (exports.RenderMaterial = {}));
Object.freeze(RenderMaterial.Params.defaults);
//# sourceMappingURL=RenderMaterial.js.map