"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RenderSchedule = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const geometry_core_1 = require("@bentley/geometry-core");
const RgbColor_1 = require("./RgbColor");
const FeatureSymbology_1 = require("./FeatureSymbology");
function interpolate(start, end, fraction) {
    return start + fraction * (end - start);
}
function interpolateRgb(start, end, fraction) {
    return new RgbColor_1.RgbColor(interpolate(start.r, end.r, fraction), interpolate(start.g, end.g, fraction), interpolate(start.b, end.b, fraction));
}
/** Namespace containing types that collectively define a script that animates the contents of a view by adjusting the visibility, position,
 * and/or symbology of groups of elements over time. A [[RenderSchedule.Script]] is hosted by a [RenderTimeline]($backend) element. The script
 * can be associated with a [DisplayStyleState]($frontend) by way of its [[DisplayStyleSettings.renderTimeline]] property.
 * @public
 */
var RenderSchedule;
(function (RenderSchedule) {
    /** Defines how two interpolate between two entries in a [[RenderSchedule.Timeline]].
     * @note Currently only Linear and Step are supported. Any other value is treated as Step.
     * @see [[RenderSchedule.TimelineEntry]].
     */
    let Interpolation;
    (function (Interpolation) {
        /** Each timeline entry's value is discrete - the timeline jumps from one entry's value directly to another. */
        Interpolation[Interpolation["Step"] = 1] = "Step";
        /** Given two entries on the timeline and a timepoint in between them, linearly interpolate based on the timepoint's distance between the
         * two entries.
         */
        Interpolation[Interpolation["Linear"] = 2] = "Linear";
    })(Interpolation = RenderSchedule.Interpolation || (RenderSchedule.Interpolation = {}));
    /** Describes the value of some property at a specific point along a [[RenderSchedule.Timeline]].
     * @see [[RenderSchedule.VisibilityEntry]]
     * @see [[RenderSchedule.ColorEntry]]
     * @see [[RenderSchedule.TransformEntry]]
     * @see [[RenderSchedule.CuttingPlaneEntry]]
     */
    class TimelineEntry {
        constructor(props) {
            this.time = props.time;
            this.interpolation = props.interpolation === Interpolation.Linear ? props.interpolation : Interpolation.Step;
        }
        toJSON() {
            const props = {
                time: this.time,
            };
            if (this.interpolation === Interpolation.Linear)
                props.interpolation = this.interpolation;
            return props;
        }
    }
    RenderSchedule.TimelineEntry = TimelineEntry;
    /** A timeline entry that controls the visibility of the associated geometry. */
    class VisibilityEntry extends TimelineEntry {
        constructor(props) {
            super(props);
            if (typeof props.value !== "number")
                this.value = 100;
            else
                this.value = Math.max(0, Math.min(100, props.value));
        }
        toJSON() {
            const props = super.toJSON();
            if (100 !== this.value)
                props.value = this.value;
            return props;
        }
    }
    RenderSchedule.VisibilityEntry = VisibilityEntry;
    /** A timeline entry controlling the color of the affected geometry. */
    class ColorEntry extends TimelineEntry {
        constructor(props) {
            super(props);
            if (props.value)
                this.value = new RgbColor_1.RgbColor(props.value.red, props.value.green, props.value.blue);
        }
        toJSON() {
            const props = super.toJSON();
            if (this.value) {
                props.value = {
                    red: this.value.r,
                    green: this.value.g,
                    blue: this.value.b,
                };
            }
            return props;
        }
    }
    RenderSchedule.ColorEntry = ColorEntry;
    /** Describes the components of a [[RenderSchedule.TransformEntry]] as a rotation around a pivot point followed by a translation. */
    class TransformComponents {
        constructor(position, pivot, orientation) {
            this.position = position;
            this.pivot = pivot;
            this.orientation = orientation;
        }
        static fromJSON(props) {
            if (props.pivot && props.position && props.orientation)
                return new TransformComponents(geometry_core_1.Vector3d.fromJSON(props.position), geometry_core_1.Vector3d.fromJSON(props.pivot), geometry_core_1.Point4d.fromJSON(props.orientation));
            else
                return undefined;
        }
        toJSON() {
            return {
                position: [this.position.x, this.position.y, this.position.z],
                pivot: [this.pivot.x, this.pivot.y, this.pivot.z],
                orientation: [this.orientation.x, this.orientation.y, this.orientation.z, this.orientation.w],
            };
        }
    }
    RenderSchedule.TransformComponents = TransformComponents;
    /** A timeline entry that applies rotation, scaling, and/or translation to the affected geometry. */
    class TransformEntry extends TimelineEntry {
        constructor(props) {
            super(props);
            this.value = props.value ? geometry_core_1.Transform.fromJSON(props.value.transform) : geometry_core_1.Transform.identity;
            if (props.value)
                this.components = TransformComponents.fromJSON(props.value);
        }
        toJSON() {
            const props = super.toJSON();
            if (this.components) {
                props.value = this.components.toJSON();
                props.value.transform = this.value.toRows();
            }
            else {
                props.value = { transform: this.value.toRows() };
            }
            return props;
        }
    }
    RenderSchedule.TransformEntry = TransformEntry;
    /** Defines a [ClipPlane]($geometry-core) associated with a [[RenderSchedule.CuttingPlaneEntry]]. */
    class CuttingPlane {
        constructor(props) {
            this.position = geometry_core_1.Point3d.fromJSON(props.position);
            this.direction = geometry_core_1.Point3d.fromJSON(props.direction);
            this.hidden = true === props.hidden;
            this.visible = true === props.visible;
        }
        toJSON() {
            const props = {
                position: [this.position.x, this.position.y, this.position.z],
                direction: [this.direction.x, this.direction.y, this.direction.z],
            };
            if (this.visible)
                props.visible = true;
            if (this.hidden)
                props.hidden = true;
            return props;
        }
    }
    RenderSchedule.CuttingPlane = CuttingPlane;
    /** A timeline entry that applies a [ClipPlane]($geometry-core) to the affected geometry. */
    class CuttingPlaneEntry extends TimelineEntry {
        constructor(props) {
            super(props);
            if (props.value)
                this.value = new CuttingPlane(props.value);
        }
        toJSON() {
            const props = super.toJSON();
            if (this.value)
                props.value = this.value.toJSON();
            return props;
        }
    }
    RenderSchedule.CuttingPlaneEntry = CuttingPlaneEntry;
    /** Identifies a fractional position along a [[RenderSchedule.Timeline]] between any two [[RenderSchedule.TimelineEntry]]'s within a [[RenderSchedule.TimelineEntryList]].
     * @internal
     */
    class Interval {
        constructor(lower = 0, upper = 0, fraction = 0) {
            this.init(lower, upper, fraction);
        }
        init(lower = 0, upper = 0, fraction = 0) {
            this.lowerIndex = lower;
            this.upperIndex = upper;
            this.fraction = fraction;
        }
    }
    RenderSchedule.Interval = Interval;
    /** A list of the [[RenderSchedule.TimelineEntry]] objects within a [[RenderSchedule.Timeline]]. The type parameters are:
     *  - T, a subclass of TimelineEntry with a `value` property specifying the value of the property controlled by the timeline at that entry's time point.
     *  - P, the JSON representation from which T is to be constructed.
     *  - V, the type of `T.value`.
     */
    class TimelineEntryList {
        constructor(props, ctor) {
            this.duration = geometry_core_1.Range1d.createNull();
            this._entries = props.map((x) => {
                const entry = new ctor(x);
                this.duration.extendX(entry.time);
                return entry;
            });
        }
        /** The number of entries in the list. */
        get length() {
            return this._entries.length;
        }
        /** An iterator over the entries in the list. */
        [Symbol.iterator]() {
            return this._entries[Symbol.iterator]();
        }
        /** Look up an entry by its position in the list. */
        getEntry(index) {
            return this._entries[index];
        }
        /** Look up the value of an entry by its position in the list. */
        getValue(index) {
            var _a;
            return (_a = this.getEntry(index)) === null || _a === void 0 ? void 0 : _a.value;
        }
        toJSON() {
            return this._entries.map((x) => x.toJSON());
        }
        /** @internal */
        findInterval(time, interval) {
            if (this.length === 0)
                return undefined;
            interval = interval !== null && interval !== void 0 ? interval : new Interval();
            if (time < this._entries[0].time) {
                interval.init(0, 0, 0);
                return interval;
            }
            const last = this.length - 1;
            if (time >= this._entries[last].time) {
                interval.init(last, last, 0);
                return interval;
            }
            for (let i = 0; i < last; i++) {
                const time0 = this._entries[i].time;
                const time1 = this._entries[i + 1].time;
                if (time0 <= time && time1 >= time) {
                    let fraction;
                    if (Interpolation.Linear === this._entries[i].interpolation)
                        fraction = (time - time0) / (time1 - time0);
                    else
                        fraction = 0;
                    interval.init(i, i + 1, fraction);
                    return interval;
                }
            }
            return undefined;
        }
    }
    RenderSchedule.TimelineEntryList = TimelineEntryList;
    const scratchInterval = new Interval();
    /** A list of [[RenderSchedule.VisibilityEntry]]s within a [[RenderSchedule.Timeline]]. */
    class VisibilityTimelineEntries extends TimelineEntryList {
        /** Returns the visibility value for the entry at the specified position in the list, or 100 (fully-visible) if no such entry exists. */
        getValue(index) {
            var _a;
            return (_a = super.getValue(index)) !== null && _a !== void 0 ? _a : 100;
        }
    }
    RenderSchedule.VisibilityTimelineEntries = VisibilityTimelineEntries;
    /** A list of [[RenderSchedule.TransformEntry]]s within a [[RenderSchedule.Timeline]]. */
    class TransformTimelineEntries extends TimelineEntryList {
        /** Returns the transform for the entry at the specified position in the list, or an identity transform if no such entry exists. */
        getValue(index) {
            var _a;
            return (_a = super.getValue(index)) !== null && _a !== void 0 ? _a : geometry_core_1.Transform.identity;
        }
    }
    RenderSchedule.TransformTimelineEntries = TransformTimelineEntries;
    /** Specifies how to animate a set of geometry over time within a [[RenderSchedule.Script]].
     * A [[RenderSchedule.Script]] can contain any number of [[RenderSchedule.Timeline]]s, each affecting different sets of geometry.
     * @see [[RenderSchedule.ElementTimeline]] and [[RenderSchedule.ModelTimeline]].
     */
    class Timeline {
        constructor(props) {
            this.duration = geometry_core_1.Range1d.createNull();
            if (props.visibilityTimeline) {
                this.visibility = new VisibilityTimelineEntries(props.visibilityTimeline, VisibilityEntry);
                this.duration.extendRange(this.visibility.duration);
            }
            if (props.colorTimeline) {
                this.color = new TimelineEntryList(props.colorTimeline, ColorEntry);
                this.duration.extendRange(this.color.duration);
            }
            if (props.transformTimeline) {
                this.transform = new TransformTimelineEntries(props.transformTimeline, TransformEntry);
                this.duration.extendRange(this.transform.duration);
            }
            if (props.cuttingPlaneTimeline) {
                this.cuttingPlane = new TimelineEntryList(props.cuttingPlaneTimeline, CuttingPlaneEntry);
                this.duration.extendRange(this.cuttingPlane.duration);
            }
        }
        toJSON() {
            var _a, _b, _c, _d;
            return {
                visibilityTimeline: (_a = this.visibility) === null || _a === void 0 ? void 0 : _a.toJSON(),
                colorTimeline: (_b = this.color) === null || _b === void 0 ? void 0 : _b.toJSON(),
                transformTimeline: (_c = this.transform) === null || _c === void 0 ? void 0 : _c.toJSON(),
                cuttingPlaneTimeline: (_d = this.cuttingPlane) === null || _d === void 0 ? void 0 : _d.toJSON(),
            };
        }
        /** Get the visibility of the geometry at the specified time point. */
        getVisibility(time) {
            var _a, _b;
            let interval;
            if (!this.visibility || !(interval = this.visibility.findInterval(time, scratchInterval)))
                return 100;
            let visibility = (_a = this.visibility.getValue(interval.lowerIndex)) !== null && _a !== void 0 ? _a : 100;
            if (interval.fraction > 0)
                visibility = interpolate(visibility, (_b = this.visibility.getValue(interval.upperIndex)) !== null && _b !== void 0 ? _b : 100, interval.fraction);
            return visibility;
        }
        /** Get the color of the geometry at the specified time point, or undefined if the color is not overridden at that time point. */
        getColor(time) {
            let interval;
            if (!this.color || !(interval = this.color.findInterval(time, scratchInterval)))
                return undefined;
            const start = this.color.getValue(interval.lowerIndex);
            if (start && interval.fraction > 0) {
                const end = this.color.getValue(interval.upperIndex);
                if (end)
                    return interpolateRgb(start, end, interval.fraction);
            }
            return start;
        }
        /** Get the transform applied to the geometry at the specified time point. */
        getAnimationTransform(time) {
            var _a, _b, _c, _d;
            let interval;
            if (!this.transform || !(interval = this.transform.findInterval(time, scratchInterval)))
                return geometry_core_1.Transform.identity;
            let transform = this.transform.getValue(interval.lowerIndex);
            if (interval.fraction > 0) {
                const comp0 = (_a = this.transform.getEntry(interval.lowerIndex)) === null || _a === void 0 ? void 0 : _a.components;
                const comp1 = (_b = this.transform.getEntry(interval.upperIndex)) === null || _b === void 0 ? void 0 : _b.components;
                if (comp0 && comp1) {
                    const sum = geometry_core_1.Point4d.interpolateQuaternions(comp0.orientation, interval.fraction, comp1.orientation);
                    const matrix = geometry_core_1.Matrix3d.createFromQuaternion(sum);
                    const pre = geometry_core_1.Transform.createTranslation(comp0.pivot);
                    const post = geometry_core_1.Transform.createTranslation(comp0.position.interpolate(interval.fraction, comp1.position));
                    const product = post.multiplyTransformMatrix3d(matrix);
                    product.multiplyTransformTransform(pre, product);
                    transform = product;
                }
                else {
                    const end = this.transform.getValue(interval.upperIndex);
                    const q0 = (_c = transform.matrix.inverse()) === null || _c === void 0 ? void 0 : _c.toQuaternion();
                    const q1 = (_d = end.matrix.inverse()) === null || _d === void 0 ? void 0 : _d.toQuaternion();
                    if (q0 && q1) {
                        const sum = geometry_core_1.Point4d.interpolateQuaternions(q0, interval.fraction, q1);
                        const matrix = geometry_core_1.Matrix3d.createFromQuaternion(sum);
                        const origin0 = geometry_core_1.Vector3d.createFrom(transform.origin);
                        const origin1 = geometry_core_1.Vector3d.createFrom(end.origin);
                        transform = geometry_core_1.Transform.createRefs(origin0.interpolate(interval.fraction, origin1), matrix);
                    }
                }
            }
            return transform;
        }
        /** Get the clipping plane applied to the geometry at the specified time point, or undefined if the geometry is unclipped at that time point. */
        getCuttingPlane(time) {
            let interval;
            if (!this.cuttingPlane || !(interval = this.cuttingPlane.findInterval(time, scratchInterval)))
                return undefined;
            const start = this.cuttingPlane.getValue(interval.lowerIndex);
            if (!start)
                return undefined;
            const position = geometry_core_1.Point3d.createFrom(start.position);
            const direction = geometry_core_1.Vector3d.createFrom(start.direction);
            const end = interval.fraction > 0 ? this.cuttingPlane.getValue(interval.upperIndex) : undefined;
            if (end) {
                position.interpolate(interval.fraction, end.position, position);
                direction.interpolate(interval.fraction, end.direction, direction);
            }
            else {
                if (start.hidden || start.visible)
                    return undefined;
            }
            direction.negate(direction);
            direction.normalizeInPlace();
            return geometry_core_1.Plane3dByOriginAndUnitNormal.create(position, direction);
        }
        /** Create a ClipVector from the [[RenderSchedule.CuttingPlane]] applied to the geometry at the specified time point, if any. */
        getClipVector(time) {
            const plane = this.getCuttingPlane(time);
            if (!plane)
                return undefined;
            const cp = geometry_core_1.ClipPlane.createPlane(plane);
            const cps = geometry_core_1.UnionOfConvexClipPlaneSets.createConvexSets([geometry_core_1.ConvexClipPlaneSet.createPlanes([cp])]);
            const prim = geometry_core_1.ClipPrimitive.createCapture(cps);
            return geometry_core_1.ClipVector.createCapture([prim]);
        }
        /** @internal */
        getFeatureAppearance(visibility, time) {
            const transparency = visibility < 100 ? (1 - visibility / 100) : undefined;
            const rgb = this.getColor(time);
            return undefined !== rgb || undefined !== transparency ? FeatureSymbology_1.FeatureAppearance.fromJSON({ rgb, transparency }) : undefined;
        }
    }
    RenderSchedule.Timeline = Timeline;
    /** Specifies how to animate the geometry belonging to a set of [GeometricElement]($backend)s as part of a [[RenderSchedule.Script]]. */
    class ElementTimeline extends Timeline {
        constructor(props) {
            super(props);
            this.batchId = props.batchId;
            this._elementIds = props.elementIds;
        }
        static fromJSON(props) {
            return new ElementTimeline(props !== null && props !== void 0 ? props : { elementIds: [], batchId: 0 });
        }
        toJSON() {
            return Object.assign(Object.assign({}, super.toJSON()), { batchId: this.batchId, elementIds: this._elementIds });
        }
        /** @internal */
        static getElementIds(ids) {
            if (typeof ids === "string")
                return bentleyjs_core_1.CompressedId64Set.iterable(ids);
            else if (Array.isArray(ids)) {
                return ids;
            }
            else {
                return [];
            }
        }
        /** The Ids of the elements controlled by this timeline. */
        get elementIds() {
            return ElementTimeline.getElementIds(this._elementIds);
        }
        /** True if this timeline affects the color or transparency of the elements. */
        get containsFeatureOverrides() {
            return undefined !== this.visibility || undefined !== this.color;
        }
        /** If true, applying this timeline requires special tiles to be generated in which groups of elements are batched into nodes.
         * @internal
         */
        get requiresBatching() {
            if (this.cuttingPlane)
                return true;
            return this.batchId !== 0 && (undefined !== this.color || undefined !== this.visibility);
        }
        /** True if this timeline affects the position, orientation, or scale of the elements. */
        get containsTransform() {
            return undefined !== this.transform;
        }
        /** @internal */
        addSymbologyOverrides(overrides, time) {
            bentleyjs_core_1.assert(0 !== this.batchId);
            const vis = this.getVisibility(time);
            if (vis <= 0) {
                overrides.setAnimationNodeNeverDrawn(this.batchId);
                return;
            }
            const appearance = this.getFeatureAppearance(vis, time);
            if (appearance)
                overrides.overrideAnimationNode(this.batchId, appearance);
        }
    }
    RenderSchedule.ElementTimeline = ElementTimeline;
    /** Specifies how to animate the geometry within a [GeometricModel]($backend) as part of a [[RenderSchedule.Script]]. */
    class ModelTimeline extends Timeline {
        constructor(props) {
            super(props);
            this.modelId = props.modelId;
            this.realityModelUrl = props.realityModelUrl;
            this.containsModelClipping = undefined !== this.cuttingPlane;
            let containsFeatureOverrides = undefined !== this.visibility || undefined !== this.color;
            let requiresBatching = false;
            let containsTransform = false;
            const transformBatchIds = [];
            const elementTimelines = [];
            for (const elProps of props.elementTimelines) {
                const el = ElementTimeline.fromJSON(elProps);
                elementTimelines.push(el);
                this.duration.extendRange(el.duration);
                if (el.containsTransform) {
                    containsTransform = true;
                    if (el.batchId)
                        transformBatchIds.push(el.batchId);
                }
                containsFeatureOverrides || (containsFeatureOverrides = el.containsFeatureOverrides);
                requiresBatching || (requiresBatching = el.requiresBatching);
            }
            this.elementTimelines = elementTimelines;
            this.transformBatchIds = transformBatchIds;
            this.containsFeatureOverrides = containsFeatureOverrides;
            this.requiresBatching = requiresBatching;
            this.containsTransform = containsTransform;
        }
        static fromJSON(props) {
            return new ModelTimeline(props !== null && props !== void 0 ? props : { elementTimelines: [], modelId: bentleyjs_core_1.Id64.invalid });
        }
        toJSON() {
            return Object.assign(Object.assign({}, super.toJSON()), { modelId: this.modelId, realityModelUrl: this.realityModelUrl, elementTimelines: this.elementTimelines.map((x) => x.toJSON()) });
        }
        /** Look up the element timeline with the specified batch Id. */
        findByBatchId(batchId) {
            return this.elementTimelines.find((x) => x.batchId === batchId);
        }
        /** @internal */
        addSymbologyOverrides(overrides, time) {
            const appearance = this.getFeatureAppearance(this.getVisibility(time), time);
            if (appearance)
                overrides.overrideModel(this.modelId, appearance);
            for (const timeline of this.elementTimelines)
                timeline.addSymbologyOverrides(overrides, time);
        }
        /** Obtain the transform applied to the model at the specified time point, if any. */
        getTransform(batchId, time) {
            var _a;
            return (_a = this.findByBatchId(batchId)) === null || _a === void 0 ? void 0 : _a.getAnimationTransform(time);
        }
    }
    RenderSchedule.ModelTimeline = ModelTimeline;
    /** Specifies how to animate the contents of a [ViewState]($frontend) over time. The script contains any number of [[RenderSchedule.ModelTimeline]]s, each describing how
     * to animate one of the models in the view.
     * @see [RenderTimeline]($backend) to create an [Element]($backend) to host a script.
     * @see [[DisplayStyleSettings.renderTimeline]] to associate a [RenderTimeline]($backend)'s script with a [DisplayStyle]($backend).
     * @see [DisplayStyleState.scheduleScript]($frontend) to obtain the script associated with a display style.
     * @see [[RenderSchedule.ScriptBuilder]] to define a new script.
     */
    class Script {
        constructor(props) {
            this.duration = geometry_core_1.Range1d.createNull();
            const modelTimelines = [];
            let containsModelClipping = false;
            let requiresBatching = false;
            let containsTransform = false;
            let containsFeatureOverrides = false;
            for (const modelProps of props) {
                const model = ModelTimeline.fromJSON(modelProps);
                modelTimelines.push(model);
                this.duration.extendRange(model.duration);
                containsModelClipping || (containsModelClipping = model.containsModelClipping);
                requiresBatching || (requiresBatching = model.requiresBatching);
                containsTransform || (containsTransform = model.containsTransform);
                containsFeatureOverrides || (containsFeatureOverrides = model.containsFeatureOverrides);
            }
            this.modelTimelines = modelTimelines;
            this.containsModelClipping = containsModelClipping;
            this.containsTransform = containsTransform;
            this.requiresBatching = requiresBatching || this.containsTransform;
            this.containsFeatureOverrides = containsFeatureOverrides;
        }
        static fromJSON(props) {
            if (!Array.isArray(props) || props.length === 0)
                return undefined;
            return new Script(props);
        }
        toJSON() {
            return this.modelTimelines.map((x) => x.toJSON());
        }
        /** Look up the timeline that animates the specified model, if any. */
        find(modelId) {
            return this.modelTimelines.find((x) => x.modelId === modelId);
        }
        /** @internal */
        getTransformBatchIds(modelId) {
            var _a;
            return (_a = this.find(modelId)) === null || _a === void 0 ? void 0 : _a.transformBatchIds;
        }
        /** @internal */
        getTransform(modelId, batchId, time) {
            var _a;
            return (_a = this.find(modelId)) === null || _a === void 0 ? void 0 : _a.getTransform(batchId, time);
        }
        /** @internal */
        addSymbologyOverrides(overrides, time) {
            for (const timeline of this.modelTimelines)
                timeline.addSymbologyOverrides(overrides, time);
        }
        /** Used by collectPredecessorIds methods of RenderTimeline and DisplayStyle.
         * @internal
         */
        discloseIds(ids) {
            for (const model of this.modelTimelines) {
                ids.add(model.modelId);
                for (const element of model.elementTimelines)
                    for (const id of element.elementIds)
                        ids.add(id);
            }
        }
    }
    RenderSchedule.Script = Script;
    /** A reference to a [[RenderSchedule.Script]] indicating the persistent [Element]($backend) from which the script was obtained.
     * Prior to the introduction of the [RenderTimeline]($backend) class in version 01.00.13 of the BisCore ECSchema, scripts were
     * stored in the JSON properties of [DisplayStyle]($backend) elements. Now they are stored in the Script property of a RenderTimeline element.
     * The `sourceId` can refer to either a DisplayStyle or a RenderTimeline.
     */
    class ScriptReference {
        constructor(sourceId, script) {
            this.sourceId = sourceId;
            this.script = script;
        }
    }
    RenderSchedule.ScriptReference = ScriptReference;
    /** Used as part of a [[RenderSchedule.ScriptBuilder]] to define a [[RenderSchedule.Timeline]].
     * @see [[RenderSchedule.ElementTimelineBuilder]] and [[RenderSchedule.ModelTimelineBuilder]].
     */
    class TimelineBuilder {
        /** Append a new [[RenderSchedule.VisibilityEntry]] to the timeline. */
        addVisibility(time, visibility, interpolation = Interpolation.Linear) {
            if (!this.visibility)
                this.visibility = [];
            this.visibility.push({ time, value: visibility, interpolation });
        }
        /** Append a new [[RenderSchedule.ColorEntry]] to the timeline. */
        addColor(time, color, interpolation = Interpolation.Linear) {
            if (!this.color)
                this.color = [];
            const value = color instanceof RgbColor_1.RgbColor ? { red: color.r, green: color.g, blue: color.b } : color;
            this.color.push({ time, value, interpolation });
        }
        /** Append a new [[RenderSchedule.CuttingPlaneEntry]] to the timeline. */
        addCuttingPlane(time, plane, interpolation = Interpolation.Linear) {
            if (!this.cuttingPlane)
                this.cuttingPlane = [];
            let value;
            if (plane) {
                value = {
                    position: [plane.position.x, plane.position.y, plane.position.z],
                    direction: [plane.direction.x, plane.direction.y, plane.direction.z],
                };
                if (plane.visible)
                    value.visible = true;
                if (plane.hidden)
                    value.hidden = true;
            }
            this.cuttingPlane.push({ time, value, interpolation });
        }
        /** Append a new [[RenderSchedule.TransformEntry]] to the timeline. */
        addTransform(time, transform, components, interpolation = Interpolation.Linear) {
            if (!this.transform)
                this.transform = [];
            const value = { transform: transform === null || transform === void 0 ? void 0 : transform.toRows() };
            if (components) {
                value.pivot = [components.pivot.x, components.pivot.y, components.pivot.z];
                value.orientation = components.orientation.toJSON();
                value.position = [components.position.x, components.position.y, components.position.z];
            }
            this.transform.push({ time, value, interpolation });
        }
        /** Obtain the JSON representation of the [[RenderSchedule.Timeline]] produced by this builder.
         * @see [[RenderSchedule.ScriptBuilder.finish]] to obtain the JSON for the entire [[RenderSchedule.Script]].
         */
        finish() {
            var _a, _b, _c, _d;
            const props = {};
            if ((_a = this.visibility) === null || _a === void 0 ? void 0 : _a.length)
                props.visibilityTimeline = this.visibility;
            if ((_b = this.color) === null || _b === void 0 ? void 0 : _b.length)
                props.colorTimeline = this.color;
            if ((_c = this.transform) === null || _c === void 0 ? void 0 : _c.length)
                props.transformTimeline = this.transform;
            if ((_d = this.cuttingPlane) === null || _d === void 0 ? void 0 : _d.length)
                props.cuttingPlaneTimeline = this.cuttingPlane;
            return props;
        }
    }
    RenderSchedule.TimelineBuilder = TimelineBuilder;
    /** As part of a [[RenderSchedule.ScriptBuilder]], assembles a [[RenderSchedule.ElementTimeline]].
     * @see [[RenderSchedule.ModelTimelineBuilder.addElementTimeline]].
     */
    class ElementTimelineBuilder extends TimelineBuilder {
        /** Constructor - typically not used directly.
         * @see [[RenderSchedule.ModelTimelineBuilder.addElementTimeline]] to create an ElementTimelineBuilder.
         */
        constructor(batchId, elementIds) {
            super();
            this.batchId = batchId;
            this.elementIds = elementIds;
        }
        /** Obtain the JSON representation of the [[RenderSchedule.ElementTimeline]] produced by this builder.
         * @see [[RenderSchedule.ScriptBuilder.finish]] to obtain the JSON for the entire [[RenderSchedule.Script]].
         */
        finish() {
            const props = super.finish();
            props.batchId = this.batchId;
            props.elementIds = this.elementIds;
            return props;
        }
    }
    RenderSchedule.ElementTimelineBuilder = ElementTimelineBuilder;
    /** As part of a [[RenderSchedule.ScriptBuilder, assembles a [[RenderSchedule.ModelTimeline]].
     * @see [[RenderSchedule.ScriptBuilder.addModelTimeline]].
     */
    class ModelTimelineBuilder extends TimelineBuilder {
        /** Constructor - typically not used directly.
         * @see [[RenderSchedule.ScriptBuilder.addModelTimeline]] to create a ModelTimelineBuilder.
         */
        constructor(modelId, obtainNextBatchId) {
            super();
            this._elements = [];
            this.modelId = modelId;
            this._obtainNextBatchId = obtainNextBatchId;
        }
        /** Add a new [[RenderSchedule.ElementTimeline]] to be applied to the specified elements.
         * This function will sort and compress the Ids if they are not already compressed.
         *
         */
        addElementTimeline(elementIds) {
            const batchId = this._obtainNextBatchId();
            let ids;
            if (typeof elementIds === "string") {
                ids = elementIds;
            }
            else {
                const sorted = Array.from(elementIds);
                bentleyjs_core_1.OrderedId64Iterable.sortArray(sorted);
                ids = bentleyjs_core_1.CompressedId64Set.compressIds(sorted);
            }
            const builder = new ElementTimelineBuilder(batchId, ids);
            this._elements.push(builder);
            return builder;
        }
        /** Obtain the JSON representation of the [[RenderSchedule.ModelTimeline]] produced by this builder.
         * @see [[RenderSchedule.ScriptBuilder.finish]] to obtain the JSON for the entire [[RenderSchedule.Script]].
         */
        finish() {
            const props = super.finish();
            props.modelId = this.modelId;
            if (undefined !== this.realityModelUrl)
                props.realityModelUrl = this.realityModelUrl;
            props.elementTimelines = this._elements.map((x) => x.finish());
            return props;
        }
    }
    RenderSchedule.ModelTimelineBuilder = ModelTimelineBuilder;
    /** Assembles the JSON representation for a new [[RenderSchedule.Script]]. As an extremely simple example, the following code produces a script that changes the color of a single element:
     * ```ts
     *  const script = new ScriptBuilder();
     *  const model = script.addModelTimeline("0x123");
     *  const element = model.addElementTimeline([ "0x456" ]);
     *  element.addColor(Date.now(), new RgbColor(0xff, 0x7f, 0));
     *  const scriptProps = script.finish();
     * ```
     */
    class ScriptBuilder {
        constructor() {
            this._nextBatchId = 1;
            this._models = [];
        }
        /** Add a new [[RenderSchedule.ModelTimeline]] to be applied to the specified model. */
        addModelTimeline(modelId) {
            const builder = new ModelTimelineBuilder(modelId, () => this._nextBatchId++);
            this._models.push(builder);
            return builder;
        }
        /** Obtain the JSON representation of the [[RenderSchedule.Script]] produced by this builder.
         * @see [RenderTimeline.scriptProps]($backend) to assign the new script to a RenderTimeline element.
         */
        finish() {
            return this._models.map((x) => x.finish());
        }
    }
    RenderSchedule.ScriptBuilder = ScriptBuilder;
})(RenderSchedule = exports.RenderSchedule || (exports.RenderSchedule = {}));
//# sourceMappingURL=RenderSchedule.js.map