"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Rendering
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RenderTexture = void 0;
/** Represents a texture image applied to a surface during rendering.
 * A RenderTexture is typically - but not always - associated with a [[RenderMaterial]].
 * @see [RenderSystem.createTextureFromImage]($frontend) to obtain a texture from an HTML image.
 * @see [RenderSystem.createTextureFromElement]($frontend) to obtain a texture from a [Texture]($backend) element.
 * @public
 */
class RenderTexture {
    constructor(params) {
        this.key = params.key;
        this.type = params.type;
        this.isOwned = params.isOwned;
    }
    get isTileSection() { return RenderTexture.Type.TileSection === this.type; }
    get isGlyph() { return RenderTexture.Type.Glyph === this.type; }
    get isSkyBox() { return RenderTexture.Type.SkyBox === this.type; }
}
exports.RenderTexture = RenderTexture;
/** @public */
(function (RenderTexture) {
    /** Enumerates the types of [[RenderTexture]]s. */
    let Type;
    (function (Type) {
        /** An image applied to a surface, with support for mip-mapping and repeating. */
        Type[Type["Normal"] = 0] = "Normal";
        /** An image containing any number of text glyphs, used for efficiently rendering readable small text. */
        Type[Type["Glyph"] = 1] = "Glyph";
        /** A non-repeating image with no mip-maps, used for example for reality models. */
        Type[Type["TileSection"] = 2] = "TileSection";
        /** A three-dimensional texture used for rendering a skybox. */
        Type[Type["SkyBox"] = 3] = "SkyBox";
        /** A non-repeating image with mip-maps and and anisotropic filtering, used for map tiles when draped on terrain. */
        Type[Type["FilteredTileSection"] = 4] = "FilteredTileSection";
        /** A thematic gradient image used for thematic display. */
        Type[Type["ThematicGradient"] = 5] = "ThematicGradient";
    })(Type = RenderTexture.Type || (RenderTexture.Type = {}));
    /** Parameters used to construct a [[RenderTexture]]. */
    class Params {
        constructor(key, type = Type.Normal, isOwned = false) {
            this.key = key;
            this.type = type;
            this.isOwned = isOwned;
        }
        get isTileSection() { return Type.TileSection === this.type; }
        get isGlyph() { return Type.Glyph === this.type; }
        get isSkyBox() { return Type.SkyBox === this.type; }
    }
    /** Obtain a RenderTexture params object with default values. */
    Params.defaults = new Params();
    RenderTexture.Params = Params;
})(RenderTexture = exports.RenderTexture || (exports.RenderTexture = {}));
//# sourceMappingURL=RenderTexture.js.map