"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RpcInterface
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcInterface = void 0;
const semver = require("semver");
const RpcConfiguration_1 = require("./rpc/core/RpcConfiguration");
const RpcRegistry_1 = require("./rpc/core/RpcRegistry");
const RpcRoutingToken_1 = require("./rpc/core/RpcRoutingToken");
/** An RPC interface is a set of operations exposed by a service that a client can call, using configurable protocols,
 * in a platform-independent way. TheRpcInterface class is the base class for RPC interface definitions and implementations.
 * @public
 */
class RpcInterface {
    /** @alpha */
    constructor(routing = RpcRoutingToken_1.RpcRoutingToken.default) {
        this.routing = routing;
        this.configuration = RpcConfiguration_1.RpcConfiguration.supply(this);
    }
    /** Determines whether the backend version of an RPC interface is compatible (according to semantic versioning) with the frontend version of the interface. */
    static isVersionCompatible(backend, frontend) {
        const difference = semver.diff(backend, frontend);
        if (semver.prerelease(backend) || semver.prerelease(frontend)) {
            return difference === null;
        }
        else if (semver.major(backend) === 0 || semver.major(frontend) === 0) {
            return difference === null || (difference === "patch" && semver.patch(frontend) < semver.patch(backend));
        }
        else {
            return difference === null || difference === "patch" || (difference === "minor" && semver.minor(frontend) < semver.minor(backend));
        }
    }
    /** Obtains the implementation result for an RPC operation. */
    async forward(parameters) {
        const parametersCompat = (arguments.length === 1 && typeof (parameters) === "object") ? parameters : arguments;
        const parametersArray = Array.isArray(parametersCompat) ? parametersCompat : Array.prototype.slice.call(parametersCompat);
        const operationName = parametersArray.pop();
        const request = new this.configuration.protocol.requestType(this, operationName, parametersArray);
        request.submit(); // eslint-disable-line @typescript-eslint/no-floating-promises
        this[RpcRegistry_1.CURRENT_REQUEST] = request;
        return request.response;
    }
}
exports.RpcInterface = RpcInterface;
RpcInterface.prototype.configurationSupplier = undefined;
//# sourceMappingURL=RpcInterface.js.map