"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TerrainSettings = exports.TerrainHeightOriginMode = void 0;
/** Correction modes for terrain height
 * @see [[TerrainProps]]
 * @public
 */
var TerrainHeightOriginMode;
(function (TerrainHeightOriginMode) {
    /** Height value indicates the geodetic height of the IModel origin (also referred to as ellipsoidal or GPS height) */
    TerrainHeightOriginMode[TerrainHeightOriginMode["Geodetic"] = 0] = "Geodetic";
    /** Height value indicates the geoidal height of the IModel origin (commonly referred to as sea level). */
    TerrainHeightOriginMode[TerrainHeightOriginMode["Geoid"] = 1] = "Geoid";
    /** Height value indicates the height of the IModel origin relative to ground level at project center. */
    TerrainHeightOriginMode[TerrainHeightOriginMode["Ground"] = 2] = "Ground";
})(TerrainHeightOriginMode = exports.TerrainHeightOriginMode || (exports.TerrainHeightOriginMode = {}));
/** Normalized version of [[TerrainProps]] for which provider has been validated and default values of all members are used.
 * @public
 */
class TerrainSettings {
    constructor(providerName = "CesiumWorldTerrain", exaggeration = 1.0, applyLighting = false, heightOrigin = 0.0, heightOriginMode = TerrainHeightOriginMode.Geodetic, locatable = true) {
        this.providerName = providerName;
        this.exaggeration = Math.min(100, Math.max(0.1, exaggeration));
        this.locatable = locatable; // eslint-disable-line deprecation/deprecation
        this.applyLighting = applyLighting;
        this.heightOrigin = heightOrigin;
        switch (heightOriginMode) {
            case TerrainHeightOriginMode.Ground:
            case TerrainHeightOriginMode.Geoid:
                this.heightOriginMode = heightOriginMode;
                break;
            default:
                this.heightOriginMode = TerrainHeightOriginMode.Geodetic;
                break;
        }
    }
    static fromJSON(json) {
        if (undefined === json)
            return new TerrainSettings();
        const providerName = "CesiumWorldTerrain"; // This is only terrain provider currently supported.
        return new TerrainSettings(providerName, json.exaggeration, json.applyLighting, json.heightOrigin, json.heightOriginMode, true !== json.nonLocatable); // eslint-disable-line deprecation/deprecation
    }
    toJSON() {
        const props = { heightOriginMode: this.heightOriginMode };
        if ("CesiumWorldTerrain" !== this.providerName)
            props.providerName = this.providerName;
        if (1 !== this.exaggeration)
            props.exaggeration = this.exaggeration;
        if (!this.locatable) // eslint-disable-line deprecation/deprecation
            props.nonLocatable = true; // eslint-disable-line deprecation/deprecation
        if (this.applyLighting)
            props.applyLighting = true;
        if (0 !== this.heightOrigin)
            props.heightOrigin = this.heightOrigin;
        return props;
    }
    equals(other) {
        return this.providerName === other.providerName && this.exaggeration === other.exaggeration && this.applyLighting === other.applyLighting
            && this.heightOrigin === other.heightOrigin && this.heightOriginMode === other.heightOriginMode && this.locatable === other.locatable; // eslint-disable-line deprecation/deprecation
    }
    /** Returns true if these settings are equivalent to the supplied JSON settings. */
    equalsJSON(json) {
        return this.equals(TerrainSettings.fromJSON(json));
    }
    /** Create a copy of this TerrainSettings, optionally modifying some of its properties.
     * @param changedProps JSON representation of the properties to change.
     * @returns A TerrainSettings with all of its properties set to match those of`this`, except those explicitly defined in `changedProps`.
     */
    clone(changedProps) {
        var _a, _b, _c, _d, _e, _f;
        if (undefined === changedProps)
            return this;
        const props = {
            providerName: (_a = changedProps.providerName) !== null && _a !== void 0 ? _a : this.providerName,
            exaggeration: (_b = changedProps.exaggeration) !== null && _b !== void 0 ? _b : this.exaggeration,
            nonLocatable: (_c = changedProps.nonLocatable) !== null && _c !== void 0 ? _c : !this.locatable,
            applyLighting: (_d = changedProps.applyLighting) !== null && _d !== void 0 ? _d : this.applyLighting,
            heightOrigin: (_e = changedProps.heightOrigin) !== null && _e !== void 0 ? _e : this.heightOrigin,
            heightOriginMode: (_f = changedProps.heightOriginMode) !== null && _f !== void 0 ? _f : this.heightOriginMode,
        };
        return TerrainSettings.fromJSON(props);
    }
}
exports.TerrainSettings = TerrainSettings;
//# sourceMappingURL=TerrainSettings.js.map