"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Symbology
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThematicDisplay = exports.ThematicDisplayMode = exports.ThematicDisplaySensorSettings = exports.ThematicDisplaySensor = exports.ThematicGradientSettings = exports.ThematicGradientColorScheme = exports.ThematicGradientMode = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const geometry_core_1 = require("@bentley/geometry-core");
const ColorDef_1 = require("./ColorDef");
const Gradient_1 = require("./Gradient");
/** A thematic gradient mode used to generate and apply a thematic effect to a scene.
 * @see [[ThematicGradientSettings.mode]]
 * @public
 */
var ThematicGradientMode;
(function (ThematicGradientMode) {
    /** Apply a smooth color gradient to the scene. */
    ThematicGradientMode[ThematicGradientMode["Smooth"] = 0] = "Smooth";
    /** Apply a stepped color gradient to the scene. */
    ThematicGradientMode[ThematicGradientMode["Stepped"] = 1] = "Stepped";
    /** Apply a stepped color gradient to the scene with delimiters (lines between the color steps). Can only be used with [[ThematicDisplayMode.Height]]. */
    ThematicGradientMode[ThematicGradientMode["SteppedWithDelimiter"] = 2] = "SteppedWithDelimiter";
    /** Apply isolines to the scene to achieve an effect similar to a contour map. Can only be used with [[ThematicDisplayMode.Height]]. */
    ThematicGradientMode[ThematicGradientMode["IsoLines"] = 3] = "IsoLines";
})(ThematicGradientMode = exports.ThematicGradientMode || (exports.ThematicGradientMode = {}));
/** A color scheme used to generate the colors of a thematic gradient within an applied range.
 * @see [[ThematicGradientSettings.colorScheme]]
 * @see [[ThematicDisplay.range]]
 * @public */
var ThematicGradientColorScheme;
(function (ThematicGradientColorScheme) {
    /** A color gradient scheme that represents a blue-to-red gradation. */
    ThematicGradientColorScheme[ThematicGradientColorScheme["BlueRed"] = 0] = "BlueRed";
    /** A color gradient scheme that represents a red-to-blue gradation. */
    ThematicGradientColorScheme[ThematicGradientColorScheme["RedBlue"] = 1] = "RedBlue";
    /** A color gradient scheme that represents a monochrome (black-to-white) gradation. */
    ThematicGradientColorScheme[ThematicGradientColorScheme["Monochrome"] = 2] = "Monochrome";
    /** A color gradient scheme that suits a topographic gradation. */
    ThematicGradientColorScheme[ThematicGradientColorScheme["Topographic"] = 3] = "Topographic";
    /** A color gradient scheme that suits a sea-mountain gradation. */
    ThematicGradientColorScheme[ThematicGradientColorScheme["SeaMountain"] = 4] = "SeaMountain";
    /** A custom color gradient scheme configured by the user.
     * @see [[ThematicGradientSettings.customKeys]]
     */
    ThematicGradientColorScheme[ThematicGradientColorScheme["Custom"] = 5] = "Custom";
})(ThematicGradientColorScheme = exports.ThematicGradientColorScheme || (exports.ThematicGradientColorScheme = {}));
/** Thematic settings specific to creating a color gradient used by [[ThematicDisplay]].
 * @public
 */
class ThematicGradientSettings {
    constructor(json) {
        var _a;
        this.customKeys = [];
        if (undefined === json) {
            this.mode = ThematicGradientMode.Smooth;
            this.stepCount = 10;
            this.marginColor = ColorDef_1.ColorDef.fromJSON();
            this.colorScheme = ThematicGradientColorScheme.BlueRed;
            this.colorMix = 0.0;
        }
        else {
            this.mode = (json.mode !== undefined && json.mode !== null) ? json.mode : ThematicGradientMode.Smooth;
            if (this.mode < ThematicGradientMode.Smooth || this.mode > ThematicGradientMode.IsoLines)
                this.mode = ThematicGradientMode.Smooth;
            this.stepCount = (typeof json.stepCount === "number") ? json.stepCount : 10;
            if (this.stepCount < 2)
                this.stepCount = 2;
            this.marginColor = ColorDef_1.ColorDef.fromJSON(json.marginColor);
            this.colorScheme = (json.colorScheme !== undefined && json.colorScheme !== null) ? json.colorScheme : ThematicGradientColorScheme.BlueRed;
            if (this.colorScheme < ThematicGradientColorScheme.BlueRed || this.colorScheme > ThematicGradientColorScheme.Custom)
                this.colorScheme = ThematicGradientColorScheme.BlueRed;
            if (json.customKeys !== undefined && json.customKeys !== null)
                json.customKeys.forEach((key) => this.customKeys.push(new Gradient_1.Gradient.KeyColor(key)));
            // Enforce 2 entries in custom color keys if violated
            if (this.colorScheme === ThematicGradientColorScheme.Custom && this.customKeys.length < 2) {
                this.customKeys = [];
                for (const keyValue of ThematicGradientSettings._defaultCustomKeys)
                    this.customKeys.push(new Gradient_1.Gradient.KeyColor({ value: keyValue[0], color: ColorDef_1.ColorDef.computeTbgrFromComponents(keyValue[1], keyValue[3], keyValue[2]) }));
            }
            this.colorMix = (_a = json.colorMix) !== null && _a !== void 0 ? _a : 0.0;
        }
    }
    static get margin() { return .001; } // A fixed portion of the gradient for out of range values.
    static get contentRange() { return 1.0 - 2.0 * ThematicGradientSettings.margin; }
    static get contentMax() { return 1.0 - ThematicGradientSettings.margin; }
    equals(other) {
        if (this.mode !== other.mode)
            return false;
        if (this.stepCount !== other.stepCount)
            return false;
        if (!this.marginColor.equals(other.marginColor))
            return false;
        if (this.colorScheme !== other.colorScheme)
            return false;
        if (this.customKeys.length !== other.customKeys.length)
            return false;
        if (this.colorMix !== other.colorMix)
            return false;
        for (let i = 0; i < this.customKeys.length; i++) {
            if (!Gradient_1.Gradient.keyColorEquals(this.customKeys[i], other.customKeys[i]))
                return false;
        }
        return true;
    }
    /** Compares two sets of thematic gradient settings.
     * @param lhs First set of thematic gradient settings to compare
     * @param rhs Second set of thematic gradient settings to compare
     * @returns 0 if lhs is equivalent to rhs, a negative number if lhs compares less than rhs, or a positive number if lhs compares greater than rhs.
     */
    static compare(lhs, rhs) {
        let diff = 0;
        if ((diff = bentleyjs_core_1.compareNumbers(lhs.mode, rhs.mode)) !== 0)
            return diff;
        if ((diff = bentleyjs_core_1.compareNumbers(lhs.stepCount, rhs.stepCount)) !== 0)
            return diff;
        if ((diff = bentleyjs_core_1.compareNumbers(lhs.marginColor.tbgr, rhs.marginColor.tbgr)) !== 0)
            return diff;
        if ((diff = bentleyjs_core_1.compareNumbers(lhs.colorScheme, rhs.colorScheme)) !== 0)
            return diff;
        if ((diff = bentleyjs_core_1.compareNumbers(lhs.colorMix, rhs.colorMix)) !== 0)
            return diff;
        if ((diff = bentleyjs_core_1.compareNumbers(lhs.customKeys.length, rhs.customKeys.length)) !== 0)
            return diff;
        for (let i = 0; i < lhs.customKeys.length; i++) {
            if ((diff = bentleyjs_core_1.compareNumbers(lhs.customKeys[i].color.tbgr, rhs.customKeys[i].color.tbgr)) !== 0)
                return diff;
        }
        return diff;
    }
    static fromJSON(json) {
        return json ? new ThematicGradientSettings(json) : this.defaults;
    }
    toJSON() {
        const props = {};
        if (ThematicGradientMode.Smooth !== this.mode)
            props.mode = this.mode;
        if (10 !== this.stepCount)
            props.stepCount = this.stepCount;
        const marginColor = this.marginColor.toJSON();
        if (0 !== marginColor)
            props.marginColor = marginColor;
        if (ThematicGradientColorScheme.BlueRed !== this.colorScheme)
            props.colorScheme = this.colorScheme;
        if (0 !== this.colorMix)
            props.colorMix = this.colorMix;
        if (this.customKeys.length > 0)
            props.customKeys = this.customKeys.map((key) => { return { value: key.value, color: key.color.toJSON() }; });
        return props;
    }
    /** Create a copy of this ThematicGradientSettings, optionally modifying some of its properties.
     * @param changedProps JSON representation of the properties to change.
     * @returns A ThematicGradientSettings with all of its properties set to match those of `this`, except those explicitly defined in `changedProps`.
     */
    clone(changedProps) {
        if (undefined === changedProps)
            return ThematicGradientSettings.fromJSON(this.toJSON());
        const props = {
            mode: undefined !== changedProps.mode ? changedProps.mode : this.mode,
            stepCount: undefined !== changedProps.stepCount ? changedProps.stepCount : this.stepCount,
            marginColor: undefined !== changedProps.marginColor ? changedProps.marginColor : this.marginColor.tbgr,
            colorScheme: undefined !== changedProps.colorScheme ? changedProps.colorScheme : this.colorScheme,
            customKeys: undefined !== changedProps.customKeys ? changedProps.customKeys : this.customKeys.map((key) => ({ value: key.value, color: key.color.tbgr })),
            colorMix: undefined !== changedProps.colorMix ? changedProps.colorMix : this.colorMix,
        };
        return ThematicGradientSettings.fromJSON(props);
    }
}
exports.ThematicGradientSettings = ThematicGradientSettings;
ThematicGradientSettings.defaults = new ThematicGradientSettings({});
ThematicGradientSettings._defaultCustomKeys = [[0.0, 255, 255, 255], [1.0, 0, 0, 0]];
/** A sensor in world space, used for [[ThematicDisplayMode.InverseDistanceWeightedSensors]].
 * @public
 */
class ThematicDisplaySensor {
    constructor(json) {
        if (undefined === json) {
            this.position = geometry_core_1.Point3d.fromJSON();
            this.value = 0;
        }
        else {
            this.position = geometry_core_1.Point3d.fromJSON(json.position);
            this.value = (typeof json.value !== "number") ? 0 : json.value;
            if (this.value < 0)
                this.value = 0;
            else if (this.value > 1)
                this.value = 1;
        }
    }
    equals(other) {
        return (this.value === other.value) && this.position.isAlmostEqual(other.position);
    }
    static fromJSON(json) {
        return new ThematicDisplaySensor(json);
    }
    toJSON() {
        return {
            position: this.position.toJSON(),
            value: this.value,
        };
    }
}
exports.ThematicDisplaySensor = ThematicDisplaySensor;
/** Settings for sensor-based thematic display for [[ThematicDisplayMode.InverseDistanceWeightedSensors]].
 * @public
 */
class ThematicDisplaySensorSettings {
    constructor(json) {
        this.sensors = [];
        if (undefined !== json) {
            if (json.sensors !== undefined && json.sensors !== null) {
                json.sensors.forEach((sensorJSON) => this.sensors.push(ThematicDisplaySensor.fromJSON(sensorJSON)));
            }
            this.distanceCutoff = (typeof json.distanceCutoff === "number") ? json.distanceCutoff : 0;
        }
        else {
            this.distanceCutoff = 0;
        }
    }
    equals(other) {
        if (this.distanceCutoff !== other.distanceCutoff)
            return false;
        const thisSensors = this.sensors;
        const otherSensors = other.sensors;
        if (thisSensors.length !== otherSensors.length)
            return false;
        for (let i = 0; i < thisSensors.length; i++) {
            if (!thisSensors[i].equals(otherSensors[i]))
                return false;
        }
        return true;
    }
    static fromJSON(json) {
        return new ThematicDisplaySensorSettings(json);
    }
    toJSON() {
        const json = {};
        json.sensors = [];
        this.sensors.forEach((sensor) => json.sensors.push(sensor.toJSON()));
        json.distanceCutoff = this.distanceCutoff;
        return json;
    }
}
exports.ThematicDisplaySensorSettings = ThematicDisplaySensorSettings;
/** The thematic display mode. This determines how to apply the thematic color gradient to the geometry.
 * @public
 */
var ThematicDisplayMode;
(function (ThematicDisplayMode) {
    /** The color gradient will be mapped to surface geometry and point clouds based on world height in meters. */
    ThematicDisplayMode[ThematicDisplayMode["Height"] = 0] = "Height";
    /** The color gradient will be mapped to surface geometry and point clouds using inverse distance weighting based on world positions and corresponding values from a list of sensors.
     * @note Performance will decrease as more sensors are added.
     */
    ThematicDisplayMode[ThematicDisplayMode["InverseDistanceWeightedSensors"] = 1] = "InverseDistanceWeightedSensors";
    /** The color gradient will be mapped to surface geometry based on the slope of the surface. The slope value is calculated based on the angle between the surface and the axis specified in the associated [[ThematicDisplay]] object.
     * @note This display mode does not affect point clouds.
     */
    ThematicDisplayMode[ThematicDisplayMode["Slope"] = 2] = "Slope";
    /** The color gradient will be mapped to surface geometry based on the direction of a sun shining on the surface.
     * @note This display mode does not affect point clouds.
     */
    ThematicDisplayMode[ThematicDisplayMode["HillShade"] = 3] = "HillShade";
})(ThematicDisplayMode = exports.ThematicDisplayMode || (exports.ThematicDisplayMode = {}));
/** The thematic display setup of a [[DisplayStyle3d]].
 * Thematic display allows a user to colorize a scene using a color gradient in a way that provides a visual cue about certain attributes of the rendered geometry. This scene colorization will be done based on certain geometric attributes like height, surface slope, position of surfaces relative to a sun position, or geometric position relative to a list of sensors.
 * The documentation for [[ThematicDisplayMode]] describes how each mode colorizes the scene.
 * @public
 */
class ThematicDisplay {
    constructor(json) {
        if (undefined === json) {
            this.displayMode = ThematicDisplayMode.Height;
            this.gradientSettings = ThematicGradientSettings.fromJSON();
            this.axis = geometry_core_1.Vector3d.fromJSON();
            this.range = geometry_core_1.Range1d.fromJSON();
            this.sunDirection = geometry_core_1.Vector3d.fromJSON();
            this.sensorSettings = ThematicDisplaySensorSettings.fromJSON();
        }
        else {
            this.displayMode = (json.displayMode !== undefined && json.displayMode !== null) ? json.displayMode : ThematicDisplayMode.Height;
            if (this.displayMode < ThematicDisplayMode.Height || this.displayMode > ThematicDisplayMode.HillShade)
                this.displayMode = ThematicDisplayMode.Height;
            this.gradientSettings = ThematicGradientSettings.fromJSON(json.gradientSettings);
            this.axis = geometry_core_1.Vector3d.fromJSON(json.axis);
            this.range = geometry_core_1.Range1d.fromJSON(json.range);
            this.sunDirection = geometry_core_1.Vector3d.fromJSON(json.sunDirection);
            this.sensorSettings = ThematicDisplaySensorSettings.fromJSON(json.sensorSettings);
        }
        if (ThematicDisplayMode.Height !== this.displayMode) {
            // Disallow isoline and stepped-with-delimiter gradient modes in any mode other than height.
            if (ThematicGradientMode.IsoLines === this.gradientSettings.mode || ThematicGradientMode.SteppedWithDelimiter === this.gradientSettings.mode) {
                const gradientSettingsJSON = this.gradientSettings.toJSON();
                gradientSettingsJSON.mode = ThematicGradientMode.Smooth;
                this.gradientSettings = ThematicGradientSettings.fromJSON(gradientSettingsJSON);
            }
            if (ThematicDisplayMode.Slope === this.displayMode) {
                if (this.range.low < 0.0)
                    this.range.low = 0.0;
                if (this.range.high > 90.0)
                    this.range.high = 90.0;
            }
        }
    }
    equals(other) {
        if (this.displayMode !== other.displayMode)
            return false;
        if (!this.gradientSettings.equals(other.gradientSettings))
            return false;
        if (!this.range.isAlmostEqual(other.range))
            return false;
        if (!this.axis.isAlmostEqual(other.axis))
            return false;
        if (!this.sunDirection.isAlmostEqual(other.sunDirection))
            return false;
        if (!this.sensorSettings.equals(other.sensorSettings))
            return false;
        return true;
    }
    static fromJSON(json) {
        return new ThematicDisplay(json);
    }
    toJSON() {
        const json = {
            displayMode: this.displayMode,
            gradientSettings: this.gradientSettings.toJSON(),
            axis: this.axis.toJSON(),
            sunDirection: this.sunDirection.toJSON(),
            range: this.range.toJSON(),
        };
        if (this.sensorSettings.sensors.length > 0)
            json.sensorSettings = this.sensorSettings.toJSON();
        return json;
    }
}
exports.ThematicDisplay = ThematicDisplay;
//# sourceMappingURL=ThematicDisplay.js.map