"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module DisplayStyles
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViewFlagOverrides = exports.ViewFlagPresence = exports.ViewFlags = exports.RenderMode = void 0;
// cspell:ignore ovrs
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
/** Enumerates the available rendering modes. The rendering mode chiefly controls whether and how surfaces and their edges are drawn.
 * Generally speaking,
 *  - Wireframe draws only edges.
 *  - SmoothShade draws only surfaces.
 *  - HiddenLine and SolidFill draw both surfaces and edges.
 *  - Lighting is only applied in SmoothShade mode.
 *
 * The [[FillFlags]] associated with planar regions controls whether and how the region's interior area is displayed in Wireframe mode.
 * [[ViewFlags]] has options for enabling display of visible and/or hidden edges in SmoothShade mode.
 * [[HiddenLine.Settings]] allow aspects of edge and surface symbology to be overridden within a view.
 * @public
 */
var RenderMode;
(function (RenderMode) {
    /** Render only edges, no surfaces, with exceptions for planar regions with [[FillFlags]] set up to render the surface in wireframe mode. */
    RenderMode[RenderMode["Wireframe"] = 0] = "Wireframe";
    /** Render only surfaces, no edges, with lighting. */
    RenderMode[RenderMode["SmoothShade"] = 6] = "SmoothShade";
    /** Render edges and surfaces. Surfaces are drawn using the view's background color instead of the element's fill color. */
    RenderMode[RenderMode["HiddenLine"] = 3] = "HiddenLine";
    /** Render edges and surfaces. */
    RenderMode[RenderMode["SolidFill"] = 4] = "SolidFill";
})(RenderMode = exports.RenderMode || (exports.RenderMode = {}));
function edgesRequired(renderMode, visibleEdges) {
    return visibleEdges || RenderMode.SmoothShade !== renderMode;
}
/** Flags for controlling how graphics appear within a View.
 * @public
 */
class ViewFlags {
    constructor() {
        /** The [[RenderMode]] of the view. */
        this.renderMode = RenderMode.Wireframe;
        /** Shows or hides dimensions. */
        this.dimensions = true;
        /** Shows or hides pattern geometry. */
        this.patterns = true;
        /** Controls whether non-zero line weights are used or display using weight 0. */
        this.weights = true;
        /** Controls whether custom line styles are used (e.g. control whether elements with custom line styles draw normally, or as solid lines). */
        this.styles = true;
        /** Controls whether element transparency is used (e.g. control whether elements with transparency draw normally, or as opaque). */
        this.transparency = true;
        /** Controls whether the fills on filled elements are displayed. */
        this.fill = true;
        /** Controls whether to display texture maps for material assignments. When off only material color is used for display. */
        this.textures = true;
        /** Controls whether materials are used (e.g. control whether geometry with materials draw normally, or as if it has no material). */
        this.materials = true;
        /** Shows or hides the ACS triad. */
        this.acsTriad = false;
        /** Shows or hides the grid. The grid settings are a design file setting. */
        this.grid = false;
        /** Shows or hides visible edges in the shaded render mode. */
        this.visibleEdges = false;
        /** Shows or hides hidden edges in the shaded render mode. */
        this.hiddenEdges = false;
        /** Controls whether the source lights in spatial models are used
         * @note Currently the renderer only supports solar lighting. For backwards-compatibility reasons, solar lights will be displayed if any combination of [[noCameraLights]], [[noSourceLights]], or [[noSolarLight]] is set to `false`.
         * @see [[lighting]] for a more convenient way to toggle lighting on and off.
         */
        this.sourceLights = false;
        /** Controls whether camera (ambient, portrait, flashbulb) lights are used.
         * @note Currently the renderer only supports solar lighting. For backwards-compatibility reasons, solar lights will be displayed if any combination of [[noCameraLights]], [[noSourceLights]], or [[noSolarLight]] is set to `false`.
         * @see [[lighting]] for a more convenient way to toggle lighting on and off.
         */
        this.cameraLights = false;
        /** Controls whether sunlight used
         * @note Currently the renderer only supports solar lighting. For backwards-compatibility reasons, solar lights will be displayed if any combination of [[noCameraLights]], [[noSourceLights]], or [[noSolarLight]] is set to `false`.
         * @see [[lighting]] for a more convenient way to toggle lighting on and off.
         */
        this.solarLight = false;
        /** Shows or hides shadows. */
        this.shadows = false;
        /** Controls whether the view's clip volume is applied. Has no effect on other types of clips like [[ModelClipGroups]]. */
        this.clipVolume = true;
        /** Shows or hides construction class geometry. */
        this.constructions = false;
        /** Draw geometry using the view's monochrome color.
         * @see [DisplayStyleSettings.monochromeColor]($common) for details on how the color is applied.
         * @see [DisplayStyleSettings.monochromeMode]($common) to control the type of monochrome display applied.
         */
        this.monochrome = false;
        /** @internal unused Ignore geometry maps */
        this.noGeometryMap = false;
        /** Display background map */
        this.backgroundMap = false;
        /** Use material colors for hidden lines */
        this.hLineMaterialColors = false;
        /** @internal 0=none, 1=generate mask, 2=use mask */
        this.edgeMask = 0;
        /** Controls whether ambient occlusion is used. */
        this.ambientOcclusion = false;
        /** Controls whether thematic display is used.
         * @note Currently, thematically displayed geometry will not receive shadows. If thematic display is enabled, shadows will not be received by thematically displayed geometry, even if shadows are enabled.
         */
        this.thematicDisplay = false;
        /** Controls whether surface discard is always applied regardless of other ViewFlags.
         * Surface shaders contain complicated logic to ensure that the edges of a surface always draw in front of the surface, and that planar surfaces sketched coincident with
         * non-planar surfaces always draw in front of those non-planar surfaces.
         * When this view flag is set to false (the default), then for 3d views if the render mode is wireframe (only edges are displayed) or smooth shader with visible edges turned off (only surfaces are displayed),
         * that logic does not execute, potentially improving performance for no degradation in visual quality. In some scenarios - such as wireframe views containing many planar regions with interior fill, or smooth views containing many coincident planar and non-planar surfaces - enabling this view flag improves display quality by forcing that logic to execute.
         */
        this.forceSurfaceDiscard = false;
        /** Whether to apply white-on-white reversal.
         * Some applications use this to cause white geometry to be drawn as black if the view's background color is white.
         * When enabled, the [[DisplayStyleSettings]]' [[WhiteOnWhiteReversalSettings]] control how white-on-white reversal is applied.
         * Default: true.
         */
        this.whiteOnWhiteReversal = true;
    }
    /** Controls whether or not lighting is applied.
     * @note Has no effect unless `renderMode` is set to [[RenderMode.SmoothShade]].
     */
    get lighting() { return this.solarLight || this.sourceLights || this.cameraLights; }
    set lighting(enable) { this.solarLight = this.sourceLights = this.cameraLights = enable; }
    clone(out) { return ViewFlags.createFrom(this, out); }
    static createFrom(other, out) {
        const val = undefined !== out ? out : new ViewFlags();
        if (other) {
            val.renderMode = other.renderMode;
            val.dimensions = other.dimensions;
            val.patterns = other.patterns;
            val.weights = other.weights;
            val.styles = other.styles;
            val.transparency = other.transparency;
            val.fill = other.fill;
            val.textures = other.textures;
            val.materials = other.materials;
            val.acsTriad = other.acsTriad;
            val.grid = other.grid;
            val.visibleEdges = other.visibleEdges;
            val.hiddenEdges = other.hiddenEdges;
            val.sourceLights = other.sourceLights;
            val.cameraLights = other.cameraLights;
            val.solarLight = other.solarLight;
            val.shadows = other.shadows;
            val.clipVolume = other.clipVolume;
            val.constructions = other.constructions;
            val.monochrome = other.monochrome;
            val.noGeometryMap = other.noGeometryMap;
            val.hLineMaterialColors = other.hLineMaterialColors;
            val.backgroundMap = other.backgroundMap;
            val.edgeMask = other.edgeMask;
            val.ambientOcclusion = other.ambientOcclusion;
            val.thematicDisplay = other.thematicDisplay;
            val.forceSurfaceDiscard = other.forceSurfaceDiscard;
            val.whiteOnWhiteReversal = other.whiteOnWhiteReversal;
        }
        return val;
    }
    /** @internal */
    hiddenEdgesVisible() {
        switch (this.renderMode) {
            case RenderMode.SolidFill:
            case RenderMode.HiddenLine:
                return this.hiddenEdges;
            case RenderMode.SmoothShade:
                return this.visibleEdges && this.hiddenEdges;
        }
        return true;
    }
    /** Returns true if the edges of surfaces should be displayed, based on [[RenderMode]] and the [[visibleEdges]] flag. */
    edgesRequired() {
        return edgesRequired(this.renderMode, this.visibleEdges);
    }
    toJSON() {
        const out = {};
        if (!this.constructions)
            out.noConstruct = true;
        if (!this.dimensions)
            out.noDim = true;
        if (!this.patterns)
            out.noPattern = true;
        if (!this.weights)
            out.noWeight = true;
        if (!this.styles)
            out.noStyle = true;
        if (!this.transparency)
            out.noTransp = true;
        if (!this.fill)
            out.noFill = true;
        if (this.grid)
            out.grid = true;
        if (this.acsTriad)
            out.acs = true;
        if (!this.textures)
            out.noTexture = true;
        if (!this.materials)
            out.noMaterial = true;
        if (!this.cameraLights)
            out.noCameraLights = true;
        if (!this.sourceLights)
            out.noSourceLights = true;
        if (!this.solarLight)
            out.noSolarLight = true;
        if (this.visibleEdges)
            out.visEdges = true;
        if (this.hiddenEdges)
            out.hidEdges = true;
        if (this.shadows)
            out.shadows = true;
        if (this.clipVolume)
            out.clipVol = true;
        if (this.hLineMaterialColors)
            out.hlMatColors = true;
        if (this.monochrome)
            out.monochrome = true;
        if (this.backgroundMap)
            out.backgroundMap = true;
        if (this.edgeMask !== 0)
            out.edgeMask = this.edgeMask;
        if (this.ambientOcclusion)
            out.ambientOcclusion = true;
        if (this.thematicDisplay)
            out.thematicDisplay = true;
        if (this.forceSurfaceDiscard)
            out.forceSurfaceDiscard = true;
        if (!this.whiteOnWhiteReversal)
            out.noWhiteOnWhiteReversal = true;
        out.renderMode = this.renderMode;
        return out;
    }
    /** Like [[toJSON]], but no properties are omitted.
     * @internal
     */
    toFullyDefinedJSON() {
        return {
            renderMode: this.renderMode,
            noConstruct: !this.constructions,
            noDim: !this.dimensions,
            noPattern: !this.patterns,
            noWeight: !this.weights,
            noStyle: !this.styles,
            noTransp: !this.transparency,
            noFill: !this.fill,
            grid: this.grid,
            acs: this.acsTriad,
            noTexture: !this.textures,
            noMaterial: !this.materials,
            noCameraLights: !this.cameraLights,
            noSourceLights: !this.sourceLights,
            noSolarLight: !this.solarLight,
            visEdges: this.visibleEdges,
            hidEdges: this.hiddenEdges,
            shadows: this.shadows,
            clipVol: this.clipVolume,
            hlMatColors: this.hLineMaterialColors,
            monochrome: this.monochrome,
            backgroundMap: this.backgroundMap,
            edgeMask: this.edgeMask,
            ambientOcclusion: this.ambientOcclusion,
            thematicDisplay: this.thematicDisplay,
            forceSurfaceDiscard: this.forceSurfaceDiscard,
            noWhiteOnWhiteReversal: !this.whiteOnWhiteReversal,
        };
    }
    static fromJSON(json) {
        const val = new ViewFlags();
        if (!json)
            return val;
        val.constructions = !bentleyjs_core_1.JsonUtils.asBool(json.noConstruct);
        val.dimensions = !bentleyjs_core_1.JsonUtils.asBool(json.noDim);
        val.patterns = !bentleyjs_core_1.JsonUtils.asBool(json.noPattern);
        val.weights = !bentleyjs_core_1.JsonUtils.asBool(json.noWeight);
        val.styles = !bentleyjs_core_1.JsonUtils.asBool(json.noStyle);
        val.transparency = !bentleyjs_core_1.JsonUtils.asBool(json.noTransp);
        val.fill = !bentleyjs_core_1.JsonUtils.asBool(json.noFill);
        val.grid = bentleyjs_core_1.JsonUtils.asBool(json.grid);
        val.acsTriad = bentleyjs_core_1.JsonUtils.asBool(json.acs);
        val.textures = !bentleyjs_core_1.JsonUtils.asBool(json.noTexture);
        val.materials = !bentleyjs_core_1.JsonUtils.asBool(json.noMaterial);
        val.cameraLights = !bentleyjs_core_1.JsonUtils.asBool(json.noCameraLights);
        val.sourceLights = !bentleyjs_core_1.JsonUtils.asBool(json.noSourceLights);
        val.solarLight = !bentleyjs_core_1.JsonUtils.asBool(json.noSolarLight);
        val.visibleEdges = bentleyjs_core_1.JsonUtils.asBool(json.visEdges);
        val.hiddenEdges = bentleyjs_core_1.JsonUtils.asBool(json.hidEdges);
        val.shadows = bentleyjs_core_1.JsonUtils.asBool(json.shadows);
        val.clipVolume = bentleyjs_core_1.JsonUtils.asBool(json.clipVol);
        val.monochrome = bentleyjs_core_1.JsonUtils.asBool(json.monochrome);
        val.edgeMask = bentleyjs_core_1.JsonUtils.asInt(json.edgeMask);
        val.hLineMaterialColors = bentleyjs_core_1.JsonUtils.asBool(json.hlMatColors);
        val.backgroundMap = bentleyjs_core_1.JsonUtils.asBool(json.backgroundMap);
        val.ambientOcclusion = bentleyjs_core_1.JsonUtils.asBool(json.ambientOcclusion);
        val.thematicDisplay = bentleyjs_core_1.JsonUtils.asBool(json.thematicDisplay);
        val.forceSurfaceDiscard = bentleyjs_core_1.JsonUtils.asBool(json.forceSurfaceDiscard);
        val.whiteOnWhiteReversal = !bentleyjs_core_1.JsonUtils.asBool(json.noWhiteOnWhiteReversal);
        const renderModeValue = bentleyjs_core_1.JsonUtils.asInt(json.renderMode);
        if (renderModeValue < RenderMode.HiddenLine)
            val.renderMode = RenderMode.Wireframe;
        else if (renderModeValue > RenderMode.SolidFill)
            val.renderMode = RenderMode.SmoothShade;
        else
            val.renderMode = renderModeValue;
        return val;
    }
    equals(other) {
        return this.renderMode === other.renderMode
            && this.dimensions === other.dimensions
            && this.patterns === other.patterns
            && this.weights === other.weights
            && this.styles === other.styles
            && this.transparency === other.transparency
            && this.fill === other.fill
            && this.textures === other.textures
            && this.materials === other.materials
            && this.acsTriad === other.acsTriad
            && this.grid === other.grid
            && this.visibleEdges === other.visibleEdges
            && this.hiddenEdges === other.hiddenEdges
            && this.sourceLights === other.sourceLights
            && this.cameraLights === other.cameraLights
            && this.solarLight === other.solarLight
            && this.shadows === other.shadows
            && this.clipVolume === other.clipVolume
            && this.constructions === other.constructions
            && this.monochrome === other.monochrome
            && this.noGeometryMap === other.noGeometryMap
            && this.hLineMaterialColors === other.hLineMaterialColors
            && this.backgroundMap === other.backgroundMap
            && this.edgeMask === other.edgeMask
            && this.ambientOcclusion === other.ambientOcclusion
            && this.thematicDisplay === other.thematicDisplay
            && this.forceSurfaceDiscard === other.forceSurfaceDiscard
            && this.whiteOnWhiteReversal === other.whiteOnWhiteReversal;
    }
}
exports.ViewFlags = ViewFlags;
/** Values used by [[ViewFlagOverrides]] to indicate which aspects of the [[ViewFlags]] are overridden.
 * @public
 */
var ViewFlagPresence;
(function (ViewFlagPresence) {
    ViewFlagPresence[ViewFlagPresence["RenderMode"] = 0] = "RenderMode";
    ViewFlagPresence[ViewFlagPresence["Dimensions"] = 1] = "Dimensions";
    ViewFlagPresence[ViewFlagPresence["Patterns"] = 2] = "Patterns";
    ViewFlagPresence[ViewFlagPresence["Weights"] = 3] = "Weights";
    ViewFlagPresence[ViewFlagPresence["Styles"] = 4] = "Styles";
    ViewFlagPresence[ViewFlagPresence["Transparency"] = 5] = "Transparency";
    ViewFlagPresence[ViewFlagPresence["Unused"] = 6] = "Unused";
    ViewFlagPresence[ViewFlagPresence["Fill"] = 7] = "Fill";
    ViewFlagPresence[ViewFlagPresence["Textures"] = 8] = "Textures";
    ViewFlagPresence[ViewFlagPresence["Materials"] = 9] = "Materials";
    ViewFlagPresence[ViewFlagPresence["VisibleEdges"] = 10] = "VisibleEdges";
    ViewFlagPresence[ViewFlagPresence["HiddenEdges"] = 11] = "HiddenEdges";
    ViewFlagPresence[ViewFlagPresence["Lighting"] = 12] = "Lighting";
    ViewFlagPresence[ViewFlagPresence["Shadows"] = 13] = "Shadows";
    ViewFlagPresence[ViewFlagPresence["ClipVolume"] = 14] = "ClipVolume";
    ViewFlagPresence[ViewFlagPresence["Constructions"] = 15] = "Constructions";
    ViewFlagPresence[ViewFlagPresence["Monochrome"] = 16] = "Monochrome";
    ViewFlagPresence[ViewFlagPresence["GeometryMap"] = 17] = "GeometryMap";
    ViewFlagPresence[ViewFlagPresence["HlineMaterialColors"] = 18] = "HlineMaterialColors";
    ViewFlagPresence[ViewFlagPresence["EdgeMask"] = 19] = "EdgeMask";
    ViewFlagPresence[ViewFlagPresence["BackgroundMap"] = 20] = "BackgroundMap";
    ViewFlagPresence[ViewFlagPresence["ForceSurfaceDiscard"] = 21] = "ForceSurfaceDiscard";
    ViewFlagPresence[ViewFlagPresence["WhiteOnWhiteReversal"] = 22] = "WhiteOnWhiteReversal";
    ViewFlagPresence[ViewFlagPresence["ThematicDisplay"] = 23] = "ThematicDisplay";
})(ViewFlagPresence = exports.ViewFlagPresence || (exports.ViewFlagPresence = {}));
/** Overrides a subset of [[ViewFlags]].
 * @public
 */
class ViewFlagOverrides {
    /** Construct a ViewFlagOverrides which overrides all flags to match the specified ViewFlags, or overrides nothing if no ViewFlags are supplied. */
    constructor(flags) {
        this._present = 0;
        this._values = new ViewFlags();
        if (undefined !== flags)
            this.overrideAll(flags);
    }
    /** Returns true if the specified flag is overridden. */
    isPresent(flag) { return 0 !== (this._present & (1 << flag)); }
    /** Mark the specified flag as overridden. */
    setPresent(flag) { this._present |= (1 << flag); }
    /** Mark the specified flag as not overridden. */
    clearPresent(flag) {
        // Bit-wise NOT produces signed one's complement in javascript...triple-shift right by zero to get correct result...
        this._present &= (~(1 << flag)) >>> 0;
    }
    /** Marks all view flags as overridden.
     * @param flags If supplied, these overrides will match the input view flags; otherwise, they will match the default view flags.
     */
    overrideAll(flags) {
        ViewFlags.createFrom(flags, this._values);
        this._present = 0xffffffff;
    }
    /** Create a copy of these overrides.
     * @param out If supplied, the input overrides will be modified to match these overrides; otherwise, a new ViewFlagOverrides object will be created as the clone.
     * @returns A copy of these overrides.
     */
    clone(out) {
        const result = undefined !== out ? out : new ViewFlagOverrides();
        result.copyFrom(this);
        return result;
    }
    /** Modify these overrides to match the input overrides. */
    copyFrom(other) {
        other._values.clone(this._values);
        this._present = other._present;
    }
    setShowDimensions(val) { this._values.dimensions = val; this.setPresent(ViewFlagPresence.Dimensions); }
    setShowPatterns(val) { this._values.patterns = val; this.setPresent(ViewFlagPresence.Patterns); }
    setShowWeights(val) { this._values.weights = val; this.setPresent(ViewFlagPresence.Weights); }
    setShowStyles(val) { this._values.styles = val; this.setPresent(ViewFlagPresence.Styles); }
    setShowTransparency(val) { this._values.transparency = val; this.setPresent(ViewFlagPresence.Transparency); }
    setShowFill(val) { this._values.fill = val; this.setPresent(ViewFlagPresence.Fill); }
    setShowTextures(val) { this._values.textures = val; this.setPresent(ViewFlagPresence.Textures); }
    setShowMaterials(val) { this._values.materials = val; this.setPresent(ViewFlagPresence.Materials); }
    setApplyLighting(val) { this._values.lighting = val; this.setPresent(ViewFlagPresence.Lighting); }
    setShowVisibleEdges(val) { this._values.visibleEdges = val; this.setPresent(ViewFlagPresence.VisibleEdges); }
    setShowHiddenEdges(val) { this._values.hiddenEdges = val; this.setPresent(ViewFlagPresence.HiddenEdges); }
    setShowShadows(val) { this._values.shadows = val; this.setPresent(ViewFlagPresence.Shadows); }
    setShowClipVolume(val) { this._values.clipVolume = val; this.setPresent(ViewFlagPresence.ClipVolume); }
    setShowConstructions(val) { this._values.constructions = val; this.setPresent(ViewFlagPresence.Constructions); }
    setMonochrome(val) { this._values.monochrome = val; this.setPresent(ViewFlagPresence.Monochrome); }
    setIgnoreGeometryMap(val) { this._values.noGeometryMap = val; this.setPresent(ViewFlagPresence.GeometryMap); }
    setShowBackgroundMap(val) { this._values.backgroundMap = val; this.setPresent(ViewFlagPresence.BackgroundMap); }
    setUseHlineMaterialColors(val) { this._values.hLineMaterialColors = val; this.setPresent(ViewFlagPresence.HlineMaterialColors); }
    setForceSurfaceDiscard(val) { this._values.forceSurfaceDiscard = val; this.setPresent(ViewFlagPresence.ForceSurfaceDiscard); }
    setWhiteOnWhiteReversal(val) { this._values.whiteOnWhiteReversal = val; this.setPresent(ViewFlagPresence.WhiteOnWhiteReversal); }
    setEdgeMask(val) { this._values.edgeMask = val; this.setPresent(ViewFlagPresence.EdgeMask); }
    setRenderMode(val) { this._values.renderMode = val; this.setPresent(ViewFlagPresence.RenderMode); }
    setThematicDisplay(val) { this._values.thematicDisplay = val; this.setPresent(ViewFlagPresence.ThematicDisplay); }
    /** Return whether these overrides applied to the specified ViewFlags require edges to be drawn. */
    edgesRequired(viewFlags) {
        const renderMode = this.isPresent(ViewFlagPresence.RenderMode) ? this._values.renderMode : viewFlags.renderMode;
        const visibleEdges = this.isPresent(ViewFlagPresence.VisibleEdges) ? this._values.visibleEdges : viewFlags.visibleEdges;
        return edgesRequired(renderMode, visibleEdges);
    }
    /** Returns true if any view flags are overridden. */
    anyOverridden() { return 0 !== this._present; }
    /** Marks all view flags as not overridden. */
    clear() { this._present = 0; }
    clearClipVolume() { this.clearPresent(ViewFlagPresence.ClipVolume); }
    /** If ViewFlags.clipVolume is overridden, return the override value; else return undefined.
     * @internal
     */
    get clipVolumeOverride() {
        return this.isPresent(ViewFlagPresence.ClipVolume) ? this._values.clipVolume : undefined;
    }
    /** Apply these overrides to the supplied ViewFlags. The values of any flags that are overridden will be replaced by the override values; the rest of the flags are untouched. */
    apply(base) {
        this.applyFlags(base);
        return base;
    }
    toJSON() {
        const props = {};
        this.applyFlags(props);
        return props;
    }
    static fromJSON(props) {
        const ovrs = new ViewFlagOverrides();
        if (!props)
            return ovrs;
        const setBoolean = (key, set) => {
            const val = props[key];
            if (typeof val === "boolean")
                set(val);
        };
        setBoolean("dimensions", (val) => ovrs.setShowDimensions(val));
        setBoolean("patterns", (val) => ovrs.setShowPatterns(val));
        setBoolean("weights", (val) => ovrs.setShowWeights(val));
        setBoolean("styles", (val) => ovrs.setShowStyles(val));
        setBoolean("transparency", (val) => ovrs.setShowTransparency(val));
        setBoolean("fill", (val) => ovrs.setShowFill(val));
        setBoolean("textures", (val) => ovrs.setShowTextures(val));
        setBoolean("materials", (val) => ovrs.setShowMaterials(val));
        setBoolean("lighting", (val) => ovrs.setApplyLighting(val));
        setBoolean("visibleEdges", (val) => ovrs.setShowVisibleEdges(val));
        setBoolean("hiddenEdges", (val) => ovrs.setShowHiddenEdges(val));
        setBoolean("shadows", (val) => ovrs.setShowShadows(val));
        setBoolean("clipVolume", (val) => ovrs.setShowClipVolume(val));
        setBoolean("constructions", (val) => ovrs.setShowConstructions(val));
        setBoolean("monochrome", (val) => ovrs.setMonochrome(val));
        setBoolean("noGeometryMap", (val) => ovrs.setIgnoreGeometryMap(val));
        setBoolean("backgroundMap", (val) => ovrs.setShowBackgroundMap(val));
        setBoolean("hLineMaterialColors", (val) => ovrs.setUseHlineMaterialColors(val));
        setBoolean("forceSurfaceDiscard", (val) => ovrs.setForceSurfaceDiscard(val));
        setBoolean("whiteOnWhiteReversal", (val) => ovrs.setWhiteOnWhiteReversal(val));
        setBoolean("thematicDisplay", (val) => ovrs.setThematicDisplay(val));
        if (typeof props.edgeMask === "number")
            ovrs.setEdgeMask(props.edgeMask);
        if (typeof props.renderMode === "number")
            ovrs.setRenderMode(props.renderMode);
        return ovrs;
    }
    applyFlags(flags) {
        if (!this.anyOverridden())
            return;
        if (this.isPresent(ViewFlagPresence.Dimensions))
            flags.dimensions = this._values.dimensions;
        if (this.isPresent(ViewFlagPresence.Patterns))
            flags.patterns = this._values.patterns;
        if (this.isPresent(ViewFlagPresence.Weights))
            flags.weights = this._values.weights;
        if (this.isPresent(ViewFlagPresence.Styles))
            flags.styles = this._values.styles;
        if (this.isPresent(ViewFlagPresence.Transparency))
            flags.transparency = this._values.transparency;
        if (this.isPresent(ViewFlagPresence.Fill))
            flags.fill = this._values.fill;
        if (this.isPresent(ViewFlagPresence.Textures))
            flags.textures = this._values.textures;
        if (this.isPresent(ViewFlagPresence.Materials))
            flags.materials = this._values.materials;
        if (this.isPresent(ViewFlagPresence.Lighting))
            flags.lighting = this._values.lighting;
        if (this.isPresent(ViewFlagPresence.VisibleEdges))
            flags.visibleEdges = this._values.visibleEdges;
        if (this.isPresent(ViewFlagPresence.HiddenEdges))
            flags.hiddenEdges = this._values.hiddenEdges;
        if (this.isPresent(ViewFlagPresence.Shadows))
            flags.shadows = this._values.shadows;
        if (this.isPresent(ViewFlagPresence.ClipVolume))
            flags.clipVolume = this._values.clipVolume;
        if (this.isPresent(ViewFlagPresence.Constructions))
            flags.constructions = this._values.constructions;
        if (this.isPresent(ViewFlagPresence.Monochrome))
            flags.monochrome = this._values.monochrome;
        if (this.isPresent(ViewFlagPresence.GeometryMap))
            flags.noGeometryMap = this._values.noGeometryMap;
        if (this.isPresent(ViewFlagPresence.BackgroundMap))
            flags.backgroundMap = this._values.backgroundMap;
        if (this.isPresent(ViewFlagPresence.HlineMaterialColors))
            flags.hLineMaterialColors = this._values.hLineMaterialColors;
        if (this.isPresent(ViewFlagPresence.ForceSurfaceDiscard))
            flags.forceSurfaceDiscard = this._values.forceSurfaceDiscard;
        if (this.isPresent(ViewFlagPresence.WhiteOnWhiteReversal))
            flags.whiteOnWhiteReversal = this._values.whiteOnWhiteReversal;
        if (this.isPresent(ViewFlagPresence.EdgeMask))
            flags.edgeMask = this._values.edgeMask;
        if (this.isPresent(ViewFlagPresence.RenderMode))
            flags.renderMode = this._values.renderMode;
        if (this.isPresent(ViewFlagPresence.ThematicDisplay))
            flags.thematicDisplay = this._values.thematicDisplay;
    }
}
exports.ViewFlagOverrides = ViewFlagOverrides;
//# sourceMappingURL=ViewFlags.js.map