/** @packageDocumentation
 * @module Views
 */
import { Id64Array, Id64String } from "@bentley/bentleyjs-core";
import { AngleProps, Range3dProps, TransformProps, XYProps, XYZProps, YawPitchRollProps } from "@bentley/geometry-core";
import { CameraProps } from "./Camera";
import { DisplayStyleProps } from "./DisplayStyleSettings";
import { DefinitionElementProps, DisplayStyleLoadProps, ElementProps, SheetProps } from "./ElementProps";
import { EntityQueryParams } from "./EntityProps";
import { ViewDetails3dProps, ViewDetailsProps } from "./ViewDetails";
/** As part of a [[ViewStateProps]], describes the [[SpatialViewDefinition]] from which a [SectionDrawing]($backend) was generated.
 * @see [[SectionDrawingProps]]
 * @public
 */
export interface SectionDrawingViewProps {
    /** The Id of the spatial view from which the SectionDrawing was generated. */
    spatialView: Id64String;
    /** If true, the spatial view should be displayed in the context of the drawing view. */
    displaySpatialView: boolean;
    /** Transform from drawing coordinates to spatial coordinates. If undefined, use identity transform. */
    drawingToSpatialTransform?: TransformProps;
}
/** Returned from [IModelDb.Views.getViewStateData]($backend).
 * @public
 */
export interface ViewStateProps {
    viewDefinitionProps: ViewDefinitionProps;
    categorySelectorProps: CategorySelectorProps;
    modelSelectorProps?: ModelSelectorProps;
    displayStyleProps: DisplayStyleProps;
    /** Sheet-specific properties, if this is a view of a [SheetModel]($backend). */
    sheetProps?: SheetProps;
    /** The Ids of the [ViewAttachment]($backend)s contained within the [SheetModel]($backend), if this is a sheet view. */
    sheetAttachments?: Id64Array;
    /** For a [DrawingViewState]($frontend), the extents of the [DrawingModel]($backend), used for determining the upper limits of the view's extents. */
    modelExtents?: Range3dProps;
    /** Information about the [SectionDrawing]($backend) relevant to displaying a drawing view. */
    sectionDrawing?: SectionDrawingViewProps;
}
/** Options for loading a [[ViewStateProps]] via [IModelConnection.Views.load]($frontend) or [IModelDb.Views.getViewStateData]($backend).
 * @public
 */
export interface ViewStateLoadProps {
    /** Options for loading the view's [[DisplayStyleProps]]. */
    displayStyle?: DisplayStyleLoadProps;
}
/** Properties that define a ModelSelector
 * @public
 */
export interface ModelSelectorProps extends DefinitionElementProps {
    models: Id64Array;
}
/** Properties that define a CategorySelector
 * @public
 */
export interface CategorySelectorProps extends DefinitionElementProps {
    categories: Id64Array;
}
/** Parameters for performing a query on [ViewDefinition]($backend) classes.
 * @public
 */
export interface ViewQueryParams extends EntityQueryParams {
    wantPrivate?: boolean;
}
/** Parameters used to construct a ViewDefinition
 * @public
 */
export interface ViewDefinitionProps extends DefinitionElementProps {
    categorySelectorId: Id64String;
    displayStyleId: Id64String;
    description?: string;
    jsonProperties?: {
        /** Additional properties of the view. */
        viewDetails?: ViewDetailsProps;
    };
}
/** Parameters to construct a ViewDefinition3d
 * @public
 */
export interface ViewDefinition3dProps extends ViewDefinitionProps {
    /** if true, camera is valid. */
    cameraOn: boolean;
    /** The lower left back corner of the view frustum. */
    origin: XYZProps;
    /** The extent of the view frustum. */
    extents: XYZProps;
    /** Rotation of the view frustum (could be undefined if going Matrix3d -> YawPitchRoll). */
    angles?: YawPitchRollProps;
    /** The camera used for this view. */
    camera: CameraProps;
    jsonProperties?: {
        /** Additional properties of the view. */
        viewDetails?: ViewDetails3dProps;
    };
}
/** Parameters to construct a SpatialViewDefinition
 * @public
 */
export interface SpatialViewDefinitionProps extends ViewDefinition3dProps {
    modelSelectorId: Id64String;
}
/** Parameters used to construct a ViewDefinition2d
 * @public
 */
export interface ViewDefinition2dProps extends ViewDefinitionProps {
    baseModelId: Id64String;
    origin: XYProps;
    delta: XYProps;
    angle: AngleProps;
}
/** @public */
export interface AuxCoordSystemProps extends ElementProps {
    type?: number;
    description?: string;
}
/**  Properties of AuxCoordSystem2d
 * @public
 */
export interface AuxCoordSystem2dProps extends AuxCoordSystemProps {
    /** Origin of the AuxCoordSystem2d */
    origin?: XYProps;
    /** Rotation angle */
    angle?: AngleProps;
}
/** Properties of AuxCoordSystem3d
 * @public
 */
export interface AuxCoordSystem3dProps extends AuxCoordSystemProps {
    /** Origin of the AuxCoordSystem3d */
    origin?: XYZProps;
    /** Yaw angle */
    yaw?: AngleProps;
    /** Pitch angle */
    pitch?: AngleProps;
    /** Roll angle */
    roll?: AngleProps;
}
//# sourceMappingURL=ViewProps.d.ts.map