/** @packageDocumentation
 * @module Geometry
 */
import { GeodeticDatum, GeodeticDatumProps } from "./GeodeticDatum";
import { GeodeticEllipsoid, GeodeticEllipsoidProps } from "./GeodeticEllipsoid";
import { Carto2DDegrees, Carto2DDegreesProps, Projection, ProjectionProps } from "./Projection";
import { AdditionalTransform, AdditionalTransformProps } from "./AdditionalTransform";
/** This type indicates possible linear and angular units supported.
 *  @public
*/
export declare type UnitType = "Meter" | "InternationalFoot" | "USSurveyFoot" | "Degree" | "Unsupported";
/** The extent in latitude, longitude bounds where a horizontal CRS is applicable
 *  @public
*/
export interface HorizontalCRSExtentProps {
    /** The South West point in latitude and longitude in degrees for the user-defined extent of the CRS */
    southWest: Carto2DDegreesProps;
    /** The North East point in latitude in longitude in degrees for the user-defined extent of the CRS.
       The latitude of the North East must be greater or equal to the latitude of the South West point
       It is possible, however, for the longitude of the South West corner to have greater value than the
       longitude of the North East point such as when the west longitude is located on the other side
       of the -180/180 degree longitude line.*/
    northEast: Carto2DDegreesProps;
}
/** The extent in latitude, longitude bounds where a horizontal CRS is applicable
 *  @public
 */
export declare class HorizontalCRSExtent implements HorizontalCRSExtentProps {
    /** The latitude minimum and maximum for the user-defined extent of the CRS */
    readonly southWest: Carto2DDegrees;
    /** The North East point in latitude in longitude in degrees for the user-defined extent of the CRS.
       The latitude of the North East must be greater or equal to the latitude of the South West point
       It is possible, however, for the longitude of the South West corner to have greater value than the
       longitude of the North East point such as when the west longitude is located on the other side
       of the -180/180 degree longitude line.*/
    readonly northEast: Carto2DDegrees;
    constructor(data?: HorizontalCRSExtentProps);
    /** Creates an extent object from JSON representation.
     * @public */
    static fromJSON(data: HorizontalCRSExtentProps): HorizontalCRSExtent;
    /** Creates a JSON from the Extent definition
     * @public */
    toJSON(): HorizontalCRSExtentProps;
    /** Compares two Extents. It applies a minuscule tolerance to comparing numbers.
     *  @public */
    equals(other: HorizontalCRSExtent): boolean;
}
/** Horizontal Geographic Coordinate Reference System definition
 *  @public
 */
export interface HorizontalCRSProps {
    /** The identifier of the horizontal CRS as stored in the dictionary or the service database */
    id?: string;
    /** Used only for user-defined definitions that will typically use a GUID as id. A display name for the CRS that allows
     *  a human to understand the nature of the definition
     */
    name?: string;
    /** Description */
    description?: string;
    /** A textual description of the source of the CRS definition. */
    source?: string;
    /** If true then indicates the definition is deprecated. It should then be used for backward compatibility only.
     *  If false or undefined then the definition is not deprecated.
     */
    deprecated?: boolean;
    /** The EPSG code of the CRS. If undefined then there is no EPSG code associated. */
    epsg?: number;
    /** The identifier of the geodetic datum as stored in the dictionary or the service database. */
    datumId?: string;
    /** The complete definition of the geodetic datum referred to by datumId. It can also be used if the datum is not stored
     *  in either service or dictionary.
     */
    datum?: GeodeticDatumProps;
    /** The identifier of the geodetic ellipsoid as stored in the dictionary or the service database. This property is exclusive
     * of having datumId and datum properties undefined.
    */
    ellipsoidId?: string;
    /** The complete definition of the geodetic datum referred to by datumId. It can also be used if the datum is not stored
     *  in either service or dictionary
     */
    ellipsoid?: GeodeticEllipsoidProps;
    /** The text indicating the unit used. */
    unit?: UnitType;
    /** Projection including projection parameters. */
    projection?: ProjectionProps;
    /** Extent representing the domain of application of the CRS. */
    extent?: HorizontalCRSExtentProps;
}
/** Horizontal Geographic Coordinate reference System implementation.
 *  An horizontal CRS defines the portion which is horizontal to the Earth surface (within the deformation brought by the projection process).
 *  There are two major classes of Horizontal Coordinate Reference Systems:
 *  - The projected CRS which rely on a projection to flatten the coordinate system space into axises using linear
 *    units (meter, US Survey Feet, ...) relative to some origin.
 *  - The non projected CRS (also named geographic CRS by EPSG nomenclature) that does
 *    not require a projection (projection method = None) and horizontal coordinates are expressed
 *    as longitude and latitude (see [[Cartographic]])
 *  Horizontal Coordinate Systems rely on a projection to flatten the surface of an ellipsoid (see [[GeodeticEllipsoid]]) which is
 *  the mathematical model of the Earth surface.
 *  Explanations in more details of the various concepts can be obtained from other sources including the page on the subject
 *  on itwinJS.org (see https://www.itwinjs.org/learning/geolocation/?term=coordinate+system).
 *  A few details are still required to grasp the model. Geographic Coordinate Reference Systems rely on the concept of geodetic datums
 *  (see [[GeodeticDatum]]) to convert latitude/longitude from one frame of reference to another. Such geodetic datum will bind the ellipsoid
 *  and possibly define transformation steps required to convert from the currently used geodetic datum to the common datum WGS84
 *  used for worldwide data (such as most popular imagery data sources). If there are no rules established to convert to WGS84
 *  or if those rules are secret then the horizontal CRS can be datum-less and must make direct use of the ellipsoid
 *  to define the earth surface.
 *  For this purpose either the geodetic datum must be specified using either datumId or datum properties or both. If
 *  both these values are undefined then it is possible to define the ellipsoid using properties ellipsoidId or ellipsoid or both.
 *  These two pairs of properties are mutually exclusive with datum related properties having precedence. If a datum or datumId is set then
 *  ellipsoid properties will automatically be undefined. If datumId or datum is defined then attempts to set ellipsoidId or ellipsoid
 *  will fail silently, the values remaining undefined. The ellipsoidId will not be a repeat of the ellipsoidId property part of the
 *  geodetic datum definition.
 *  @public
 */
export declare class HorizontalCRS implements HorizontalCRSProps {
    /** The identifier of the horizontal CRS as stored in the dictionary or the service database */
    readonly id?: string;
    /** Used only for user-defined definitions that will typically use a GUID as id. A display name for the CRS that allows
     ** a human to understand the nature of the definition
     */
    readonly name?: string;
    /** Description */
    readonly description?: string;
    /** The source of the CRS definition. */
    readonly source?: string;
    /** If true then indicates the definition is deprecated. It should then be used for backward compatibility only.
     *  If false then the definition is not deprecated. Default is false.
     */
    readonly deprecated: boolean;
    /** The EPSG code of the CRS. If undefined then there is no EPSG code associated. */
    readonly epsg?: number;
    /** The identifier of the geodetic datum as stored in the dictionary or the service database. */
    readonly datumId?: string;
    /** The complete definition of the geodetic datum referred to by datumId. It can also be used if the datum is not stored
     *  in either service or dictionary.
     */
    readonly datum?: GeodeticDatum;
    /** The identifier of the geodetic ellipsoid as stored in the dictionary or the service database. This property is exclusive
     *  of having datumId and datum properties undefined.
    */
    readonly ellipsoidId?: string;
    /** The complete definition of the geodetic ellipsoid referred to by ellipsoidId. It can also be used if the ellipsoid is not stored
     *  in either service or dictionary. This property is exclusive
     *  of having datumId and datum properties undefined.
    */
    readonly ellipsoid?: GeodeticEllipsoid;
    /** The text indicating the unit used. */
    readonly unit?: UnitType;
    /** Projection including projection parameters. */
    readonly projection?: Projection;
    /** Extent representing the domain of application of the CRS. */
    readonly extent?: HorizontalCRSExtent;
    constructor(_data?: HorizontalCRSProps);
    /** Creates an Horizontal CRS from JSON representation.
     * @public */
    static fromJSON(data: HorizontalCRSProps): HorizontalCRS;
    /** Creates a JSON from the Horizontal CRS definition
     * @public */
    toJSON(): HorizontalCRSProps;
    /** Compares two horizontal CRS. It is not an equivalence test as descriptive properties are also compared
     * but number compares are applied a minuscule tolerance.
     *  @public */
    equals(other: HorizontalCRS): boolean;
}
/** Vertical Geographic Coordinate reference System definition
 * @public
 */
export interface VerticalCRSProps {
    /** Vertical CRS Key name. */
    id: "GEOID" | "ELLIPSOID" | "NGVD29" | "NAVD88";
}
/** Vertical Coordinate reference System implementation.
 *  The VerticalCRS contains currently a single identifier property of string type. Although
 *  we currently only support four distinct key values "GEOID", "ELLIPSOID", "NAVD88" and "NGVD29"
 *  we expect to support a broader set in the future including, eventually, user defined vertical CRS
 *  which will require additional parameters to be added.
 *  @public
*/
export declare class VerticalCRS implements VerticalCRSProps {
    /** Vertical CRS Key name. The only supported values are currently "GEOID", "ELLIPSOID", "NAVD88" and "NGVD29". The default is ELLIPSOID */
    readonly id: "GEOID" | "ELLIPSOID" | "NGVD29" | "NAVD88";
    constructor(data?: VerticalCRSProps);
    /** Creates a Vertical CRS from JSON representation.
     * @public */
    static fromJSON(data: VerticalCRSProps): VerticalCRS;
    /** Creates a JSON from the Vertical CRS definition
     * @public */
    toJSON(): VerticalCRSProps;
    /** Compares two vertical CRS.
     *  @public */
    equals(other: VerticalCRS): boolean;
}
/** Geographic Coordinate Reference System definition that includes both the horizontal and vertical definitions
 *  @public
 */
export interface GeographicCRSProps {
    /** The horizontal portion of the geographic coordinate reference system. */
    horizontalCRS?: HorizontalCRSProps;
    /** The vertical portion of the geographic coordinate reference system. */
    verticalCRS?: VerticalCRSProps;
    /** The optional additional transform the geographic coordinate reference system. */
    additionalTransform?: AdditionalTransformProps;
}
/** Geographic Coordinate Reference System implementation. This is the class that indicates the definition of a Geographic
 *  coordinate reference system comprised of two components: Horizontal and Vertical.
 *  The vertical component (see [[VerticalCRS]]) is the simplest being formed of a simple identifier as a string.
 *  The horizontal component contains a list of identification and documentation properties as well as
 *  defining details possibly including the projection with method and parameters, the definition of the datum, ellipsoid, extent and so on.
 *  The principle of describing a Geographic CRS is that the definition may be incomplete. The whole set of classes related to geographic
 *  coordinate reference system classes ([[GeodeticEllipsoid]], [[GeodeticDatum]], [[Projection]], [[GeodeticTransform]], ...) are designed
 *  so that they can be parsed from incomplete JSON fragments, or produce incomplete JSON fragments such as would be
 *  generated from a request to a REST API to a server when select OData clauses are used.
 *  Often GeographicCRS would knowingly be created incomplete but with sufficient information to perform conversion from some
 *  reprojection engine (the present set of classes do not provide any GeographicCRS conversion).
 *  For example the following definitions are quite sufficient to request conversion to or from by a reprojection engine:
 *  { horizontalCRS: {id: "LL84"}, verticalCRS: {id:"GEOID"}
 *  or
 *  { horizontalCRS: {datumId: "WGS84", projection: {method: "None"}}, verticalCRS: "ELLIPSOID"}
 *  The reprojection engine will use the engine internal dictionary to obtain the details if it can.
 *  Some definitions will originate from other sources (a parsed WKT for example) and the reprojection engine will require
 *  all mathematical and operational details to perform any conversion (descriptive information are ignored in the conversion process).
 *  @note In the absence of the verticalCRS property then ELLIPSOID (Geodetic elevation) will be assumed by reprojection engines.
 *  @note see important detailed explanation in the [[HorizontalCRS]] documentation.
 *  @note Earth Centered, Earth Fixed coordinate system (ECEF) is a full 3D cartesian system that unambiguously
 *        expressed coordinates relative to the Earth Center. Since there is no horizontal portion independent from
 *        the vertical portion this system cannot be represented by a GeographicCRS and remains a separate concept.
 *  @public
*/
export declare class GeographicCRS implements GeographicCRSProps {
    /** The horizontal portion of the geographic coordinate reference system. */
    readonly horizontalCRS?: HorizontalCRS;
    /** The vertical portion of the geographic coordinate reference system. */
    readonly verticalCRS?: VerticalCRS;
    /** The optional additional transform the geographic coordinate reference system. */
    readonly additionalTransform?: AdditionalTransform;
    constructor(data?: GeographicCRSProps);
    /** Creates a Geographic CRS from JSON representation.
     * @public */
    static fromJSON(data: GeographicCRSProps): GeographicCRS;
    /** Creates a JSON from the Geographic CRS definition
     * @public */
    toJSON(): GeographicCRSProps;
    /** Compares two Geographic CRS. It is a strict compare operation not an equivalence test though
     * number compares are applied a minuscule tolerance.
     * It takes into account descriptive properties not only mathematical definition properties.
     * @public
     */
    equals(other: GeographicCRS): boolean;
}
//# sourceMappingURL=CoordinateReferenceSystem.d.ts.map