"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Geometry
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrustumPlanes = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const geometry_core_1 = require("@bentley/geometry-core");
/** Represents a frustum as 6 planes and provides containment and intersection testing
 * @internal
 */
class FrustumPlanes {
    constructor(frustum) {
        if (undefined !== frustum) {
            this.init(frustum);
        }
    }
    get isValid() { return undefined !== this._planes; }
    // Order: right, left, top, bottom, back, front
    get planes() { return this._planes; }
    init(frustum) {
        if (undefined === this._planes) {
            this._planes = [];
        }
        else {
            this._planes.length = 0;
        }
        FrustumPlanes.addPlaneFromPoints(this._planes, frustum.points, 1, 3, 5); // right
        FrustumPlanes.addPlaneFromPoints(this._planes, frustum.points, 0, 4, 2); // left
        FrustumPlanes.addPlaneFromPoints(this._planes, frustum.points, 2, 6, 3); // top
        FrustumPlanes.addPlaneFromPoints(this._planes, frustum.points, 0, 1, 4); // bottom
        FrustumPlanes.addPlaneFromPoints(this._planes, frustum.points, 0, 2, 1); // back
        FrustumPlanes.addPlaneFromPoints(this._planes, frustum.points, 4, 5, 6); // front
    }
    computeFrustumContainment(box, sphere) { return this.computeContainment(box.points, sphere); }
    intersectsFrustum(box) { return FrustumPlanes.Containment.Outside !== this.computeFrustumContainment(box); }
    containsPoint(point, tolerance = 1.0e-8) { return FrustumPlanes.Containment.Outside !== this.computeContainment([point], undefined, tolerance); }
    computeContainment(points, sphere, tolerance = 1.0e-8) {
        bentleyjs_core_1.assert(this.isValid);
        if (undefined === this._planes) {
            return FrustumPlanes.Containment.Outside;
        }
        let allInside = true;
        for (const plane of this._planes) {
            if (sphere) { // if sphere provide detect total inside and outside without using corners.
                const centerDistance = plane.altitude(sphere.center);
                const tolerancePlusRadius = tolerance + sphere.radius;
                if (centerDistance < -tolerancePlusRadius)
                    return FrustumPlanes.Containment.Outside;
                if (centerDistance > tolerancePlusRadius)
                    continue;
            }
            let nOutside = 0;
            for (const point of points) {
                if (plane.altitude(point) + tolerance < 0.0) {
                    ++nOutside;
                    allInside = false;
                }
            }
            if (nOutside === points.length) {
                return FrustumPlanes.Containment.Outside;
            }
        }
        return allInside ? FrustumPlanes.Containment.Inside : FrustumPlanes.Containment.Partial;
    }
    intersectsRay(origin, direction) {
        bentleyjs_core_1.assert(this.isValid);
        if (undefined === this._planes) {
            return false;
        }
        let tFar = 1e37;
        let tNear = -tFar;
        for (const plane of this._planes) {
            const vD = plane.velocity(direction);
            const vN = plane.altitude(origin);
            if (0.0 === vD) {
                // ray is parallel... no need to continue testing if outside halfspace.
                if (vN < 0.0) {
                    return false;
                }
            }
            else {
                const rayDistance = -vN / vD;
                if (vD < 0.0) {
                    tFar = Math.min(rayDistance, tFar);
                }
                else {
                    tNear = Math.max(rayDistance, tNear);
                }
            }
        }
        return tNear <= tFar;
    }
}
exports.FrustumPlanes = FrustumPlanes;
/** @internal */
(function (FrustumPlanes) {
    /** @internal */
    let Containment;
    (function (Containment) {
        Containment[Containment["Outside"] = 0] = "Outside";
        Containment[Containment["Partial"] = 1] = "Partial";
        Containment[Containment["Inside"] = 2] = "Inside";
    })(Containment = FrustumPlanes.Containment || (FrustumPlanes.Containment = {}));
    /** @internal */
    function addPlaneFromPoints(planes, points, i0, i1, i2, expandPlaneDistance = 1.0e-6) {
        const normal = geometry_core_1.Vector3d.createCrossProductToPoints(points[i2], points[i1], points[i0]);
        normal.normalizeInPlace();
        const plane = geometry_core_1.ClipPlane.createNormalAndDistance(normal, normal.dotProduct(points[i0]) - expandPlaneDistance);
        if (undefined !== plane) {
            planes.push(plane);
        }
    }
    FrustumPlanes.addPlaneFromPoints = addPlaneFromPoints;
})(FrustumPlanes = exports.FrustumPlanes || (exports.FrustumPlanes = {}));
//# sourceMappingURL=FrustumPlanes.js.map