"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module IpcSocket
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IpcWebSocketBackend = exports.IpcWebSocketFrontend = exports.IpcWebSocket = exports.IpcWebSocketMessageType = exports.IpcWebSocketTransport = void 0;
/** @internal */
class IpcWebSocketTransport {
}
exports.IpcWebSocketTransport = IpcWebSocketTransport;
/** @internal */
var IpcWebSocketMessageType;
(function (IpcWebSocketMessageType) {
    IpcWebSocketMessageType[IpcWebSocketMessageType["Send"] = 0] = "Send";
    IpcWebSocketMessageType[IpcWebSocketMessageType["Push"] = 1] = "Push";
    IpcWebSocketMessageType[IpcWebSocketMessageType["Invoke"] = 2] = "Invoke";
    IpcWebSocketMessageType[IpcWebSocketMessageType["Response"] = 3] = "Response";
})(IpcWebSocketMessageType = exports.IpcWebSocketMessageType || (exports.IpcWebSocketMessageType = {}));
/** @internal */
class IpcWebSocket {
    constructor() {
        this._channels = new Map();
        IpcWebSocket.receivers.add(async (e, m) => this.broadcast(e, m));
    }
    addListener(channel, listener) {
        let listeners = this._channels.get(channel);
        if (!listeners) {
            listeners = new Set();
            this._channels.set(channel, listeners);
        }
        if (!listeners.has(listener))
            listeners.add(listener);
        return () => listeners.delete(listener);
    }
    removeListener(channel, listener) {
        var _a;
        (_a = this._channels.get(channel)) === null || _a === void 0 ? void 0 : _a.delete(listener);
    }
    async broadcast(evt, message) {
        if (message.type !== IpcWebSocketMessageType.Send && message.type !== IpcWebSocketMessageType.Push)
            return;
        const handlers = this._channels.get(message.channel);
        if (!handlers)
            return;
        let arg = message.data;
        if (typeof (arg) === "undefined")
            arg = [];
        for (const handler of handlers)
            handler(evt, ...arg);
    }
}
exports.IpcWebSocket = IpcWebSocket;
IpcWebSocket.receivers = new Set();
/** @internal */
class IpcWebSocketFrontend extends IpcWebSocket {
    constructor() {
        super();
        this._nextRequest = 0;
        this._pendingRequests = new Map();
        IpcWebSocket.receivers.add(async (e, m) => this.dispatch(e, m));
    }
    send(channel, ...data) {
        IpcWebSocket.transport.send({ type: IpcWebSocketMessageType.Send, channel, data });
    }
    async invoke(channel, methodName, ...args) {
        const requestId = ++this._nextRequest;
        IpcWebSocket.transport.send({ type: IpcWebSocketMessageType.Invoke, channel, method: methodName, data: args, request: requestId });
        return new Promise((resolve) => {
            this._pendingRequests.set(requestId, resolve);
        });
    }
    async dispatch(_evt, message) {
        if (message.type !== IpcWebSocketMessageType.Response || !message.response)
            return;
        const pendingHandler = this._pendingRequests.get(message.response);
        if (!pendingHandler)
            return;
        this._pendingRequests.delete(message.response);
        pendingHandler(message.data);
    }
}
exports.IpcWebSocketFrontend = IpcWebSocketFrontend;
/** @internal */
class IpcWebSocketBackend extends IpcWebSocket {
    constructor() {
        super();
        this._handlers = new Map();
        IpcWebSocket.receivers.add(async (e, m) => this.dispatch(e, m));
    }
    send(channel, ...data) {
        IpcWebSocket.transport.send({ type: IpcWebSocketMessageType.Push, channel, data });
    }
    handle(channel, handler) {
        this._handlers.set(channel, handler);
        return () => {
            if (this._handlers.get(channel) === handler)
                this._handlers.delete(channel);
        };
    }
    async dispatch(_evt, message) {
        if (message.type !== IpcWebSocketMessageType.Invoke || !message.method)
            return;
        const handler = this._handlers.get(message.channel);
        if (!handler)
            return;
        let args = message.data;
        if (typeof (args) === "undefined")
            args = [];
        const response = await handler({}, message.method, ...args);
        IpcWebSocket.transport.send({
            type: IpcWebSocketMessageType.Response,
            channel: message.channel,
            response: message.request,
            data: response,
        });
    }
}
exports.IpcWebSocketBackend = IpcWebSocketBackend;
//# sourceMappingURL=IpcWebSocket.js.map