/** @packageDocumentation
 * @module RpcInterface
 */
import { GuidString, Id64String } from "@bentley/bentleyjs-core";
import { Range3dProps } from "@bentley/geometry-core";
import { ElementLoadOptions, ElementProps } from "../ElementProps";
import { CodeProps } from "../Code";
import { EntityQueryParams } from "../EntityProps";
import { GeoCoordinatesResponseProps, IModelCoordinatesResponseProps } from "../GeoCoordinateServices";
import { GeometrySummaryRequestProps } from "../GeometrySummary";
import { IModelConnectionProps, IModelRpcOpenProps, IModelRpcProps } from "../IModel";
import { MassPropertiesRequestProps, MassPropertiesResponseProps } from "../MassProperties";
import { ModelProps } from "../ModelProps";
import { QueryLimit, QueryPriority, QueryQuota, QueryResponse } from "../Paging";
import { RpcInterface } from "../RpcInterface";
import { SnapRequestProps, SnapResponseProps } from "../Snapping";
import { ViewStateLoadProps, ViewStateProps } from "../ViewProps";
import { RpcNotFoundResponse } from "./core/RpcControl";
import { GeometryContainmentRequestProps, GeometryContainmentResponseProps } from "../GeometryContainment";
import { RpcRoutingToken } from "./core/RpcRoutingToken";
import { TextureLoadProps } from "../TextureProps";
/** Response if the IModelDb was not found at the backend
 * (if the service has moved)
 * @public
 */
export declare class IModelNotFoundResponse extends RpcNotFoundResponse {
    isIModelNotFoundResponse: boolean;
}
/** The RPC interface for reading from an iModel.
 * All operations only require read-only access.
 * This interface is not normally used directly. See IModelConnection for higher-level and more convenient API for accessing iModels from a frontend.
 * @internal
 */
export declare abstract class IModelReadRpcInterface extends RpcInterface {
    /** Returns the IModelReadRpcInterface instance for the frontend. */
    static getClient(): IModelReadRpcInterface;
    /** Returns the IModelReadRpcInterface instance for a custom RPC routing configuration. */
    static getClientForRouting(token: RpcRoutingToken): IModelReadRpcInterface;
    /** The immutable name of the interface. */
    static readonly interfaceName = "IModelReadRpcInterface";
    /** The semantic version of the interface. */
    static interfaceVersion: string;
    openForRead(_iModelToken: IModelRpcOpenProps): Promise<IModelConnectionProps>;
    close(_iModelToken: IModelRpcProps): Promise<boolean>;
    queryRows(_iModelToken: IModelRpcProps, _ecsql: string, _bindings?: any[] | object, _limit?: QueryLimit, _quota?: QueryQuota, _priority?: QueryPriority, _restartToken?: string, _abbreviateBlobs?: boolean): Promise<QueryResponse>;
    getModelProps(_iModelToken: IModelRpcProps, _modelIds: Id64String[]): Promise<ModelProps[]>;
    queryModelRanges(_iModelToken: IModelRpcProps, _modelIds: Id64String[]): Promise<Range3dProps[]>;
    queryModelProps(_iModelToken: IModelRpcProps, _params: EntityQueryParams): Promise<ModelProps[]>;
    getElementProps(_iModelToken: IModelRpcProps, _elementIds: Id64String[]): Promise<ElementProps[]>;
    queryElementProps(_iModelToken: IModelRpcProps, _params: EntityQueryParams): Promise<ElementProps[]>;
    queryEntityIds(_iModelToken: IModelRpcProps, _params: EntityQueryParams): Promise<Id64String[]>;
    getClassHierarchy(_iModelToken: IModelRpcProps, _startClassName: string): Promise<string[]>;
    getAllCodeSpecs(_iModelToken: IModelRpcProps): Promise<any[]>;
    getViewStateData(_iModelToken: IModelRpcProps, _viewDefinitionId: string, _options?: ViewStateLoadProps): Promise<ViewStateProps>;
    readFontJson(_iModelToken: IModelRpcProps): Promise<any>;
    getToolTipMessage(_iModelToken: IModelRpcProps, _elementId: string): Promise<string[]>;
    getViewThumbnail(_iModelToken: IModelRpcProps, _viewId: string): Promise<Uint8Array>;
    getDefaultViewId(_iModelToken: IModelRpcProps): Promise<Id64String>;
    requestSnap(_iModelToken: IModelRpcProps, _sessionId: string, _props: SnapRequestProps): Promise<SnapResponseProps>;
    cancelSnap(_iModelToken: IModelRpcProps, _sessionId: string): Promise<void>;
    getGeometryContainment(_iModelToken: IModelRpcProps, _props: GeometryContainmentRequestProps): Promise<GeometryContainmentResponseProps>;
    getMassProperties(_iModelToken: IModelRpcProps, _props: MassPropertiesRequestProps): Promise<MassPropertiesResponseProps>;
    getIModelCoordinatesFromGeoCoordinates(_iModelToken: IModelRpcProps, _props: string): Promise<IModelCoordinatesResponseProps>;
    getGeoCoordinatesFromIModelCoordinates(_iModelToken: IModelRpcProps, _props: string): Promise<GeoCoordinatesResponseProps>;
    getGeometrySummary(_iModelToken: IModelRpcProps, _props: GeometrySummaryRequestProps): Promise<string>;
    getTextureImage(_iModelToken: IModelRpcProps, _textureLoadProps: TextureLoadProps): Promise<Uint8Array | undefined>;
    loadElementProps(_iModelToken: IModelRpcProps, _elementIdentifier: Id64String | GuidString | CodeProps, _options?: ElementLoadOptions): Promise<ElementProps | undefined>;
}
//# sourceMappingURL=IModelReadRpcInterface.d.ts.map