"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RpcInterface
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcControlChannel = exports.RpcNotFoundResponse = exports.RpcPendingResponse = exports.RpcControlResponse = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const IModelError_1 = require("../../IModelError");
const RpcInterface_1 = require("../../RpcInterface");
const RpcManager_1 = require("../../RpcManager");
const RpcConfiguration_1 = require("./RpcConfiguration");
const RpcOperation_1 = require("./RpcOperation");
const RpcRegistry_1 = require("./RpcRegistry");
/** An RPC operation control response.
 * @public
 */
class RpcControlResponse {
}
exports.RpcControlResponse = RpcControlResponse;
/** A pending RPC operation response.
 * @public
 */
class RpcPendingResponse extends RpcControlResponse {
    /** Constructs a pending response. */
    constructor(message = "") {
        super();
        this.message = message;
    }
}
exports.RpcPendingResponse = RpcPendingResponse;
/** A RPC operation response.
 * @public
 */
class RpcNotFoundResponse extends RpcControlResponse {
}
exports.RpcNotFoundResponse = RpcNotFoundResponse;
/** Manages requests and responses for an RPC configuration.
 * @internal
 */
class RpcControlChannel {
    constructor(configuration) {
        var _a;
        this._initialized = false;
        this._clientActive = false;
        this._describeEndpoints = undefined;
        this._channelInterface = (_a = class extends RpcInterface_1.RpcInterface {
                async describeEndpoints() { return this.forward(arguments); }
            },
            _a.interfaceVersion = "CONTROL",
            _a.interfaceName = "",
            _a);
        this._channelImpl = class extends RpcInterface_1.RpcInterface {
            async describeEndpoints() {
                const endpoints = [];
                this.configuration.interfaces().forEach((definition) => {
                    if (!RpcRegistry_1.RpcRegistry.instance.isRpcInterfaceInitialized(definition))
                        return;
                    const description = { interfaceName: definition.interfaceName, interfaceVersion: definition.interfaceVersion, operationNames: [], compatible: true };
                    RpcOperation_1.RpcOperation.forEach(definition, (operation) => description.operationNames.push(operation.operationName));
                    endpoints.push(description);
                });
                return endpoints;
            }
        };
        this._configuration = configuration;
        RpcControlChannel.channels.push(this);
    }
    /** @internal */
    async describeEndpoints() {
        this.activateClient();
        return this._describeEndpoints();
    }
    /** @internal */
    static obtain(configuration) {
        if (RpcControlChannel._obtainLock)
            return undefined;
        ++RpcControlChannel._obtainLock;
        const channel = new RpcControlChannel(configuration);
        --RpcControlChannel._obtainLock;
        return channel;
    }
    computeId() {
        const interfaces = [];
        this._configuration.interfaces().forEach((definition) => interfaces.push(`${definition.interfaceName}@${definition.interfaceVersion}`));
        const id = interfaces.sort().join(",");
        if (typeof (btoa) !== "undefined")
            return btoa(id);
        else if (typeof (Buffer) !== "undefined")
            return Buffer.from(id, "binary").toString("base64");
        else
            return id;
    }
    activateClient() {
        if (this._clientActive)
            return;
        if (!this._initialized) {
            if (this._configuration.interfaces().length)
                throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, `Invalid state.`);
            this.initialize();
        }
        this._clientActive = true;
        const token = { key: "none", contextId: "none", iModelId: "none", changeSetId: "none" };
        RpcOperation_1.RpcOperation.forEach(this._channelInterface, (operation) => operation.policy.token = (_request) => { var _a; return (_a = RpcOperation_1.RpcOperation.fallbackToken) !== null && _a !== void 0 ? _a : token; });
        const client = RpcManager_1.RpcManager.getClientForInterface(this._channelInterface);
        this._describeEndpoints = async () => client.describeEndpoints();
    }
    /** @internal */
    initialize() {
        if (this._initialized)
            throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, `Already initialized.`);
        this._initialized = true;
        const id = this.computeId();
        Object.defineProperty(this._channelInterface, "interfaceName", { value: id });
        Object.defineProperty(this._channelImpl, "interfaceName", { value: id });
        RpcConfiguration_1.RpcConfiguration.assign(this._channelInterface, () => this._configuration.constructor);
        RpcManager_1.RpcManager.registerImpl(this._channelInterface, this._channelImpl);
        RpcManager_1.RpcManager.initializeInterface(this._channelInterface);
    }
    /** @internal */
    handleUnknownOperation(invocation, _error) {
        const op = invocation.request.operation;
        if (op.interfaceVersion === "CONTROL" && op.operationName === "describeEndpoints") {
            op.interfaceDefinition = this._channelInterface.interfaceName;
            return true;
        }
        return false;
    }
}
exports.RpcControlChannel = RpcControlChannel;
/** @internal */
RpcControlChannel.channels = [];
RpcControlChannel._obtainLock = 0;
//# sourceMappingURL=RpcControl.js.map