"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RpcInterface
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcInvocation = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const CommonLoggerCategory_1 = require("../../CommonLoggerCategory");
const IModelError_1 = require("../../IModelError");
const RpcInterface_1 = require("../../RpcInterface");
const RpcConfiguration_1 = require("./RpcConfiguration");
const RpcConstants_1 = require("./RpcConstants");
const RpcControl_1 = require("./RpcControl");
const RpcMarshaling_1 = require("./RpcMarshaling");
const RpcOperation_1 = require("./RpcOperation");
const RpcProtocol_1 = require("./RpcProtocol");
const RpcRegistry_1 = require("./RpcRegistry");
/** An RPC operation invocation in response to a request.
 * @public
 */
class RpcInvocation {
    /** Constructs an invocation. */
    constructor(protocol, request) {
        this._threw = false;
        this._pending = false;
        this._notFound = false;
        this._noContent = false;
        this._timeIn = 0;
        this._timeOut = 0;
        /** The operation of the request. */
        this.operation = undefined;
        this._timeIn = new Date().getTime();
        this.protocol = protocol;
        this.request = request;
        try {
            try {
                this.operation = RpcOperation_1.RpcOperation.lookup(this.request.operation.interfaceDefinition, this.request.operation.operationName);
                const backend = this.operation.interfaceVersion;
                const frontend = this.request.operation.interfaceVersion;
                if (!RpcInterface_1.RpcInterface.isVersionCompatible(backend, frontend)) {
                    throw new IModelError_1.IModelError(bentleyjs_core_1.RpcInterfaceStatus.IncompatibleVersion, `Backend version ${backend} does not match frontend version ${frontend} for RPC interface ${this.operation.operationName}.`);
                }
            }
            catch (error) {
                if (this.handleUnknownOperation(error)) {
                    this.operation = RpcOperation_1.RpcOperation.lookup(this.request.operation.interfaceDefinition, this.request.operation.operationName);
                }
                else {
                    throw error;
                }
            }
            this.operation.policy.invocationCallback(this);
            this.result = this.resolve();
        }
        catch (error) {
            this.result = this.reject(error);
        }
        this.fulfillment = this.result.then(async (value) => this._threw ? this.fulfillRejected(value) : this.fulfillResolved(value), async (reason) => this.fulfillRejected(reason));
    }
    /** The status for this request. */
    get status() {
        if (this._threw) {
            return RpcConstants_1.RpcRequestStatus.Rejected;
        }
        else {
            if (this._pending)
                return RpcConstants_1.RpcRequestStatus.Pending;
            else if (this._notFound)
                return RpcConstants_1.RpcRequestStatus.NotFound;
            else if (this._noContent)
                return RpcConstants_1.RpcRequestStatus.NoContent;
            else
                return RpcConstants_1.RpcRequestStatus.Resolved;
        }
    }
    /** The elapsed time for this invocation. */
    get elapsed() {
        return this._timeOut - this._timeIn;
    }
    /**
     * The invocation for the current RPC operation.
     * @note The return value of this function is only reliable in an RPC impl class member function where program control was received from the RpcInvocation constructor function.
     */
    static current(rpcImpl) {
        return rpcImpl[RpcRegistry_1.CURRENT_INVOCATION];
    }
    handleUnknownOperation(error) {
        return this.protocol.configuration.controlChannel.handleUnknownOperation(this, error);
    }
    async resolve() {
        try {
            const clientRequestContext = await RpcConfiguration_1.RpcConfiguration.requestContext.deserialize(this.request);
            clientRequestContext.enter();
            this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.RequestReceived, this);
            const parameters = RpcMarshaling_1.RpcMarshaling.deserialize(this.protocol, this.request.parameters);
            this.applyPolicies(parameters);
            const impl = RpcRegistry_1.RpcRegistry.instance.getImplForInterface(this.operation.interfaceDefinition);
            impl[RpcRegistry_1.CURRENT_INVOCATION] = this;
            const op = this.lookupOperationFunction(impl);
            // @typescript-eslint/return-await doesn't agree with awaiting values that *might* be a promise
            // eslint-disable-next-line @typescript-eslint/return-await
            return await op.call(impl, ...parameters);
        }
        catch (error) {
            return this.reject(error);
        }
    }
    applyPolicies(parameters) {
        if (!parameters || !Array.isArray(parameters)) {
            return;
        }
        for (let i = 0; i !== parameters.length; ++i) {
            const parameter = parameters[i];
            const isToken = typeof (parameter) === "object" && parameter !== null && parameter.hasOwnProperty("iModelId") && parameter.hasOwnProperty("contextId");
            if (isToken && this.protocol.checkToken && !this.operation.policy.allowTokenMismatch) {
                const inflated = this.protocol.inflateToken(parameter, this.request);
                parameters[i] = inflated;
                if (!RpcInvocation.compareTokens(parameter, inflated)) {
                    if (RpcConfiguration_1.RpcConfiguration.throwOnTokenMismatch) {
                        throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "IModelRpcProps mismatch detected for this request.");
                    }
                    else {
                        bentleyjs_core_1.Logger.logWarning(CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceBackend, "IModelRpcProps mismatch detected for this request.");
                    }
                }
            }
        }
    }
    static compareTokens(a, b) {
        return a.key === b.key &&
            a.contextId === b.contextId &&
            a.iModelId === b.iModelId &&
            (undefined === a.changeSetId || (a.changeSetId === b.changeSetId)) &&
            a.openMode === b.openMode;
    }
    async reject(error) {
        this._threw = true;
        return error;
    }
    async fulfillResolved(value) {
        this._timeOut = new Date().getTime();
        this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.BackendResponseCreated, this);
        const result = await RpcMarshaling_1.RpcMarshaling.serialize(this.protocol, value);
        return this.fulfill(result, value);
    }
    async fulfillRejected(reason) {
        this._timeOut = new Date().getTime();
        if (!RpcConfiguration_1.RpcConfiguration.developmentMode)
            reason.stack = undefined;
        const result = await RpcMarshaling_1.RpcMarshaling.serialize(this.protocol, reason);
        let isNoContentError = false;
        try {
            isNoContentError = reason.errorNumber === bentleyjs_core_1.IModelStatus.NoContent;
        }
        catch (_a) { }
        if (reason instanceof RpcControl_1.RpcPendingResponse) {
            this._pending = true;
            this._threw = false;
            result.objects = reason.message;
            this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.BackendReportedPending, this);
        }
        else if (this.supportsNoContent() && isNoContentError) {
            this._noContent = true;
            this._threw = false;
            this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.BackendReportedNoContent, this);
        }
        else if (reason instanceof RpcControl_1.RpcNotFoundResponse) {
            this._notFound = true;
            this._threw = false;
            this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.BackendReportedNotFound, this);
        }
        else {
            this._threw = true;
            this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.BackendErrorOccurred, this);
        }
        return this.fulfill(result, reason);
    }
    supportsNoContent() {
        if (!this.request.protocolVersion) {
            return false;
        }
        return RpcProtocol_1.RpcProtocol.protocolVersion >= 1 && this.request.protocolVersion >= 1;
    }
    fulfill(result, rawResult) {
        const fulfillment = {
            result,
            rawResult,
            status: this.protocol.getCode(this.status),
            id: this.request.id,
            interfaceName: (typeof (this.operation) === "undefined") ? "" : this.operation.interfaceDefinition.interfaceName,
        };
        try {
            const impl = RpcRegistry_1.RpcRegistry.instance.getImplForInterface(this.operation.interfaceDefinition);
            if (impl[RpcRegistry_1.CURRENT_INVOCATION] === this) {
                impl[RpcRegistry_1.CURRENT_INVOCATION] = undefined;
            }
        }
        catch (_err) { }
        return fulfillment;
    }
    lookupOperationFunction(implementation) {
        const func = implementation[this.operation.operationName];
        if (!func || typeof (func) !== "function") {
            throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, `RPC interface class "${implementation.constructor.name}" does not implement operation "${this.operation.operationName}".`, bentleyjs_core_1.Logger.logError, CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceBackend);
        }
        return func;
    }
}
exports.RpcInvocation = RpcInvocation;
//# sourceMappingURL=RpcInvocation.js.map