"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------------
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RpcInterface
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcPushConnection = exports.RpcPushSubscription = exports.RpcPushChannel = exports.RpcPushService = exports.RpcPushTransport = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const IModelError_1 = require("../../IModelError");
/** Defines a transport for push messages.
 *  @alpha
 */
class RpcPushTransport {
}
exports.RpcPushTransport = RpcPushTransport;
/** Defines a source of push messages.
 *  @alpha
 */
class RpcPushService {
    constructor(name) {
        this.name = name;
    }
}
exports.RpcPushService = RpcPushService;
/** The push service for the dedicated backend (for example, an electron or mobile app). */
RpcPushService.dedicated = new RpcPushService("dedicated");
/** Defines a named stream of push messages.
 *  @alpha
 */
class RpcPushChannel {
    constructor(name, service) {
        this._subscribers = [];
        this._refCount = 1;
        this.name = name;
        this.service = service;
    }
    static setup(transport) {
        transport.onMessage = RpcPushChannel.notifySubscribers.bind(RpcPushChannel);
        RpcPushChannel.enabled = true;
    }
    static notifySubscribers(channelId, messageData) {
        const channel = this._channels.get(channelId);
        if (!channel)
            return;
        for (const subscriber of channel._subscribers)
            subscriber.onMessage.raiseEvent(messageData);
    }
    get id() { return RpcPushChannel.formatId(this.name, this.service); }
    get enabled() { return RpcPushChannel.enabled; }
    subscribe() {
        const subscription = new RpcPushSubscription(this);
        this._subscribers.push(subscription);
        return subscription;
    }
    static formatId(name, service) {
        return `${service.name}-${name}`;
    }
    /** Creates a new RpcPushChannel.
     * @throws IModelError if a channel with the specified name and service already exist.
     */
    static create(name, service = RpcPushService.dedicated) {
        return this.get(name, service, false);
    }
    /** Obtains an RpcPushChannel, creating it if one with the specified name and service does not already exists. */
    static obtain(name, service = RpcPushService.dedicated) {
        return this.get(name, service, true);
    }
    static get(name, service, reuseExisting) {
        const id = this.formatId(name, service);
        let channel = this._channels.get(id);
        if (channel) {
            if (!reuseExisting)
                throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, `Channel "${id}" already exists.`);
            ++channel._refCount;
            return channel;
        }
        channel = new RpcPushChannel(name, service);
        this._channels.set(id, channel);
        return channel;
    }
    dispose() {
        if (this.isDisposed)
            return;
        bentleyjs_core_1.assert(this._refCount > 0);
        if (--this._refCount === 0) {
            RpcPushChannel._channels.delete(this.id);
            this._subscribers.length = 0;
        }
    }
    get isDisposed() {
        return 0 === this._refCount;
    }
}
exports.RpcPushChannel = RpcPushChannel;
/** @internal */
RpcPushChannel.enabled = false;
RpcPushChannel._channels = new Map();
/** Receives push messages from the backend.
 *  @alpha
 */
class RpcPushSubscription {
    /** @internal */
    constructor(channel) {
        this.onMessage = new bentleyjs_core_1.BeEvent();
        this.channel = channel;
    }
}
exports.RpcPushSubscription = RpcPushSubscription;
/** Sends push messages to the frontend.
 *  @alpha
 */
class RpcPushConnection {
    constructor(channel, client) {
        this.channel = channel;
        this.client = client;
    }
    static for(_channel, _client = undefined) {
        throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Not implemented.");
    }
}
exports.RpcPushConnection = RpcPushConnection;
//# sourceMappingURL=RpcPush.js.map