"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RpcInterface
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeRpcRequest = exports.RpcRequest = exports.ResponseLike = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const CommonLoggerCategory_1 = require("../../CommonLoggerCategory");
const IModelError_1 = require("../../IModelError");
const RpcConfiguration_1 = require("./RpcConfiguration");
const RpcConstants_1 = require("./RpcConstants");
const RpcMarshaling_1 = require("./RpcMarshaling");
const RpcOperation_1 = require("./RpcOperation");
const RpcProtocol_1 = require("./RpcProtocol");
const RpcRegistry_1 = require("./RpcRegistry");
/* eslint-disable @typescript-eslint/naming-convention */
// cspell:ignore csrf
const aggregateLoad = { lastRequest: 0, lastResponse: 0 };
/** @public */
class ResponseLike {
    constructor(data) {
        this._data = Promise.resolve(data);
    }
    get body() { return null; }
    async arrayBuffer() { return this._data; }
    async blob() { throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Not implemented."); }
    async formData() { throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Not implemented."); }
    async json() { return this._data; }
    async text() { return this._data; }
    get bodyUsed() { return false; }
    get headers() { throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Not implemented."); }
    get ok() { return this.status >= 200 && this.status <= 299; }
    get redirected() { return false; }
    get status() { return 200; }
    get statusText() { return ""; }
    get trailer() { throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Not implemented."); }
    get type() { return "basic"; }
    get url() { return ""; }
    clone() { return Object.assign({}, this); }
}
exports.ResponseLike = ResponseLike;
class Cancellable {
    constructor(task) {
        this.promise = new Promise((resolve, reject) => {
            // eslint-disable-next-line @typescript-eslint/unbound-method
            this.cancel = () => resolve(undefined);
            task.then(resolve, reject);
        });
    }
    cancel() { }
}
/** A RPC operation request.
 * @public
 */
class RpcRequest {
    /** Constructs an RPC request. */
    constructor(client, operation, parameters) {
        this._resolve = () => undefined;
        this._resolveRaw = () => undefined;
        this._reject = () => undefined;
        this._rejectRaw = () => undefined;
        this._created = 0;
        this._lastSubmitted = 0;
        this._lastUpdated = 0;
        this._status = RpcConstants_1.RpcRequestStatus.Unknown;
        this._extendedStatus = "";
        this._connecting = false;
        this._active = true;
        this._hasRawListener = false;
        this._raw = undefined;
        this._attempts = 0;
        this._retryAfter = null;
        this._transientFaults = 0;
        this._response = undefined;
        this._created = new Date().getTime();
        this.path = "";
        this.method = "";
        this.client = client;
        this.protocol = client.configuration.protocol;
        this.operation = RpcOperation_1.RpcOperation.lookup(client.constructor, operation);
        this.parameters = parameters;
        this.retryInterval = this.operation.policy.retryInterval(client.configuration);
        this.response = new Promise((resolve, reject) => { this._resolve = resolve; this._reject = reject; });
        this._rawPromise = new Promise((resolve, reject) => { this._resolveRaw = resolve; this._rejectRaw = reject; });
        this.id = RpcConfiguration_1.RpcConfiguration.requestContext.getId(this) || bentleyjs_core_1.Guid.createValue();
        this.setStatus(RpcConstants_1.RpcRequestStatus.Created);
        this.operation.policy.requestCallback(this);
    }
    /** All RPC requests that are currently in flight. */
    static get activeRequests() { return this._activeRequests; }
    /** The aggregate operations profile of all active RPC interfaces. */
    static get aggregateLoad() { return aggregateLoad; }
    /**
     * The request for the current RPC operation.
     * @note The return value of this function is only reliable if program control was received from a RPC interface class member function that directly returns the result of calling RpcInterface.forward.
     */
    static current(context) {
        return context[RpcRegistry_1.CURRENT_REQUEST];
    }
    /** The status of this request. */
    get status() { return this._status; }
    /** Extended status information for this request (if available). */
    get extendedStatus() { return this._extendedStatus; }
    /** The last submission for this request. */
    get lastSubmitted() { return this._lastSubmitted; }
    /** The last status update received for this request. */
    get lastUpdated() { return this._lastUpdated; }
    /** Whether a connection is active for this request. */
    get connecting() { return this._connecting; }
    /** Whether this request is pending. */
    get pending() {
        switch (this.status) {
            case RpcConstants_1.RpcRequestStatus.Submitted:
            case RpcConstants_1.RpcRequestStatus.Pending: {
                return true;
            }
            default: {
                return false;
            }
        }
    }
    /** The elapsed time for this request. */
    get elapsed() {
        return this._lastUpdated - this._created;
    }
    /** An attempt-specific value for when to next retry this request. */
    get retryAfter() { return this._retryAfter; }
    /** Finds the first parameter of a given structural type if present. */
    findParameterOfType(requiredProperties) {
        for (const param of this.parameters) {
            if (typeof (param) === "object" && param !== null) {
                for (const prop of Object.getOwnPropertyNames(requiredProperties)) {
                    if (param.hasOwnProperty(prop) && typeof (param[prop]) === requiredProperties[prop]) {
                        return param;
                    }
                }
            }
        }
        return undefined;
    }
    /** Finds the first IModelRpcProps parameter if present. */
    findTokenPropsParameter() {
        return this.findParameterOfType({ iModelId: "string" });
    }
    /** The raw implementation response for this request. */
    get rawResponse() {
        this._hasRawListener = true;
        return this._rawPromise;
    }
    /** Sets the last updated time for the request. */
    setLastUpdatedTime() {
        this._lastUpdated = new Date().getTime();
    }
    /** Override to describe available headers based on a protocol-specific criteria (such as a CORS whitelist). */
    isHeaderAvailable(_name) {
        return true;
    }
    computeRetryAfter(attempts) {
        return (((Math.pow(2, attempts) - 1) / 2) * 500) + 500;
    }
    recordTransientFault() {
        ++this._transientFaults;
    }
    resetTransientFaultCount() {
        this._transientFaults = 0;
    }
    /* @internal */
    cancel() {
        if (typeof (this._sending) === "undefined") {
            return;
        }
        this._sending.cancel();
        this._sending = undefined;
        this._connecting = false;
        RpcRequest._activeRequests.delete(this.id);
        this.setStatus(RpcConstants_1.RpcRequestStatus.Cancelled);
    }
    /* @internal */
    async submit() {
        if (!this._active)
            return;
        this._lastSubmitted = new Date().getTime();
        this._retryAfter = null;
        ++this._attempts;
        if (this.status === RpcConstants_1.RpcRequestStatus.Created || this.status === RpcConstants_1.RpcRequestStatus.NotFound || this.status === RpcConstants_1.RpcRequestStatus.Cancelled) {
            this.setStatus(RpcConstants_1.RpcRequestStatus.Submitted);
        }
        try {
            this._connecting = true;
            RpcRequest._activeRequests.set(this.id, this);
            this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.RequestCreated, this);
            this._sending = new Cancellable(this.setHeaders().then(async () => this.send()));
            this.operation.policy.sentCallback(this);
            const response = await this._sending.promise;
            if (typeof (response) === "undefined") {
                return;
            }
            this._sending = undefined;
            const status = this.protocol.getStatus(response);
            if (this._hasRawListener && status === RpcConstants_1.RpcRequestStatus.Resolved && typeof (this._response) !== "undefined") {
                this._connecting = false;
                RpcRequest._activeRequests.delete(this.id);
                this.resolveRaw();
            }
            else {
                this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.ResponseLoading, this);
                if (status === RpcConstants_1.RpcRequestStatus.Unknown) {
                    this._connecting = false;
                    RpcRequest._activeRequests.delete(this.id);
                    this.handleUnknownResponse(response);
                    return;
                }
                const value = await this.load();
                this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.ResponseLoaded, this);
                RpcRequest._activeRequests.delete(this.id);
                this._connecting = false;
                this.handleResponse(response, value);
            }
        }
        catch (err) {
            this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.ConnectionErrorReceived, this, err);
            RpcRequest._activeRequests.delete(this.id);
            this._connecting = false;
            this.reject(err);
        }
    }
    handleUnknownResponse(code) {
        this.reject(new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, `Unknown response ${code}.`));
    }
    handleResponse(code, value) {
        const status = this.protocol.getStatus(code);
        if (RpcConstants_1.RpcRequestStatus.isTransientError(status)) {
            return this.handleTransientError(status);
        }
        switch (status) {
            case RpcConstants_1.RpcRequestStatus.Resolved: {
                return this.handleResolved(value);
            }
            case RpcConstants_1.RpcRequestStatus.Rejected: {
                return this.handleRejected(value);
            }
            case RpcConstants_1.RpcRequestStatus.Pending: {
                return this.setPending(status, value.objects);
            }
            case RpcConstants_1.RpcRequestStatus.NotFound: {
                return this.handleNotFound(status, value);
            }
            case RpcConstants_1.RpcRequestStatus.NoContent: {
                return this.handleNoContent();
            }
        }
    }
    handleResolved(value) {
        try {
            this._raw = value.objects;
            const result = RpcMarshaling_1.RpcMarshaling.deserialize(this.protocol, value);
            if (ArrayBuffer.isView(result)) {
                this._raw = result.buffer;
            }
            return this.resolve(result);
        }
        catch (err) {
            return this.reject(err);
        }
    }
    handleRejected(value) {
        this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.BackendErrorReceived, this);
        try {
            const error = RpcMarshaling_1.RpcMarshaling.deserialize(this.protocol, value);
            const hasInfo = error && typeof (error) === "object" && error.hasOwnProperty("name") && error.hasOwnProperty("message");
            const name = hasInfo ? error.name : "";
            const message = hasInfo ? error.message : "";
            const errorNumber = (hasInfo && error.hasOwnProperty("errorNumber")) ? error.errorNumber : bentleyjs_core_1.BentleyStatus.ERROR;
            return this.reject(new IModelError_1.BackendError(errorNumber, name, message, bentleyjs_core_1.Logger.logError, CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceFrontend, () => error));
        }
        catch (err) {
            return this.reject(err);
        }
    }
    handleNoContent() {
        return this.reject(new IModelError_1.NoContentError());
    }
    handleNotFound(status, value) {
        const response = RpcMarshaling_1.RpcMarshaling.deserialize(this.protocol, value);
        this.setStatus(status);
        let resubmitted = false;
        RpcRequest.notFoundHandlers.raiseEvent(this, response, () => {
            if (resubmitted)
                throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, `Already resubmitted using this handler.`);
            resubmitted = true;
            this.submit(); // eslint-disable-line @typescript-eslint/no-floating-promises
        }, (reason) => this.reject(reason));
        return;
    }
    resolve(result) {
        if (!this._active)
            return;
        this._active = false;
        this.setLastUpdatedTime();
        this._resolve(result);
        if (this._hasRawListener) {
            if (typeof (this._raw) === "undefined") {
                throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Cannot access raw response.");
            }
            this._resolveRaw(new ResponseLike(this._raw));
        }
        this.setStatus(RpcConstants_1.RpcRequestStatus.Resolved);
        this.dispose();
    }
    resolveRaw() {
        if (typeof (this._response) === "undefined") {
            throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Cannot access raw response.");
        }
        this._active = false;
        this.setLastUpdatedTime();
        this._resolveRaw(this._response);
        this.setStatus(RpcConstants_1.RpcRequestStatus.Resolved);
        this.dispose();
    }
    reject(reason) {
        if (!this._active)
            return;
        this._active = false;
        this.setLastUpdatedTime();
        this._reject(reason);
        if (this._hasRawListener) {
            this._rejectRaw(reason);
        }
        this.setStatus(RpcConstants_1.RpcRequestStatus.Rejected);
        this.dispose();
    }
    /** @internal */
    dispose() {
        this.setStatus(RpcConstants_1.RpcRequestStatus.Disposed);
        this._raw = undefined;
        this._response = undefined;
        const client = this.client;
        if (client[RpcRegistry_1.CURRENT_REQUEST] === this) {
            client[RpcRegistry_1.CURRENT_REQUEST] = undefined;
        }
    }
    setPending(status, extendedStatus) {
        if (!this._active)
            return;
        this.setLastUpdatedTime();
        this._extendedStatus = extendedStatus;
        this.setStatus(status);
        RpcRequest.events.raiseEvent(RpcConstants_1.RpcRequestEvent.PendingUpdateReceived, this);
    }
    handleTransientError(status) {
        if (!this._active)
            return;
        this.setLastUpdatedTime();
        this._retryAfter = this.computeRetryAfter(this._attempts - 1);
        if (this._transientFaults > this.protocol.configuration.transientFaultLimit) {
            this.reject(new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, `Exceeded transient fault limit.`));
        }
        else {
            this.setStatus(status);
            RpcRequest.events.raiseEvent(RpcConstants_1.RpcRequestEvent.TransientErrorReceived, this);
        }
    }
    async setHeaders() {
        const versionHeader = this.protocol.protocolVersionHeaderName;
        if (versionHeader && RpcProtocol_1.RpcProtocol.protocolVersion && this.isHeaderAvailable(versionHeader)) {
            this.setHeader(versionHeader, RpcProtocol_1.RpcProtocol.protocolVersion.toString());
        }
        const headerNames = this.protocol.serializedClientRequestContextHeaderNames;
        const headerValues = await RpcConfiguration_1.RpcConfiguration.requestContext.serialize(this);
        if (headerNames.id)
            this.setHeader(headerNames.id, headerValues.id || this.id); // Cannot be empty
        if (headerNames.applicationVersion)
            this.setHeader(headerNames.applicationVersion, headerValues.applicationVersion);
        if (headerNames.applicationId)
            this.setHeader(headerNames.applicationId, headerValues.applicationId);
        if (headerNames.sessionId)
            this.setHeader(headerNames.sessionId, headerValues.sessionId);
        if (headerNames.authorization && headerValues.authorization)
            this.setHeader(headerNames.authorization, headerValues.authorization);
        if (headerNames.userId && headerValues.userId)
            this.setHeader(headerNames.userId, headerValues.userId);
        if (headerValues.csrfToken)
            this.setHeader(headerValues.csrfToken.headerName, headerValues.csrfToken.headerValue);
    }
    setStatus(status) {
        if (this._status === status)
            return;
        this._status = status;
        RpcRequest.events.raiseEvent(RpcConstants_1.RpcRequestEvent.StatusChanged, this);
    }
}
exports.RpcRequest = RpcRequest;
RpcRequest._activeRequests = new Map();
/** Events raised by RpcRequest. See [[RpcRequestEvent]] */
RpcRequest.events = new bentleyjs_core_1.BeEvent();
/** Resolvers for "not found" requests. See [[RpcRequestNotFoundHandler]] */
RpcRequest.notFoundHandlers = new bentleyjs_core_1.BeEvent();
/** @internal */
exports.initializeRpcRequest = (() => {
    let initialized = false;
    return () => {
        if (initialized) {
            return;
        }
        initialized = true;
        RpcRequest.events.addListener((type, request) => {
            if (type !== RpcConstants_1.RpcRequestEvent.StatusChanged)
                return;
            switch (request.status) {
                case RpcConstants_1.RpcRequestStatus.Submitted: {
                    aggregateLoad.lastRequest = request.lastSubmitted;
                    break;
                }
                case RpcConstants_1.RpcRequestStatus.Pending:
                case RpcConstants_1.RpcRequestStatus.Resolved:
                case RpcConstants_1.RpcRequestStatus.Rejected: {
                    aggregateLoad.lastResponse = request.lastUpdated;
                    break;
                }
            }
        });
        RpcProtocol_1.RpcProtocol.events.addListener((type) => {
            const now = new Date().getTime();
            switch (type) {
                case RpcConstants_1.RpcProtocolEvent.RequestReceived: {
                    aggregateLoad.lastRequest = now;
                    break;
                }
                case RpcConstants_1.RpcProtocolEvent.BackendReportedPending:
                case RpcConstants_1.RpcProtocolEvent.BackendErrorOccurred:
                case RpcConstants_1.RpcProtocolEvent.BackendResponseCreated: {
                    aggregateLoad.lastResponse = now;
                    break;
                }
            }
        });
    };
})();
//# sourceMappingURL=RpcRequest.js.map