"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RpcInterface
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.BentleyCloudRpcProtocol = void 0;
const url_1 = require("url");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const CommonLoggerCategory_1 = require("../../CommonLoggerCategory");
const IModelError_1 = require("../../IModelError");
const RpcConfiguration_1 = require("../core/RpcConfiguration");
const RpcOperation_1 = require("../core/RpcOperation");
const WebAppRpcProtocol_1 = require("./WebAppRpcProtocol");
var AppMode;
(function (AppMode) {
    AppMode["MilestoneReview"] = "1";
    AppMode["WorkGroupEdit"] = "2";
})(AppMode || (AppMode = {}));
/** An http protocol for Bentley cloud RPC interface deployments.
 * @public
 */
class BentleyCloudRpcProtocol extends WebAppRpcProtocol_1.WebAppRpcProtocol {
    constructor() {
        super(...arguments);
        this.checkToken = true;
        /** The name of various HTTP request headers based on client's request context */
        this.serializedClientRequestContextHeaderNames = {
            /** The name of the HTTP request id header. */
            id: "X-Correlation-Id",
            /** The name of the HTTP application id header. */
            applicationId: "X-Application-Id",
            /** The name of the HTTP application version header. */
            applicationVersion: "X-Application-Version",
            /** The name of the HTTP session id header. */
            sessionId: "X-Session-Id",
            /** The name of the HTTP authorization header. */
            authorization: "Authorization",
            /** The id of the authorized user */
            userId: "X-User-Id",
        };
        /** The name of the RPC protocol version header. */
        this.protocolVersionHeaderName = "X-Protocol-Version";
    }
    /** Returns the operation specified by an OpenAPI-compatible URI path. */
    getOperationFromPath(path) {
        const url = new url_1.URL(path, "https://localhost/");
        const components = url.pathname.split("/");
        const operationComponent = components.slice(-1)[0];
        const encodedRequest = url.searchParams.get("parameters") || "";
        const firstHyphen = operationComponent.indexOf("-");
        const lastHyphen = operationComponent.lastIndexOf("-");
        const interfaceDefinition = operationComponent.slice(0, firstHyphen);
        const interfaceVersion = operationComponent.slice(firstHyphen + 1, lastHyphen);
        const operationName = operationComponent.slice(lastHyphen + 1);
        return { interfaceDefinition, operationName, interfaceVersion, encodedRequest };
    }
    /** Supplies the OpenAPI-compatible URI path for an RPC operation. */
    supplyPathForOperation(operation, request) {
        const prefix = this.pathPrefix;
        const appTitle = this.info.title;
        const appVersion = this.info.version;
        const operationId = `${operation.interfaceDefinition.interfaceName}-${operation.interfaceVersion}-${operation.operationName}`;
        let appMode = "";
        let contextId = "";
        let iModelId = "";
        let routeChangeSetId;
        /* Note: The changeSetId field is omitted in the route in the case of ReadWrite connections since the connection is generally expected to be at the
         * latest version and not some specific changeSet. Also, for the first version (before any changeSets), the changeSetId in the route is arbitrarily
         * set to "0" instead of an empty string, since the latter is more un-intuitive for a route. However, in all other use cases, including the changeSetId
         * held by the IModelRpcProps itself, the changeSetId of "" (i.e., empty string) signifies the first version - this is more intuitive and retains
         * compatibility with the majority of use cases. */
        if (request === undefined) {
            appMode = "{modeId}";
            contextId = "{contextId}";
            iModelId = "{iModelId}";
            routeChangeSetId = "{changeSetId}";
        }
        else {
            let token = operation.policy.token(request) || RpcOperation_1.RpcOperation.fallbackToken;
            if (!token || !token.iModelId) {
                if (RpcConfiguration_1.RpcConfiguration.disableRoutingValidation) {
                    token = { key: "" };
                }
                else {
                    throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Invalid iModelToken for RPC operation request", bentleyjs_core_1.Logger.logError, CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceFrontend);
                }
            }
            contextId = encodeURIComponent(token.contextId || "");
            iModelId = encodeURIComponent(token.iModelId);
            if (token.openMode === bentleyjs_core_1.OpenMode.Readonly) {
                // Use "0" if changeSetId omitted or empty string.
                routeChangeSetId = token.changeSetId || "0";
                appMode = AppMode.MilestoneReview;
            }
            else {
                appMode = AppMode.WorkGroupEdit;
            }
        }
        return `${prefix}/${appTitle}/${appVersion}/mode/${appMode}/context/${contextId}/imodel/${iModelId}${!!routeChangeSetId ? `/changeset/${routeChangeSetId}` : ""}/${operationId}`;
    }
    /**
     * Inflates the IModelRpcProps from the URL path for each request on the backend.
     * @note This function updates the IModelRpcProps value supplied in the request body.
     */
    inflateToken(tokenFromBody, request) {
        const urlPathComponents = request.path.split("/");
        const iModelKey = tokenFromBody.key;
        let openMode = tokenFromBody.openMode;
        let iModelId = tokenFromBody.iModelId;
        let contextId = tokenFromBody.contextId;
        let changeSetId = tokenFromBody.changeSetId;
        for (let i = 0; i <= urlPathComponents.length; ++i) {
            const key = urlPathComponents[i];
            const value = urlPathComponents[i + 1];
            if (key === "mode") {
                openMode = (value === AppMode.WorkGroupEdit) ? bentleyjs_core_1.OpenMode.ReadWrite : bentleyjs_core_1.OpenMode.Readonly;
                ++i;
            }
            else if (key === "context") {
                contextId = value;
                ++i;
            }
            else if (key === "imodel") {
                iModelId = value;
                ++i;
            }
            else if (key === "changeset") {
                changeSetId = (value === "0") ? "" : value;
                ++i;
            }
        }
        return { key: iModelKey, contextId, iModelId, changeSetId, openMode };
    }
    /** Returns the OpenAPI-compatible URI path parameters for an RPC operation.
     * @internal
     */
    supplyPathParametersForOperation(_operation) {
        return [
            { name: "modeId", in: "path", required: true, schema: { type: "string" } },
            { name: "contextId", in: "path", required: true, schema: { type: "string" } },
            { name: "iModelId", in: "path", required: true, schema: { type: "string" } },
            { name: "changeSetId", in: "path", required: false, schema: { type: "string" } },
        ];
    }
}
exports.BentleyCloudRpcProtocol = BentleyCloudRpcProtocol;
//# sourceMappingURL=BentleyCloudRpcProtocol.js.map