"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RpcInterface
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebAppRpcLogging = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const CommonLoggerCategory_1 = require("../../CommonLoggerCategory");
const RpcConstants_1 = require("../core/RpcConstants");
const RpcInvocation_1 = require("../core/RpcInvocation");
const WebAppRpcRequest_1 = require("./WebAppRpcRequest");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const os = (typeof (process) !== "undefined") ? require("os") : undefined;
function getHostname() {
    if (os !== undefined) {
        return os.hostname();
    }
    else {
        if (typeof (window) !== "undefined") {
            return window.location.host;
        }
        else {
            return "imodeljs-mobile";
        }
    }
}
/** @internal */
class WebAppRpcLogging {
    static async logProtocolEvent(event, object) {
        if (object instanceof WebAppRpcRequest_1.WebAppRpcRequest) {
            switch (event) {
                case RpcConstants_1.RpcProtocolEvent.RequestCreated: return WebAppRpcLogging.logRequest(CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceFrontend, "RpcInterface.frontend.request", object);
                case RpcConstants_1.RpcProtocolEvent.ResponseLoaded: return WebAppRpcLogging.logResponse(CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceFrontend, "RpcInterface.frontend.response", object, object.metadata.status, object.elapsed);
                case RpcConstants_1.RpcProtocolEvent.ConnectionErrorReceived: return WebAppRpcLogging.logErrorFrontend("RpcInterface.frontend.connectionError", object);
                case RpcConstants_1.RpcProtocolEvent.ConnectionAborted: return WebAppRpcLogging.logErrorFrontend("RpcInterface.frontend.connectionAborted", object);
            }
        }
        else if (object instanceof RpcInvocation_1.RpcInvocation) {
            switch (event) {
                case RpcConstants_1.RpcProtocolEvent.RequestReceived: return WebAppRpcLogging.logRequest(CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceBackend, "RpcInterface.backend.request", object.request);
                case RpcConstants_1.RpcProtocolEvent.BackendErrorOccurred: return WebAppRpcLogging.logErrorBackend("RpcInterface.backend.error", object);
                case RpcConstants_1.RpcProtocolEvent.BackendResponseCreated: return WebAppRpcLogging.logResponse(CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceBackend, "RpcInterface.backend.response", object.request, object.status, object.elapsed);
            }
        }
    }
    static getRpcInterfaceName(g) {
        return (typeof g === "string") ? g : g.interfaceName;
    }
    static findPathIds(path) {
        let contextId = "";
        let iModelId = "";
        const tokens = path.split("/");
        for (let i = 0; i !== tokens.length; ++i) {
            if ((/^context$/i).test(tokens[i])) {
                contextId = tokens[i + 1] || "";
            }
            if ((/^imodel$/i).test(tokens[i])) {
                iModelId = tokens[i + 1] || "";
            }
        }
        return { contextId, iModelId };
    }
    static buildOperationDescriptor(operation) {
        if (!operation) {
            return "unknown.unknown";
        }
        const interfaceName = typeof (operation.interfaceDefinition) === "string" ? operation.interfaceDefinition : operation.interfaceDefinition.interfaceName;
        const operationName = operation.operationName;
        return `${interfaceName}.${operationName}`;
    }
    static logRequest(loggerCategory, message, object) {
        const operationDescriptor = WebAppRpcLogging.buildOperationDescriptor(object.operation);
        const pathIds = WebAppRpcLogging.findPathIds(object.path);
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `${message}.${operationDescriptor}`, () => (Object.assign({ method: object.method, path: object.path, operation: object.operation.operationName, rpcInterface: WebAppRpcLogging.getRpcInterfaceName(object.operation.interfaceDefinition), 
            // Alert! The following properties are required by Bentley DevOps standards. Do not change their names!
            ActivityId: object.id, TimeElapsed: ("elapsed" in object) ? object.elapsed : 0, MachineName: getHostname() }, pathIds)));
    }
    static logResponse(loggerCategory, message, object, status, elapsed) {
        const operationDescriptor = WebAppRpcLogging.buildOperationDescriptor(object.operation);
        const pathIds = WebAppRpcLogging.findPathIds(object.path);
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `${message}.${operationDescriptor}`, () => (Object.assign({ method: object.method, path: object.path, operation: object.operation.operationName, rpcInterface: WebAppRpcLogging.getRpcInterfaceName(object.operation.interfaceDefinition), status, 
            // Alert! The following properties are required by Bentley DevOps standards. Do not change their names!
            ActivityId: object.id, TimeElapsed: elapsed, MachineName: getHostname() }, pathIds)));
    }
    static logErrorFrontend(message, request) {
        const operationDescriptor = WebAppRpcLogging.buildOperationDescriptor(request.operation);
        const pathIds = WebAppRpcLogging.findPathIds(request.path);
        bentleyjs_core_1.Logger.logInfo(CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceFrontend, `${message}.${operationDescriptor}`, () => (Object.assign({ method: request.method, path: request.path, 
            // Alert! The following properties are required by Bentley DevOps standards. Do not change their names!
            ActivityId: request.id, MachineName: getHostname() }, pathIds)));
    }
    static async logErrorBackend(message, invocation) {
        const operationDescriptor = WebAppRpcLogging.buildOperationDescriptor(invocation.operation);
        const pathIds = WebAppRpcLogging.findPathIds(invocation.request.path);
        const result = await invocation.result;
        const errorMessage = result.message ? result.message : result.objects; // Can be an error or an RpcSerializedValue
        bentleyjs_core_1.Logger.logInfo(CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceBackend, `${message}.${operationDescriptor}`, () => (Object.assign({ method: invocation.request.method, path: invocation.request.path, status: invocation.status, errorMessage, 
            // Alert! The following properties are required by Bentley DevOps standards. Do not change their names!
            ActivityId: invocation.request.id, MachineName: getHostname() }, pathIds)));
    }
}
exports.WebAppRpcLogging = WebAppRpcLogging;
//# sourceMappingURL=WebAppRpcLogging.js.map