"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RpcInterface
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebAppRpcProtocol = void 0;
const RpcConfiguration_1 = require("../core/RpcConfiguration");
const RpcConstants_1 = require("../core/RpcConstants");
const RpcProtocol_1 = require("../core/RpcProtocol");
const OpenAPI_1 = require("./OpenAPI");
const WebAppRpcLogging_1 = require("./WebAppRpcLogging");
const WebAppRpcRequest_1 = require("./WebAppRpcRequest");
const CommonLoggerCategory_1 = require("../../CommonLoggerCategory");
const RpcInterface_1 = require("../../RpcInterface");
const RpcManager_1 = require("../../RpcManager");
const RpcRoutingToken_1 = require("../core/RpcRoutingToken");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
class InitializeInterface extends RpcInterface_1.RpcInterface {
    async initialize() { return this.forward(arguments); }
    static createRequest(protocol) {
        const routing = RpcRoutingToken_1.RpcRoutingToken.generate();
        const config = class extends RpcConfiguration_1.RpcConfiguration {
            constructor() {
                super(...arguments);
                this.interfaces = () => [InitializeInterface];
                this.protocol = protocol;
            }
        };
        RpcConfiguration_1.RpcConfiguration.assignWithRouting(InitializeInterface, routing, config);
        const instance = RpcConfiguration_1.RpcConfiguration.obtain(config);
        RpcConfiguration_1.RpcConfiguration.initializeInterfaces(instance);
        const client = RpcManager_1.RpcManager.getClientForInterface(InitializeInterface, routing);
        return new (protocol.requestType)(client, "initialize", []);
    }
}
InitializeInterface.interfaceName = "InitializeInterface";
InitializeInterface.interfaceVersion = "1.0.0";
/** The HTTP application protocol.
 * @public
 */
class WebAppRpcProtocol extends RpcProtocol_1.RpcProtocol {
    /** Constructs an HTTP protocol. */
    constructor(configuration) {
        super(configuration);
        this.preserveStreams = true;
        /** @internal */
        this.allowedHeaders = new Set();
        /** An optional prefix for RPC operation URI paths. */
        this.pathPrefix = "";
        /** The RPC request class for this protocol. */
        this.requestType = WebAppRpcRequest_1.WebAppRpcRequest;
        this.events.addListener(WebAppRpcLogging_1.WebAppRpcLogging.logProtocolEvent);
    }
    /** @internal */
    async initialize() {
        if (this._initialized) {
            return this._initialized;
        }
        return this._initialized = new Promise(async (resolve) => {
            try {
                const request = InitializeInterface.createRequest(this);
                const response = await request.preflight();
                if (response && response.ok) {
                    (response.headers.get("Access-Control-Allow-Headers") || "").split(",").forEach((v) => this.allowedHeaders.add(v.trim()));
                }
            }
            catch (err) {
                bentleyjs_core_1.Logger.logWarning(CommonLoggerCategory_1.CommonLoggerCategory.RpcInterfaceFrontend, "Unable to discover backend capabilities.", () => err);
            }
            resolve();
        });
    }
    /** Convenience handler for an RPC operation get request for an HTTP server. */
    async handleOperationGetRequest(req, res) {
        return this.handleOperationPostRequest(req, res);
    }
    /** Convenience handler for an RPC operation post request for an HTTP server. */
    async handleOperationPostRequest(req, res) {
        const request = await WebAppRpcRequest_1.WebAppRpcRequest.parseRequest(this, req);
        const fulfillment = await this.fulfill(request);
        WebAppRpcRequest_1.WebAppRpcRequest.sendResponse(this, request, fulfillment, res);
    }
    /** Convenience handler for an OpenAPI description request for an HTTP server. */
    handleOpenApiDescriptionRequest(_req, res) {
        const description = JSON.stringify(this.openAPIDescription);
        res.send(description);
    }
    /** Converts an HTTP content type value to an RPC content type value. */
    static computeContentType(httpType) {
        if (!httpType)
            return RpcConstants_1.RpcContentType.Unknown;
        if (httpType.indexOf(RpcConstants_1.WEB_RPC_CONSTANTS.ANY_TEXT) === 0) {
            return RpcConstants_1.RpcContentType.Text;
        }
        else if (httpType.indexOf(RpcConstants_1.WEB_RPC_CONSTANTS.BINARY) === 0) {
            return RpcConstants_1.RpcContentType.Binary;
        }
        else if (httpType.indexOf(RpcConstants_1.WEB_RPC_CONSTANTS.MULTIPART) === 0) {
            return RpcConstants_1.RpcContentType.Multipart;
        }
        else {
            return RpcConstants_1.RpcContentType.Unknown;
        }
    }
    /** Supplies the status corresponding to a protocol-specific code value. */
    getStatus(code) {
        switch (code) {
            case 404: return RpcConstants_1.RpcRequestStatus.NotFound;
            case 202: return RpcConstants_1.RpcRequestStatus.Pending;
            case 200: return RpcConstants_1.RpcRequestStatus.Resolved;
            case 500: return RpcConstants_1.RpcRequestStatus.Rejected;
            case 204: return RpcConstants_1.RpcRequestStatus.NoContent;
            case 502: return RpcConstants_1.RpcRequestStatus.BadGateway;
            case 503: return RpcConstants_1.RpcRequestStatus.ServiceUnavailable;
            case 504: return RpcConstants_1.RpcRequestStatus.GatewayTimeout;
            default: return RpcConstants_1.RpcRequestStatus.Unknown;
        }
    }
    /** Supplies the protocol-specific code corresponding to a status value. */
    getCode(status) {
        switch (status) {
            case RpcConstants_1.RpcRequestStatus.NotFound: return 404;
            case RpcConstants_1.RpcRequestStatus.Pending: return 202;
            case RpcConstants_1.RpcRequestStatus.Resolved: return 200;
            case RpcConstants_1.RpcRequestStatus.Rejected: return 500;
            case RpcConstants_1.RpcRequestStatus.NoContent: return 204;
            case RpcConstants_1.RpcRequestStatus.BadGateway: return 502;
            case RpcConstants_1.RpcRequestStatus.ServiceUnavailable: return 503;
            case RpcConstants_1.RpcRequestStatus.GatewayTimeout: return 504;
            default: return 501;
        }
    }
    /** Whether an HTTP status code indicates a request timeout. */
    isTimeout(code) {
        return code === 504;
    }
    /** An OpenAPI-compatible description of this protocol.
     * @internal
     */
    get openAPIDescription() { return new OpenAPI_1.RpcOpenAPIDescription(this); }
}
exports.WebAppRpcProtocol = WebAppRpcProtocol;
//# sourceMappingURL=WebAppRpcProtocol.js.map