"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module RpcInterface
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebAppRpcRequest = void 0;
require("@ungap/url-search-params/index");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const IModelError_1 = require("../../IModelError");
const RpcConstants_1 = require("../core/RpcConstants");
const RpcMarshaling_1 = require("../core/RpcMarshaling");
const RpcRequest_1 = require("../core/RpcRequest");
const RpcMultipartParser_1 = require("./multipart/RpcMultipartParser");
const RpcMultipart_1 = require("./RpcMultipart");
const WebAppRpcProtocol_1 = require("./WebAppRpcProtocol");
/** A web application RPC request.
 * @public
 */
class WebAppRpcRequest extends RpcRequest_1.RpcRequest {
    /** Constructs a web application request. */
    constructor(client, operation, parameters) {
        super(client, operation, parameters);
        this._loading = false;
        this._request = {};
        this._pathSuffix = "";
        /** Convenience access to the protocol of this request. */
        this.protocol = this.client.configuration.protocol;
        /** Standardized access to metadata about the request (useful for purposes such as logging). */
        this.metadata = { status: 0, message: "" };
        this.path = this.protocol.supplyPathForOperation(this.operation, this);
        this.method = "head";
        this._request.headers = {};
    }
    get _headers() { return this._request.headers; }
    /** Parse headers */
    static parseHeaders(protocol, req) {
        const headerNames = protocol.serializedClientRequestContextHeaderNames;
        const parsedHeaders = {
            id: req.header(headerNames.id) || "",
            applicationId: req.header(headerNames.applicationId) || "",
            applicationVersion: req.header(headerNames.applicationVersion) || "",
            sessionId: req.header(headerNames.sessionId) || "",
            authorization: headerNames.authorization ? req.header(headerNames.authorization) : undefined,
            userId: headerNames.userId ? req.header(headerNames.userId) : undefined,
        };
        return parsedHeaders;
    }
    /** Parses a request. */
    static async parseRequest(protocol, req) {
        const operation = protocol.getOperationFromPath(req.url);
        const parsedHeaders = this.parseHeaders(protocol, req);
        const request = Object.assign(Object.assign({}, parsedHeaders), { operation: {
                interfaceDefinition: operation.interfaceDefinition,
                operationName: operation.operationName,
                interfaceVersion: operation.interfaceVersion,
            }, method: req.method, path: req.url, parameters: operation.encodedRequest ? WebAppRpcRequest.parseFromPath(operation) : await WebAppRpcRequest.parseFromBody(req), caching: operation.encodedRequest ? RpcConstants_1.RpcResponseCacheControl.Immutable : RpcConstants_1.RpcResponseCacheControl.None });
        request.ip = req.ip;
        request.protocolVersion = 0;
        if (protocol.protocolVersionHeaderName) {
            const version = req.header(protocol.protocolVersionHeaderName);
            if (version) {
                request.protocolVersion = parseInt(version, 10);
            }
        }
        if (!request.id) {
            throw new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, `Invalid request.`);
        }
        return request;
    }
    /** Sends the response for a web request. */
    static sendResponse(protocol, request, fulfillment, res) {
        const transportType = WebAppRpcRequest.computeTransportType(fulfillment.result, fulfillment.rawResult);
        if (transportType === RpcConstants_1.RpcContentType.Binary) {
            WebAppRpcRequest.sendBinary(protocol, request, fulfillment, res);
        }
        else if (transportType === RpcConstants_1.RpcContentType.Multipart) {
            WebAppRpcRequest.sendMultipart(protocol, request, fulfillment, res);
        }
        else if (transportType === RpcConstants_1.RpcContentType.Stream) {
            WebAppRpcRequest.sendStream(protocol, request, fulfillment, res);
        }
        else {
            WebAppRpcRequest.sendText(protocol, request, fulfillment, res);
        }
    }
    /** Determines the most efficient transport type for an RPC value. */
    static computeTransportType(value, source) {
        if (source instanceof Uint8Array || (Array.isArray(source) && source[0] instanceof Uint8Array)) {
            return RpcConstants_1.RpcContentType.Binary;
        }
        else if (value.data.length > 0) {
            return RpcConstants_1.RpcContentType.Multipart;
        }
        else if (value.stream) {
            return RpcConstants_1.RpcContentType.Stream;
        }
        else {
            return RpcConstants_1.RpcContentType.Text;
        }
    }
    /** @internal */
    async preflight() {
        this.method = "options";
        this._request.method = "options";
        await this.setHeaders();
        await this.send();
        return this._response;
    }
    isHeaderAvailable(name) {
        const allowed = this.protocol.allowedHeaders;
        return allowed.has("*") || allowed.has(name);
    }
    /** Sets request header values. */
    setHeader(name, value) {
        this._headers[name] = value;
    }
    /** Sends the request. */
    async send() {
        if (this.method !== "options") {
            await this.protocol.initialize();
        }
        this._loading = true;
        await this.setupTransport();
        return new Promise(async (resolve, reject) => {
            try {
                resolve(await this.performFetch());
            }
            catch (reason) {
                reject(new IModelError_1.ServerError(-1, reason || "Server connection error."));
            }
        });
    }
    computeRetryAfter(attempts) {
        const retryAfter = this._response && this._response.headers.get("Retry-After");
        if (retryAfter) {
            this.resetTransientFaultCount();
            const r = Number(retryAfter);
            if (Number.isFinite(r)) {
                return r * 1000;
            }
            const d = Date.parse(retryAfter);
            if (!Number.isNaN(d)) {
                return d - Date.now();
            }
        }
        else {
            this.recordTransientFault();
        }
        return super.computeRetryAfter(attempts);
    }
    handleUnknownResponse(code) {
        if (this.protocol.isTimeout(code)) {
            this.reject(new IModelError_1.ServerTimeoutError("Request timeout."));
        }
        else {
            this.reject(new IModelError_1.ServerError(code, "Unknown server response code."));
        }
    }
    async load() {
        return new Promise(async (resolve, reject) => {
            try {
                if (!this._loading)
                    return;
                const response = this._response;
                if (!response) {
                    reject(new IModelError_1.IModelError(bentleyjs_core_1.BentleyStatus.ERROR, "Invalid state."));
                    return;
                }
                const contentType = response.headers.get(RpcConstants_1.WEB_RPC_CONSTANTS.CONTENT);
                const responseType = WebAppRpcProtocol_1.WebAppRpcProtocol.computeContentType(contentType);
                if (responseType === RpcConstants_1.RpcContentType.Binary) {
                    resolve(await this.loadBinary(response));
                }
                else if (responseType === RpcConstants_1.RpcContentType.Multipart) {
                    resolve(await this.loadMultipart(response, contentType));
                }
                else {
                    resolve(await this.loadText(response));
                }
                this._loading = false;
                this.setLastUpdatedTime();
                this.protocol.events.raiseEvent(RpcConstants_1.RpcProtocolEvent.ResponseLoaded, this);
            }
            catch (reason) {
                if (!this._loading)
                    return;
                this._loading = false;
                reject(new IModelError_1.ServerError(this.metadata.status, reason || "Unknown server response error."));
            }
        });
    }
    /** Override to supply an alternate fetch function. */
    supplyFetch() {
        return fetch;
    }
    /** Override to supply an alternate Request function. */
    supplyRequest() {
        return Request;
    }
    static configureResponse(protocol, request, fulfillment, res) {
        const success = protocol.getStatus(fulfillment.status) === RpcConstants_1.RpcRequestStatus.Resolved;
        if (success && request.caching === RpcConstants_1.RpcResponseCacheControl.Immutable) {
            res.set("Cache-Control", "private, max-age=31536000, immutable");
        }
        if (fulfillment.retry) {
            res.set("Retry-After", fulfillment.retry);
        }
    }
    static sendText(protocol, request, fulfillment, res) {
        const response = (fulfillment.status === 204) ? "" : fulfillment.result.objects;
        res.set(RpcConstants_1.WEB_RPC_CONSTANTS.CONTENT, RpcConstants_1.WEB_RPC_CONSTANTS.TEXT);
        WebAppRpcRequest.configureResponse(protocol, request, fulfillment, res);
        res.status(fulfillment.status).send(response);
    }
    static sendBinary(protocol, request, fulfillment, res) {
        const data = fulfillment.result.data[0];
        const response = Buffer.isBuffer(data) ? data : Buffer.from(data);
        res.set(RpcConstants_1.WEB_RPC_CONSTANTS.CONTENT, RpcConstants_1.WEB_RPC_CONSTANTS.BINARY);
        WebAppRpcRequest.configureResponse(protocol, request, fulfillment, res);
        res.status(fulfillment.status).send(response);
    }
    static sendMultipart(protocol, request, fulfillment, res) {
        const response = RpcMultipart_1.RpcMultipart.createStream(fulfillment.result);
        const headers = response.getHeaders();
        for (const header in headers) {
            if (headers.hasOwnProperty(header)) {
                res.set(header, headers[header]);
            }
        }
        WebAppRpcRequest.configureResponse(protocol, request, fulfillment, res);
        res.status(fulfillment.status);
        response.pipe(res);
    }
    static sendStream(protocol, request, fulfillment, res) {
        const response = fulfillment.result.stream;
        WebAppRpcRequest.configureResponse(protocol, request, fulfillment, res);
        res.status(fulfillment.status);
        response.pipe(res);
    }
    static parseFromPath(operation) {
        const decoded = operation.encodedRequest ? Buffer.from(operation.encodedRequest, "base64").toString("binary") : "";
        return RpcMarshaling_1.RpcSerializedValue.create(decoded);
    }
    static async parseFromBody(req) {
        const contentType = WebAppRpcProtocol_1.WebAppRpcProtocol.computeContentType(req.header(RpcConstants_1.WEB_RPC_CONSTANTS.CONTENT));
        if (contentType === RpcConstants_1.RpcContentType.Binary) {
            const objects = JSON.stringify([RpcMarshaling_1.MarshalingBinaryMarker.createDefault()]);
            const data = [req.body];
            return RpcMarshaling_1.RpcSerializedValue.create(objects, data);
        }
        else if (contentType === RpcConstants_1.RpcContentType.Multipart) {
            return RpcMultipart_1.RpcMultipart.parseRequest(req);
        }
        else {
            return RpcMarshaling_1.RpcSerializedValue.create(req.body);
        }
    }
    async performFetch() {
        const requestClass = this.supplyRequest();
        const fetchFunction = this.supplyFetch();
        const path = new URL(this.path, typeof (location) !== "undefined" ? location.origin : undefined);
        if (this._pathSuffix) {
            const params = new URLSearchParams();
            params.set("parameters", this._pathSuffix);
            path.search = `?${params.toString()}`;
        }
        const request = new requestClass(path.toString(), this._request);
        const response = await fetchFunction(request);
        this._response = response;
        this.metadata.status = response.status;
        return response.status;
    }
    async loadText(response) {
        const value = await response.text();
        this.metadata.message = value;
        return RpcMarshaling_1.RpcSerializedValue.create(value);
    }
    async loadBinary(response) {
        const value = new Uint8Array(await response.arrayBuffer());
        const objects = JSON.stringify(RpcMarshaling_1.MarshalingBinaryMarker.createDefault());
        return RpcMarshaling_1.RpcSerializedValue.create(objects, [value]);
    }
    async loadMultipart(response, contentType) {
        const data = await response.arrayBuffer();
        const value = new RpcMultipartParser_1.RpcMultipartParser(contentType, Buffer.from(data)).parse();
        return value;
    }
    async setupTransport() {
        if (this.method === "options") {
            return;
        }
        const parameters = (await this.protocol.serialize(this)).parameters;
        const transportType = WebAppRpcRequest.computeTransportType(parameters, this.parameters);
        if (transportType === RpcConstants_1.RpcContentType.Binary) {
            this.setupBinaryTransport(parameters);
        }
        else if (transportType === RpcConstants_1.RpcContentType.Multipart) {
            this.setupMultipartTransport(parameters);
        }
        else {
            this.setupTextTransport(parameters);
        }
    }
    setupBinaryTransport(parameters) {
        this._headers[RpcConstants_1.WEB_RPC_CONSTANTS.CONTENT] = RpcConstants_1.WEB_RPC_CONSTANTS.BINARY;
        this._request.method = "post";
        this._request.body = parameters.data[0];
    }
    setupMultipartTransport(parameters) {
        // IMPORTANT: do not set a multipart Content-Type header value. The browser does this automatically!
        delete this._headers[RpcConstants_1.WEB_RPC_CONSTANTS.CONTENT];
        this._request.method = "post";
        this._request.body = RpcMultipart_1.RpcMultipart.createForm(parameters);
    }
    setupTextTransport(parameters) {
        if (this.operation.policy.allowResponseCaching(this)) {
            const encodedBody = btoa(parameters.objects);
            if (encodedBody.length <= WebAppRpcRequest.maxUrlComponentSize) {
                this._request.method = "get";
                this._request.body = undefined;
                delete this._headers[RpcConstants_1.WEB_RPC_CONSTANTS.CONTENT];
                this._pathSuffix = encodedBody;
                return;
            }
        }
        this._pathSuffix = "";
        this._headers[RpcConstants_1.WEB_RPC_CONSTANTS.CONTENT] = RpcConstants_1.WEB_RPC_CONSTANTS.TEXT;
        this._request.method = "post";
        this._request.body = parameters.objects;
    }
}
exports.WebAppRpcRequest = WebAppRpcRequest;
/** The maximum size permitted for an encoded component in a URL.
 * @note This is used for features like encoding the payload of a cacheable request in the URL.
 */
WebAppRpcRequest.maxUrlComponentSize = 1024;
//# sourceMappingURL=WebAppRpcRequest.js.map