/** @packageDocumentation
 * @module Tile
 */
import { ByteStream } from "@bentley/bentleyjs-core";
import { TileHeader } from "./TileIO";
/** Known version of the [glTF format](https://www.khronos.org/gltf/).
 * @internal
 */
export declare enum GltfVersions {
    Version1 = 1,
    Version2 = 2,
    CurrentVersion = 1,
    Gltf1SceneFormat = 0
}
/** @internal */
export declare enum GltfV2ChunkTypes {
    JSON = 1313821514,
    Binary = 5130562
}
/** @internal */
export declare enum GltfMeshMode {
    Points = 0,
    Lines = 1,
    LineStrip = 3,
    Triangles = 4
}
/** @internal */
export declare enum GltfDataType {
    SignedByte = 5120,
    UnsignedByte = 5121,
    SignedShort = 5122,
    UnsignedShort = 5123,
    UInt32 = 5125,
    Float = 5126,
    Rgb = 6407,
    Rgba = 6408,
    IntVec2 = 35667,
    IntVec3 = 35668,
    FloatVec2 = 35664,
    FloatVec3 = 35665,
    FloatVec4 = 35666,
    FloatMat3 = 35675,
    FloatMat4 = 35676,
    Sampler2d = 35678
}
/** @internal */
export declare enum GltfConstants {
    CullFace = 2884,
    DepthTest = 2929,
    Nearest = 9728,
    Linear = 9729,
    LinearMipmapLinear = 9987,
    ClampToEdge = 33071,
    ArrayBuffer = 34962,
    ElementArrayBuffer = 34963,
    FragmentShader = 35632,
    VertexShader = 35633
}
/** @internal */
export declare type GltfDataBuffer = Uint8Array | Uint16Array | Uint32Array | Float32Array;
/**
 * A chunk of binary data exposed as a typed array.
 * The count member indicates how many elements exist. This may be less than this.buffer.length due to padding added to the
 * binary stream to ensure correct alignment.
 * @internal
 */
export declare class GltfBufferData {
    readonly buffer: GltfDataBuffer;
    readonly count: number;
    constructor(buffer: GltfDataBuffer, count: number);
    /**
     * Create a GltfBufferData of the desired type. The actual type may differ from the desired type - for example, small 32-bit integers
     * may be represented as 8-bit or 16-bit integers instead.
     * If the actual data type is not convertible to the desired type, this function returns undefined.
     */
    static create(bytes: Uint8Array, actualType: GltfDataType, expectedType: GltfDataType, count: number): GltfBufferData | undefined;
    private static createDataBuffer;
}
/**
 * A view of a chunk of glTF binary data containing an array of elements of a specific data type.
 * The count member indicates how many elements exist; this may be smaller than this.data.length.
 * The count member may also indicate the number of elements of a type containing more than one value of the
 * underlying type. For example, a buffer of 4 32-bit floating point 'vec2' elements will have a count of 4,
 * but its data member will contain 8 32-bit floating point values (2 per vec2).
 * The accessor member may contain additional JSON data specific to a particular buffer.
 * @internal
 */
export declare class GltfBufferView {
    readonly data: Uint8Array;
    readonly count: number;
    readonly type: GltfDataType;
    readonly accessor: any;
    readonly stride: number;
    get byteLength(): number;
    constructor(data: Uint8Array, count: number, type: GltfDataType, accessor: any, stride: number);
    toBufferData(desiredType: GltfDataType): GltfBufferData | undefined;
}
/** Header preceding glTF data.
 * @internal
 */
export declare class GltfHeader extends TileHeader {
    readonly gltfLength: number;
    readonly scenePosition: number;
    readonly sceneStrLength: number;
    readonly binaryPosition: number;
    get isValid(): boolean;
    constructor(stream: ByteStream);
}
//# sourceMappingURL=GltfTileIO.d.ts.map