"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tile
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.GltfHeader = exports.GltfBufferView = exports.GltfBufferData = exports.GltfConstants = exports.GltfDataType = exports.GltfMeshMode = exports.GltfV2ChunkTypes = exports.GltfVersions = void 0;
const TileIO_1 = require("./TileIO");
/** Known version of the [glTF format](https://www.khronos.org/gltf/).
 * @internal
 */
var GltfVersions;
(function (GltfVersions) {
    GltfVersions[GltfVersions["Version1"] = 1] = "Version1";
    GltfVersions[GltfVersions["Version2"] = 2] = "Version2";
    GltfVersions[GltfVersions["CurrentVersion"] = 1] = "CurrentVersion";
    GltfVersions[GltfVersions["Gltf1SceneFormat"] = 0] = "Gltf1SceneFormat";
})(GltfVersions = exports.GltfVersions || (exports.GltfVersions = {}));
/** @internal */
var GltfV2ChunkTypes;
(function (GltfV2ChunkTypes) {
    GltfV2ChunkTypes[GltfV2ChunkTypes["JSON"] = 1313821514] = "JSON";
    GltfV2ChunkTypes[GltfV2ChunkTypes["Binary"] = 5130562] = "Binary";
})(GltfV2ChunkTypes = exports.GltfV2ChunkTypes || (exports.GltfV2ChunkTypes = {}));
/** @internal */
var GltfMeshMode;
(function (GltfMeshMode) {
    GltfMeshMode[GltfMeshMode["Points"] = 0] = "Points";
    GltfMeshMode[GltfMeshMode["Lines"] = 1] = "Lines";
    GltfMeshMode[GltfMeshMode["LineStrip"] = 3] = "LineStrip";
    GltfMeshMode[GltfMeshMode["Triangles"] = 4] = "Triangles";
})(GltfMeshMode = exports.GltfMeshMode || (exports.GltfMeshMode = {}));
/** @internal */
var GltfDataType;
(function (GltfDataType) {
    GltfDataType[GltfDataType["SignedByte"] = 5120] = "SignedByte";
    GltfDataType[GltfDataType["UnsignedByte"] = 5121] = "UnsignedByte";
    GltfDataType[GltfDataType["SignedShort"] = 5122] = "SignedShort";
    GltfDataType[GltfDataType["UnsignedShort"] = 5123] = "UnsignedShort";
    GltfDataType[GltfDataType["UInt32"] = 5125] = "UInt32";
    GltfDataType[GltfDataType["Float"] = 5126] = "Float";
    GltfDataType[GltfDataType["Rgb"] = 6407] = "Rgb";
    GltfDataType[GltfDataType["Rgba"] = 6408] = "Rgba";
    GltfDataType[GltfDataType["IntVec2"] = 35667] = "IntVec2";
    GltfDataType[GltfDataType["IntVec3"] = 35668] = "IntVec3";
    GltfDataType[GltfDataType["FloatVec2"] = 35664] = "FloatVec2";
    GltfDataType[GltfDataType["FloatVec3"] = 35665] = "FloatVec3";
    GltfDataType[GltfDataType["FloatVec4"] = 35666] = "FloatVec4";
    GltfDataType[GltfDataType["FloatMat3"] = 35675] = "FloatMat3";
    GltfDataType[GltfDataType["FloatMat4"] = 35676] = "FloatMat4";
    GltfDataType[GltfDataType["Sampler2d"] = 35678] = "Sampler2d";
})(GltfDataType = exports.GltfDataType || (exports.GltfDataType = {}));
/** @internal */
var GltfConstants;
(function (GltfConstants) {
    GltfConstants[GltfConstants["CullFace"] = 2884] = "CullFace";
    GltfConstants[GltfConstants["DepthTest"] = 2929] = "DepthTest";
    GltfConstants[GltfConstants["Nearest"] = 9728] = "Nearest";
    GltfConstants[GltfConstants["Linear"] = 9729] = "Linear";
    GltfConstants[GltfConstants["LinearMipmapLinear"] = 9987] = "LinearMipmapLinear";
    GltfConstants[GltfConstants["ClampToEdge"] = 33071] = "ClampToEdge";
    GltfConstants[GltfConstants["ArrayBuffer"] = 34962] = "ArrayBuffer";
    GltfConstants[GltfConstants["ElementArrayBuffer"] = 34963] = "ElementArrayBuffer";
    GltfConstants[GltfConstants["FragmentShader"] = 35632] = "FragmentShader";
    GltfConstants[GltfConstants["VertexShader"] = 35633] = "VertexShader";
})(GltfConstants = exports.GltfConstants || (exports.GltfConstants = {}));
/**
 * A chunk of binary data exposed as a typed array.
 * The count member indicates how many elements exist. This may be less than this.buffer.length due to padding added to the
 * binary stream to ensure correct alignment.
 * @internal
 */
class GltfBufferData {
    constructor(buffer, count) {
        this.buffer = buffer;
        this.count = count;
    }
    /**
     * Create a GltfBufferData of the desired type. The actual type may differ from the desired type - for example, small 32-bit integers
     * may be represented as 8-bit or 16-bit integers instead.
     * If the actual data type is not convertible to the desired type, this function returns undefined.
     */
    static create(bytes, actualType, expectedType, count) {
        if (expectedType !== actualType) {
            // Some data is stored in smaller data types to save space if no values exceed the maximum of the smaller type.
            switch (expectedType) {
                case GltfDataType.Float:
                case GltfDataType.UnsignedByte:
                    return undefined;
                case GltfDataType.UnsignedShort:
                    if (GltfDataType.UnsignedByte !== actualType)
                        return undefined;
                    break;
                case GltfDataType.UInt32:
                    if (GltfDataType.UnsignedByte !== actualType && GltfDataType.UnsignedShort !== actualType)
                        return undefined;
                    break;
            }
        }
        const data = this.createDataBuffer(bytes, actualType);
        return undefined !== data ? new GltfBufferData(data, count) : undefined;
    }
    static createDataBuffer(bytes, actualType) {
        // NB: Endianness of typed array data is determined by the 'platform byte order'. Actual data is always little-endian.
        // We are assuming little-endian platform. If we find a big-endian platform, we'll need to use a DataView instead.
        switch (actualType) {
            case GltfDataType.UnsignedByte:
                return bytes;
            case GltfDataType.UnsignedShort:
                return new Uint16Array(bytes.buffer, bytes.byteOffset, bytes.byteLength / 2);
            case GltfDataType.UInt32:
                return new Uint32Array(bytes.buffer, bytes.byteOffset, bytes.byteLength / 4);
            case GltfDataType.Float:
                return new Float32Array(bytes.buffer, bytes.byteOffset, bytes.byteLength / 4);
            default:
                return undefined;
        }
    }
}
exports.GltfBufferData = GltfBufferData;
/**
 * A view of a chunk of glTF binary data containing an array of elements of a specific data type.
 * The count member indicates how many elements exist; this may be smaller than this.data.length.
 * The count member may also indicate the number of elements of a type containing more than one value of the
 * underlying type. For example, a buffer of 4 32-bit floating point 'vec2' elements will have a count of 4,
 * but its data member will contain 8 32-bit floating point values (2 per vec2).
 * The accessor member may contain additional JSON data specific to a particular buffer.
 * @internal
 */
class GltfBufferView {
    constructor(data, count, type, accessor, stride) {
        this.data = data;
        this.count = count;
        this.type = type;
        this.accessor = accessor;
        this.stride = stride;
    }
    get byteLength() { return this.data.length; }
    toBufferData(desiredType) {
        return GltfBufferData.create(this.data, this.type, desiredType, this.count);
    }
}
exports.GltfBufferView = GltfBufferView;
/** Header preceding glTF data.
 * @internal
 */
class GltfHeader extends TileIO_1.TileHeader {
    constructor(stream) {
        super(stream);
        this.scenePosition = 0;
        this.sceneStrLength = 0;
        this.binaryPosition = 0;
        this.gltfLength = stream.nextUint32;
        this.sceneStrLength = stream.nextUint32;
        const value5 = stream.nextUint32;
        // Early versions of the reality data tile publisher incorrectly put version 2 into header - handle these old tiles
        // validating the chunk type.
        if (this.version === GltfVersions.Version2 && value5 === GltfVersions.Gltf1SceneFormat)
            this.version = GltfVersions.Version1;
        if (this.version === GltfVersions.Version1) {
            const gltfSceneFormat = value5;
            if (GltfVersions.Gltf1SceneFormat !== gltfSceneFormat) {
                this.invalidate();
                return;
            }
            this.scenePosition = stream.curPos;
            this.binaryPosition = stream.curPos + this.sceneStrLength;
        }
        else if (this.version === GltfVersions.Version2) {
            const sceneChunkType = value5;
            this.scenePosition = stream.curPos;
            stream.curPos = stream.curPos + this.sceneStrLength;
            const binaryLength = stream.nextUint32;
            const binaryChunkType = stream.nextUint32;
            if (GltfV2ChunkTypes.JSON !== sceneChunkType || GltfV2ChunkTypes.Binary !== binaryChunkType || 0 === binaryLength) {
                this.invalidate();
                return;
            }
            this.binaryPosition = stream.curPos;
        }
        else {
            this.invalidate();
        }
    }
    get isValid() { return TileIO_1.TileFormat.Gltf === this.format; }
}
exports.GltfHeader = GltfHeader;
//# sourceMappingURL=GltfTileIO.js.map