"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tile
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.FeatureTableHeader = exports.ImdlHeader = exports.CurrentImdlVersion = exports.ImdlFlags = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const geometry_core_1 = require("@bentley/geometry-core");
const TileIO_1 = require("./TileIO");
/** Flags describing the geometry contained within a tile in iMdl format.
 * @internal
 */
var ImdlFlags;
(function (ImdlFlags) {
    /** No special flags */
    ImdlFlags[ImdlFlags["None"] = 0] = "None";
    /** The tile contains some curved geometry */
    ImdlFlags[ImdlFlags["ContainsCurves"] = 1] = "ContainsCurves";
    /** Some geometry within the tile range was omitted based on its size */
    ImdlFlags[ImdlFlags["Incomplete"] = 4] = "Incomplete";
    /** The tile must be refined by sub-division, not magnification. */
    ImdlFlags[ImdlFlags["DisallowMagnification"] = 8] = "DisallowMagnification";
})(ImdlFlags = exports.ImdlFlags || (exports.ImdlFlags = {}));
/** Describes the maximum major and minor version of the iMdl tile format supported by this version of this package.
 * @internal
 */
var CurrentImdlVersion;
(function (CurrentImdlVersion) {
    /** The unsigned 16-bit major version number. If the major version specified in the tile header is greater than this value, then this
     * front-end is not capable of reading the tile content. Otherwise, this front-end can read the tile content even if the header specifies a
     * greater minor version than CurrentVersion.Minor, although some data may be skipped.
     */
    CurrentImdlVersion[CurrentImdlVersion["Major"] = 25] = "Major";
    /** The unsigned 16-bit minor version number. If the major version in the tile header is equal to CurrentVersion.Major, then this package can
     * read the tile content even if the minor version in the tile header is greater than this value, although some data may be skipped.
     */
    CurrentImdlVersion[CurrentImdlVersion["Minor"] = 0] = "Minor";
    /** The unsigned 32-bit version number derived from the 16-bit major and minor version numbers. */
    CurrentImdlVersion[CurrentImdlVersion["Combined"] = 1638400] = "Combined";
})(CurrentImdlVersion = exports.CurrentImdlVersion || (exports.CurrentImdlVersion = {}));
/** Header embedded at the beginning of binary tile data in iMdl format describing its contents.
 * @internal
 */
class ImdlHeader extends TileIO_1.TileHeader {
    /** Deserialize a header from the binary data at the stream's current position.
     * If the binary data does not contain a valid header, the Header will be marked 'invalid'.
     */
    constructor(stream) {
        super(stream);
        this.headerLength = stream.nextUint32;
        this.flags = stream.nextUint32;
        this.contentRange = new geometry_core_1.Range3d();
        TileIO_1.nextPoint3d64FromByteStream(stream, this.contentRange.low);
        TileIO_1.nextPoint3d64FromByteStream(stream, this.contentRange.high);
        this.tolerance = stream.nextFloat64;
        this.numElementsIncluded = stream.nextUint32;
        this.numElementsExcluded = stream.nextUint32;
        this.tileLength = stream.nextUint32;
        // empty sub-volume bit field introduced in format v02.00
        this.emptySubRanges = this.versionMajor >= 2 ? stream.nextUint32 : 0;
        // Skip any unprocessed bytes in header
        const remainingHeaderBytes = this.headerLength - stream.curPos;
        bentleyjs_core_1.assert(remainingHeaderBytes >= 0);
        stream.advance(remainingHeaderBytes);
        if (stream.isPastTheEnd)
            this.invalidate();
    }
    get versionMajor() { return this.version >>> 0x10; }
    get versionMinor() { return (this.version & 0xffff) >>> 0; }
    get isValid() { return TileIO_1.TileFormat.IModel === this.format; }
    get isReadableVersion() { return this.versionMajor <= CurrentImdlVersion.Major; }
}
exports.ImdlHeader = ImdlHeader;
/** Header preceding the feature table embedded in an iMdl tile's content.
 * @internal
 */
class FeatureTableHeader {
    constructor(length, maxFeatures, count) {
        this.length = length;
        this.maxFeatures = maxFeatures;
        this.count = count;
    }
    static readFrom(stream) {
        const length = stream.nextUint32;
        const maxFeatures = stream.nextUint32;
        const count = stream.nextUint32;
        return stream.isPastTheEnd ? undefined : new FeatureTableHeader(length, maxFeatures, count);
    }
}
exports.FeatureTableHeader = FeatureTableHeader;
FeatureTableHeader.sizeInBytes = 12;
//# sourceMappingURL=IModelTileIO.js.map